	SUBROUTINE ANNULUS
C
C       This subroutine  averages and integrates pixel values within
C  elliptical annuli where the central location is given either by a TV 
C  cursor , from a file, or manually.  Pixel values of 0 are regarded as
C  edited areas and are not counted.  The eccentricity, inclination, and
C  inner and outer radii are input either manually or from a file.  Allowance
C  is made for the center to be off the map.  The output is sent to a file.
C  The virtual memory is increased and decreased as needed.
C			Donald H. Gudehus, August, l982
C			Version 2, Donald H. Gudehus, November, 1988
C			Added code for Trapix
C			Version 3, Donald H. Gudehus, August, 1993
C			Added code for SAOimage
C			Version 3U, Donald H. Gudehus, January 31, 2008
C			Converted to Unix and FITS
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
	INCLUDE 'saofacts.for'
C
	CHARACTER*2 COMMAND
	CHARACTER*3 PRGFMAT,FILEFMT
	CHARACTER*6 IDREAD
	CHARACTER*40 FILEKEYWORD
	CHARACTER*50 CHFITSMHV
	CHARACTER*80 FILENAME_IN
C
	INTEGER*2 IX(5),IY(5)
	INTEGER ADDRESS,LENGTH,IOCHFL
C
	LOGICAL IOFL,MORE,OFIL,MESSAG
C
	REAL*4 RADIUS(MAXDIAG),ADU(MAXDIAG),LOGINTADU(MAXDIAG),
	1POSITION(2)
C  This common block is not in conflict with ANNULUS2 
	COMMON /REAL4_SCRATCH_BLOCK/ RADIUS,ADU,LOGINTADU
C
	INTEGER RADII(2),NCOLS,NROWS
	REAL*4 COLUMN,ROW,INCLINA,ECCENTR,FORESHORT
C  These common blocks are shared with ANNULUS1
	COMMON /INTEGER4_SCRATCH_BLOCK2/ RADII,NCOLS,NROWS,
	1MNCOLUMN,MNROW,MXCOLUMN,MXROW
	COMMON /REAL4_SCRATCH_BLOCK2/ COLUMN,ROW,INCLINA,
	1ECCENTR,FORESHORT
C
	CHARACTER*6 ID
	COMMON /ID/ ID
C
	LOGICAL GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
	COMMON /DISPLAY_AVAILABILITY/ GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
C
	CHARACTER*80 DNAME
	LOGICAL EXTENDED
	COMMON /DFILE/ DNAME,EXTENDED
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	CALL MESSAGE('ANNULUS, Version 3')
	CALL MESSAGE('Open the file to be measured:')
	PI=3.14159265
	LU_IN=11
	FILEKEYWORD='FILE='
	FILENAME_IN=' '
	IOFL=.TRUE.
	MAPKEY=0
	PLANEKEY=0
	NMAPS=0
	NPLNS=0
	NROWS=0
	NCOLS=0
	PRGFMAT=' '
	CALL OPENFITS(LU_IN,IOFL,FILEKEYWORD,FILENAME_IN,MAPKEY,PLANEKEY,NMAPS,
	1NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
C Initialize column and row for the first time through.
        POSITION(1)=(1.0*NCOLS)/2.0
        POSITION(2)=(1.0*NROWS)/2.0
	NOBJECTS=0
	MESSAG=.TRUE.
	COMMAND=' '
	DO WHILE (COMMAND.NE.'S'.AND.COMMAND.NE.'AS'
	1.AND.COMMAND.NE.'F'.AND.COMMAND.NE.'M')
	  CALL MESSAGE('Enter a command:')
	  COMMAND='M'
	  IF (SAOIMAGE) THEN
	    CALL MESSAGE('     AS = Position from cursor;  SAOimage '//
	1   'already loaded')
	    CALL MESSAGE('      S = Position from cursor;  need to '//
	1   'load SAOimage')
	  END IF
	  CALL MESSAGE('      F = Coordinates, eccentricity, and '//
	1 'inclination from file prepared')
	  CALL MESSAGE('          by option CT')
	  CALL MESSAGE('      M = Manual input of coordinates (default)')
	  CALL USRINP(%REF(COMMAND),1,'A',2,1,'COMMAND=',' ')
	  CALL ALPHA(%REF(COMMAND),1,2,NC,NF,NR,2)
	END DO
C
	IF (COMMAND.EQ.'S') THEN
	  NPIC_SAO=1
	  IF (IFB_SAO.EQ.0) THEN
	    CALL SAOSTART
	    CALL SAOFIRST
	  ELSE
	    CALL SAOERAS
	  END IF
	  CALL SAODELS(LU_IN,1,0)
	END IF
  5	CONTINUE !  Return point for more objects
	IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
  25	  CALL MESSAGE('Be sure SAOimage has focus, position the '//
	1 'cursor, and then press <CR>')
	  CALL SAO_CURN(NPICN,IXN,IYN,XN,YN,POSITION(1),POSITION(2),KEY,MESSAG)
	  IF (KEY.EQ.13) THEN
	    IF (NPICN.EQ.0.OR.(NPICN.NE.0.AND.TV_FILENAME(NPICN)
	1   .NE.DNAME)) THEN
	      CALL MESSAGE('Try again')
	      GO TO 25
	    END IF
C  Draw the point on SAOimage
	    CALL SAODOT(INT(POSITION(1)),INT(POSITION(2)),NPICN,207)
	  ELSE
	    CALL BELL
	    CALL MESSAGE('You must press the <CR>')
	    GO TO 25
	  END IF
	  POSITION(1)=INT(POSITION(1))+0.5
	  POSITION(2)=INT(POSITION(2))+0.5
	  WRITE (6,2000) POSITION(1),POSITION(2)
2000	  FORMAT(' ','The cursor is at',F6.1,2X,F6.1)
	  WRITE (7,2000) POSITION(1),POSITION(2)
	END IF
	IF (COMMAND.EQ.'F'.AND.NOBJECTS.EQ.0) THEN
	  CALL OPENCF(21,1,'FILE_21=',0)
	END IF
	IF (COMMAND.EQ.'F') THEN
	  CALL USRINP(%REF(ID),1,'A',6,0,'ID=','Enter the object''s '//
	1 'ID:')
	  CALL ALPHA(%REF(ID),1,6,LX,NF,NR,2)
	  REWIND 21
	  READ(21,1000,ERR=35)
	  GO TO 36
 35	  CALL MESSAGE('Couldn''t read the header')
	  RETURN
 36	  CONTINUE
1000	  FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',2X,'INCLINATION',
	1 1X,'ECCENTRICITY')
	  IDREAD=' '
	  DO WHILE (IDREAD.NE.ID)
	    READ(21,1001,END=38) IDREAD,COLUMN,ROW,INCLINA,ECCENTR
	  END DO
1001	  FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3)
	  GO TO 39
 38	  CALL MESSAGE('Couldn''t find the object')
	  RETURN
 39	  GO TO 45
	END IF
	IF (COMMAND.EQ.'M') THEN
	  CALL USRINP(POSITION,2,'F',4,1,'POSITION=',
	1 'Enter column and row:  (default = last position, '//
	1 'fractional values allowed)')
	END IF
C  Correct for proper reading from the map.  Note that COLUMN and ROW
C  may be negative if the center lies off the map.
        COLUMN=POSITION(1)
        ROW=POSITION(2)
C  Input inclination and eccentricity manually
        INCLINA=0.0
        CALL USRINP(INCLINA,1,'F',4,1,'INCLINATION=',
     1  'Enter the ellipse inclination angle:  (default = 0.0 degrees)')
        ECCENTR=0.
        CALL USRINP(ECCENTR,1,'F',4,1,'ECCENTRICITY=',
     1  'Enter the eccentricity of the ellipse:  (default = 0.0)')
C
C  Get the size of the area
 45	RADII(1)=0
	RADII(2)=100
	CALL USRINP(RADII,2,'I',4,1,'RADII=',
	1'Enter the inner and outer radii:  (default = 0 100)')
	IF (RADII(1).LT.0.OR.RADII(2).LE.RADII(1)) GO TO 45
	MNCOLUMN=JINT(COLUMN)-RADII(2)-1 !  Must have area for interpo-
	MNROW=JINT(ROW)-RADII(2)-1       !  lation 1 pixel larger on
					 !  each side if possible
	IF (MNCOLUMN.LT.1) MNCOLUMN=1
	IF (MNROW.LT.1) MNROW=1
	MXCOLUMN=JINT(COLUMN)+RADII(2)+1
	MXROW=JINT(ROW)+RADII(2)+1
	IF (MXCOLUMN.GT.NCOLS) MXCOLUMN=NCOLS
	IF (MXROW.GT.NROWS) MXROW=NROWS
	NCOL=MXCOLUMN-MNCOLUMN+1
	NROW=MXROW-MNROW+1
	LENGTH=NCOL*NROW
C  Get the sky level to be subtracted before integrating
	SKY=-999.999
	CALL USRINP(SKY,1,'F',4,3,'SKY=',
	1'Enter the sky level to be subtracted before integrating:  '//
	2'(default = minimum)')
C  Prepare the output file
	IF (COMMAND.NE.'F') THEN
	  CALL USRINP(%REF(ID),1,'A',6,1,'ID=','Enter the object''s '//
	1 'ID:  (default = last entry)')
	  CALL ALPHA(%REF(ID),1,6,LX,NF,NR,2)
	END IF
	IF (NOBJECTS.EQ.0) THEN
	  OFIL=.FALSE.
	  CALL USRINP(OFIL,1,'L',1,1,'OLD=',
	1 'Add the results to an existing file?  (default = N)')
	  IF (OFIL) THEN
	    IOCHFL=1
	  ELSE
	    IOCHFL=2
	  END IF
	  CALL OPENCF(22,IOCHFL,'FILE_22=',0)
	  IF (.NOT.OFIL) THEN
	    WRITE(22,1100)
1100	    FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',1X,'INCLINATION',
	1   1X,'ECCENTRICITY',1X,'RADIUS',5X,'ADU',3X,'LOG(INTEGRAL)')
	  END IF
	END IF
	NOBJECTS=NOBJECTS+1
	CALL DYNAMIC(ADDRESS,LENGTH,'  R4',.TRUE.)
	CALL ANNULUS1(LU_IN,%VAL(ADDRESS),NCOL,NROW,RADIUS,ADU)
	CALL DYNAMIC(ADDRESS,LENGTH,'  R4',.FALSE.)
	NPTS=RADII(2)-RADII(1)+1
C  The following section is for integration
	IF (SKY.EQ.-999.999) THEN
C  Find the mimimum of the data	  
	  ADUMIN=ADU(1)
	  DO II=1,NPTS
	    IF (ADU(II).LT.ADUMIN) ADUMIN=ADU(II)
	  END DO
	ELSE
	  ADUMIN=SKY
	END IF
C  Do the integration, putting the log of the result into LOGINTADU(I)
	IF (RADIUS(1).NE.0.0) THEN
	  CALL MESSAGE('Integration does not begin at the center of the '//
	1 'image;  invalid result')
	END IF
	FORESHORT=SQRT(1.0-ECCENTR**2)
	LOGINTADU(2)=(ADU(1)-ADUMIN)+PI*(ADU(2)-ADUMIN)*
	12*RADIUS(2)*FORESHORT
	DO II=3,NPTS
	  LOGINTADU(II)=LOGINTADU(II-1)+PI*(ADU(II)-ADUMIN)*2*
	1 RADIUS(II)*FORESHORT
	END DO
C  Now take the log
	LOGINTADU(1)=-9.999 !  The integration must be
			    !  essentially zero at the start
	DO II=2,NPTS
	  IF (LOGINTADU(II).GT.0.0) THEN
	    LOGINTADU(II)=LOG10(LOGINTADU(II))
	  ELSE
	    LOGINTADU(II)=-9.999
	  END IF
	END DO
	DO K=1,NPTS
	  WRITE(22,1101) ID,COLUMN,ROW,INCLINA,
	1 ECCENTR,RADIUS(K),ADU(K),LOGINTADU(K)
	END DO
1101    FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,F8.3,1X,F13.6,
	11X,F11.8)
	MORE=.TRUE.
	CALL USRINP(MORE,1,'L',1,1,'MORE=','More objects to work with?  
	1(default = Y)')
	IF (MORE) GO TO 5
	CALL ANNULUS2(22)
	CALL CLOSEFITS(LU_IN)
	IF (COMMAND.EQ.'F') CLOSE(UNIT=21)
	CLOSE(UNIT=22)
	RETURN
	END
