	SUBROUTINE FAKGAL
C
C       This subroutine allows one to create maps containing artificial
C  galaxy images.  If the file already exists, the new images are
C  added/subtracted to/from what is there;  otherwise a new file is created
C  with a size  specified by the user.  The parameters may be read from a 
C  character file prepared by options BB  (not ready), EA, or PR.  Alternately,
C  the parameters for the galaxies may be input manually whereby the user 
C  enters the central location to a fraction of a pixel location, the 
C  inclination, the eccentricity, and the surface brightness formula and its
C  parameters.  For the manual case, one has the choice of writing to a one 
C  dimensional character file of type 1 or a FITS file, and of adding image
C  smearing.  One can also include the effects of an absorbing galaxy whose
C  parameters are input from a file or manually.  A dynamic memory assignment
C  is used in which the area necessary to create the image is big enough to
C  contain pixels with approximately one millionth the central intensity. 
C  Allowance is made for the center of the image to be outside the boundaries
C  of the map.
C			Donald H. Gudehus, November, 1982
C			Version 2U, Donald H. Gudehus, February 25, 2008
C			Modified code for Unix and FITS
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	BYTE DAYO(9)
C
	CHARACTER*1 CHOICE,CHOICE_ABS,GRADIENT,FILE_TYPE,ITERATION_SCHEME
	CHARACTER*2 COMMAND
	CHARACTER*3 FILEFMT,PRGFMAT
	CHARACTER*6 IDREAD,CHOICENLF
	CHARACTER*40 FILEKEYWORD,COMMENT,DNAMEREAD
	CHARACTER*50 CHFITSMHV
	CHARACTER*80 FILENAME
C
	INTEGER MAPKEY,PLANEKEY,NMAPS,NPLNS,NROWS,NCOLS,STATUS,NC,NF,NR,
	1NPTS,NPTS_ABS,NSKY,LENGTHX,LENGTHY,LENGTH,ADDRESS1,ADDRESS2,
	2MNCOLUMN,MNROW,MXCOLUMN,MXROW,IRADIICONVOLVE(2),NPOINTS_S,NPOINTS_C
C
	LOGICAL CONVOLVE,NEEDTOZERO,FIRST_TIME,MORE
C
	REAL*4 DATA(MAXSIZE),INDEXMINUSRADIUS,INDEXMINUSRADIUS_ABS,
	1PARAM_PRF(5,MAXDIAG),PARAM_PRF_ABS(5,MAXDIAG),POSITION(2),
	2POSITION_ABS(2),COLUMN,COLUMN_ABS,ROW,ROW_ABS,INCLINA,INCLINA_ABS,
	3ECCENTR,ECCENTR_ABS,RADII(3),FWHMA,FWHMB,THETA,LOGINTADU,
	4PARAMETRS(4),PARAMETRS_ABS(4),GRADPARAM(2),SKY,SKY_ABS,SCALE,
	5RADIUSLIMIT,RADIUSLIMIT_ABS,RADIUSOUT(MAXDIAG),ARRAYOUT(MAXDIAG),
	6ARRAYOUT_ABS(MAXDIAG),OPTICALDEPTH,SMEARREAD(MAXDIAG),
	7SMEARRADIUS(MAXDIAG),SMEAR(MAXDIAG),ARRAYINCONVOLVE(MAXDIAG),
	8RADIUSINCONVOLVE(MAXDIAG)
C
	REAL*8 DATADP(MAXSIZE)
C
C
	CHARACTER*6 ID,IDSMEAR,ID_ABSORB
	CHARACTER*27 IDBBEN
	COMMON /ID/ ID,IDSMEAR,ID_ABSORB,IDBBEN
C
	LOGICAL OFIL
	COMMON /OLDFILE/ OFIL
C
	CHARACTER*80 DCNAME
	COMMON /CFILE/ DCNAME

	CHARACTER*80 DNAME
	LOGICAL EXTENDED
	COMMON /DFILE/ DNAME,EXTENDED
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	CALL MESSAGE('FAKGAL, Version 2U')
	NEEDTOZERO=.FALSE.
	FIRST_TIME=.TRUE.
	LU_OUT=11
	FILEKEYWORD='FILE='
	FILENAME=' '
	IOFL=.FALSE. !  The file may or may not exist
	MAPKEY=0
	PLANEKEY=0
	NMAPS=0
	NPLNS=0
	NROWS=0
	NCOLS=0
	PRGFMAT=' '
C
	CALL MESSAGE('FAKGAL, Version 2')
	PI=3.14159265
	GRADIENT='N' !  Initialization for all commands
	FIRSTTIME=.TRUE.
	COMMAND=' '
	DO WHILE(COMMAND.NE.'B'.AND.COMMAND.NE.'P'.AND.COMMAND.NE.'PA'.AND.
	1COMMAND.NE.'PB'.AND.COMMAND.NE.'M'.AND.COMMAND.NE.'MA'.AND.COMMAND.NE.
	2'MB')
	  COMMAND='P'
	  CALL MESSAGE('Enter a command:')
	  CALL MESSAGE('     B =  Parameters contained in file prepared'//
	1 ' by option BB')
	  CALL MESSAGE('     P =  Parameters contained in a file prepared'//
	1 ' by options EA or PR (default)')
	  CALL MESSAGE('     PA = Like P, but with additional absorbing'//
	1 ' galaxy')
	  CALL MESSAGE('     PB = Like PA, but only the absorption is written')
	  CALL MESSAGE('     M =  Surface brightness formula parameters'//
	1 ' input manually')
	  CALL MESSAGE('     MA = Like M, but with additional absorbing'//
	1 ' galaxy')
	  CALL MESSAGE('     MB = Like MB, but only the absorption is written')
	  CALL USRINP(%REF(COMMAND),1,'A',2,1,'COMMAND=',' ')
	  CALL ALPHA(%REF(COMMAND),1,2,NC,NF,NR,2)
	END DO
C  Only command M can write to a character file
	IF (COMMAND.EQ.'M') THEN
	  FILE_TYPE=' '
	  DO WHILE(FILE_TYPE.NE.'C'.AND.FILE_TYPE.NE.'F') 
	    FILE_TYPE='F'
	    CALL MESSAGE('Enter the file type that you wish to write to:')
	    CALL MESSAGE('          C = Character file')
	    CALL USRINP(%REF(FILE_TYPE),1,'A',1,1,'FILE_TYPE=',
	1   '          F = FITS file (default)')
	  END DO
	ELSE
	  FILE_TYPE='F'
	END IF
	IF (FILE_TYPE.EQ.'F') THEN
	  CALL MESSAGE('Enter the name of the file to be written to')
	  CALL OPENFITS(LU_OUT,IOFL,FILEKEYWORD,FILENAME,MAPKEY,PLANEKEY,NMAPS,
	1 NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
	  IF (.NOT.IOFL.OR.EXTENDED) NEEDTOZERO=.TRUE.
	  FILEFMT=CHFITSMHV(LU_OUT,IMNUM,'BITPIX','R4')
	  IF (NEEDTOZERO) THEN
	    IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1   FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
	      DO J=1,NROWS
	        CALL RDFITSROW(LU_OUT,IMNUM,IPNUM,DATA,MAXDIAG,J)
	        DO I=1,NCOLS
	          DATA(I)=0.0
	        END DO
	        CALL WRFITSROW(LU_OUT,IMNUM,IPNUM,DATA,MAXDIAG,J)
	      END DO
	    ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	      DO J=1,NROWS
	        CALL RDDPFITSROW(LU_OUT,IMNUM,IPNUM,DATADP,MAXDIAG,J)
	        DO I=1,NCOLS
	          DATADP(I)=0.0D0
	        END DO
	        CALL WRDPFITSROW(LU_OUT,IMNUM,IPNUM,DATADP,MAXDIAG,J)
	      END DO
	    END IF
	  END IF
	ELSE IF (FILE_TYPE.EQ.'C') THEN
C  Open the output data file
	  CALL OPENCF(22,0,'FILE_22=',0)
	  IF (.NOT.OFIL) THEN
	    WRITE(22,1100)
1100	    FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',1X,'INCLINATION',
	1   1X,'ECCENTRICITY',1X,'RADIUS',5X,'ADU',3X,'LOG(INTEGRAL)')
	  END IF
	END IF
C  Get the input data file information
	IF (COMMAND.EQ.'B'.OR.COMMAND(1:1).EQ.'P') THEN
	  CALL MESSAGE('Open the input character data file')
	  CALL OPENCF(21,1,'FILE_21=',0)
  12	  CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1 'Enter the ID of the object:  (default = last entry)')
	  CALL ALPHA(%REF(ID),1,6,NC,NF,NR,2)
	  REWIND 21
	  IF (COMMAND.EQ.'B') THEN
	    CALL MESSAGE('Sorry, not ready yet')
	    IDBBEN=ID
	    READ(21,1000,ERR=24) COMMENT,DAYO
1000	    FORMAT(' ',A/' File measured:  ',9A)
	    READ(21,1001,ERR=24)
	    READ(21,1002,ERR=24)
1001	    FORMAT(' FILE      COLUMN      ROW      SKY(AVG)   NSKY ',
	1   'STDEV(SKY)  STAR(AV-AV)  NSTR STDEV(STAR) MAG(AV-AV) ',
	2   'RADIUS  FWHMA   FWHMB   THETA')
1002	    FORMAT(' ID                           SKY(CNTRD)        ',
	1   'QUADRANTS   STAR(AV-CT)                   MAG(AV-CT)')
	    IDREAD=' '
	    DO WHILE (IDBBEN.NE.IDREAD)
	      READ(21,1003) DNAMEREAD,COLUMN,ROW,SKY,NSKY,RMSK,STAR,
	1     NSTR,RMSR,AGA,RADII(1),FWHMA,FWHMB,THETA
	      READ(21,1004) IDREAD,SKYC,STARAC,AAC
1003	      FORMAT(' ',A7,F9.3,1X,F9.3,1X,F12.3,1X,I6,1X,F8.3,1X,
	1     F13.3,1X,I6,1X,F10.3,1X,F9.3,1X,F7.1,1X,F7.3,1X,F7.3,1X,
	2     F7.2)
1004	      FORMAT(' ',A27,F12.3,9X,I1,1X,I1,1X,I1,1X,I1,1X,F13.3,19X,
	1     F9.3)
	    END DO
  24	    ECCENTR=SQRT(1.0-(FWHMB/FWHMA)**2)
C
	  ELSE IF (COMMAND.EQ.'P'.OR.COMMAND.EQ.'PA') THEN
	    CALL RDDATFL1A(21,ID,PARAM_PRF,MAXDIAG,NPTS,
	1   INDEXMINUSRADIUS)
	    COLUMN=PARAM_PRF(1,NPTS)
	    ROW=PARAM_PRF(2,NPTS)
	    SKY=-999.999
	    CALL USRINP(SKY,1,'F',4,3,'SKY=',
	1   'Enter the sky to be subtracted from the data in the file:'//
	2   '  (default = minimum)')
	    IF (SKY.EQ.-999.999) THEN
	      SKY=PARAM_PRF(5,1)
	      DO I=1,NPTS
	        SKY=AMIN1(SKY,PARAM_PRF(5,I))
	      END DO
	    END IF
	    IF (COMMAND.EQ.'PA'.OR.COMMAND.EQ.'PB') THEN
	      CALL MESSAGE('Open the input data file for absorption')
	      CALL OPENCF(24,1,'FILE_24=',0)
	      CALL USRINP(%REF(ID_ABSORB),1,'A',6,1,'ID_ABSORBER=',
	1     'Enter the ID of the absorbing object:  (default = last entry)')
	      CALL ALPHA(%REF(ID_ABSORB),1,6,NC,NF,NR,2)
	      REWIND 24
	      CALL RDDATFL1A(24,ID_ABSORB,PARAM_PRF_ABS,MAXDIAG,NPTS_ABS,
	1     INDEXMINUSRADIUS_ABS)
C  The absorber distribution will be normalized to 1 at the center, but
C  this must occur after the sky is subtracted at the end of FAKGAL1.
	      COLUMN_ABS=PARAM_PRF_ABS(1,NPTS_ABS)
	      ROW_ABS=PARAM_PRF_ABS(2,NPTS_ABS)
	      SKY_ABS=-999.999
	      CALL MESSAGE('Enter the sky to be subtracted from the data in'//
	1     ' the absorption file:')
	      CALL USRINP(SKY_ABS,1,'F',4,3,'SKY_ABSORBER=',
	1     '(default = minimum)')
	      IF (SKY_ABS.EQ.-999.999) THEN
	        SKY_ABS=PARAM_PRF_ABS(5,1)
	        DO I=1,NPTS_ABS
	          SKY_ABS=AMIN1(SKY_ABS,PARAM_PRF_ABS(5,I))
	        END DO
	      END IF
	    END IF
	  END IF
	END IF
C
  49	IF (COMMAND.EQ.'B'.OR.COMMAND(1:1).EQ.'M') THEN
	  CHOICE=' '
	  DO WHILE(CHOICE.NE.'H'.AND.CHOICE.NE.'M'.AND.CHOICE.NE.'P'
	1 .AND.CHOICE.NE.'T'.AND.CHOICE.NE.'V'.AND.CHOICE.NE.'G')
	    CHOICE='H'
	    CALL MESSAGE('Enter the surface brightness law:')
	    CALL MESSAGE('     H = Hubble  (default)')
	    CALL MESSAGE('     M = Modified Hubble')
	    CALL MESSAGE('     P = Hubble with power series for "1/a"')
	    CALL MESSAGE('     T = Tapered Hubble')
	    CALL MESSAGE('     V = de Vaucouleurs')
	    CALL MESSAGE('     G = Gaussian')
	    CALL USRINP(%REF(CHOICE),1,'A',1,1,'CHOICE=',' ')
	  END DO
C
	  IF (COMMAND.EQ.'B') THEN
	    IF (CHOICE.EQ.'H') THEN
C  Get the Hubble "a"
	      A=FWHMA/(2.0*SQRT(2.0)-2.0)
C  Get the central surface brightness
	      PARAMETRS(1)=(STARAC/(2*PI*A**2))/
	1     (1.0/(1.0+RADII(1)/A)-1.0+LOG(1.0+RADII(1)/A)-
	2     (RADII(1)**2/(RADII(3)**2-RADII(2)**2))*
	3     (1.0/(1.0+RADII(3)/A)-1.0/(1.0+RADII(2)/A)+
	4     LOG(1.0+RADII(3)/A)-LOG(1.0+RADII(2)/A)))
	      PARAMETRS(2)=A
	    END IF
C
	  ELSE IF (COMMAND(1:1).EQ.'M') THEN
	    IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M') THEN
	      CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1     'Enter the central surface brightness (ADU) and the Hubble'//
	2     ' "a" (pixels):')
	    ELSE IF (CHOICE.EQ.'P') THEN
	      CALL MESSAGE('Enter the central surface brightness (ADU),
	1 the Hubble "a" (pixels),')
	      CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1     'and the first and second order coefficients for "1/a"'//
	2     ' (pixels):')
	    ELSE IF (CHOICE.EQ.'T') THEN
	      CALL MESSAGE('Enter the central surface brightness (ADU),'//
	1    ' the Hubble "a" (pixels),')
	      CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1     'and the taper "b" (pixels):')
	    ELSE IF (CHOICE.EQ.'V') THEN
	      CALL MESSAGE('Enter the surface brightness at 1/2 the'//
	1     ' total light (ADU) and the')
	      CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1     'corresponding radius (pixels)')
	    ELSE IF (CHOICE.EQ.'G') THEN
	      CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1     'Enter the central surface brightness (ADU) and the'//
	2     ' FWHM (pixels):')
C  Convert from FWHM to a scaling parameter
	      PARAMETRS(2)=PARAMETRS(2)/(2.0*SQRT(LOG(2.0)))
	    END IF
	    IF (FILE_TYPE.EQ.'C') THEN
	      SKY=0.0
	      CALL USRINP(SKY,1,'F',4,1,'SKY=','Enter the sky to be added to'//
	1     ' the data:  (default = 0.0)')
	    END IF
C  Inquire about a gradient, but only when there is no absorbing galaxy
	    IF (COMMAND.EQ.'M') THEN
	      GRADIENT='N'
	      CALL MESSAGE('Enter a choice:')
	      CALL MESSAGE('     N = No gradient applied (default)') 
	      CALL MESSAGE('     L = Multiplies by a gradient linear in'//
	1     ' magnitude and log(radius)')
	      CALL USRINP(%REF(GRADIENT),1,'A',1,1,'GRADIENT=',
	1     '     R = Multiplies by a gradient linear in'//
	2     ' magnitude and radius')
	      IF (GRADIENT.EQ.'L'.OR.GRADIENT.EQ.'R') THEN
	        CALL USRINP(GRADPARAM,2,'F',4,0,'PARAMETERS=',
	1       'Enter the constant and linear term:')
	      END IF
	    END IF
	    CALL USRINP(POSITION,2,'F',4,1,'POSITION=',
	1   'Enter the column and row:  (default = last position,'//
	2   ' fractional values allowed)')
	    IF (FILE_TYPE.EQ.'F') THEN
	      COLUMN=POSITION(1)
	      ROW=POSITION(2)
	    END IF
	    INCLINA=0.0
	    CALL MESSAGE('Enter the ellipse inclination angle (Cartesian'//
	1   ' system):')
	    CALL USRINP(INCLINA,1,'F',4,1,'INCLINATION=',
	1   '(default = 0.0 degrees)')
	    ECCENTR=0.
	    CALL USRINP(ECCENTR,1,'F',4,1,'ECCENTRICITY=',
	1   'Enter the eccentricity of the ellipse:  (default = 0.0)')
C  Get additional information if there is an absorbing galaxy
	    IF (COMMAND.EQ.'MA'.OR.COMMAND.EQ.'MB') THEN
	      CHOICE_ABS=' '
	      DO WHILE(CHOICE_ABS.NE.'H'.AND.CHOICE_ABS.NE.'M'.AND.
	1     CHOICE_ABS.NE.'P'.AND.CHOICE_ABS.NE.'T'.AND.CHOICE_ABS.NE.'V'
	2     .AND.CHOICE_ABS.NE.'G')
	        CHOICE_ABS='H'
	        CALL MESSAGE('Enter the surface brightness law for the'//
	1       ' absorbing galaxy:')
	        CALL MESSAGE('     H = Hubble  (default)')
	        CALL MESSAGE('     M = Modified Hubble')
	        CALL MESSAGE('     P = Hubble with power series for "1/a"')
	        CALL MESSAGE('     T = Tapered Hubble')
	        CALL MESSAGE('     V = de Vaucouleurs')
	        CALL MESSAGE('     G = Gaussian')
	        CALL USRINP(%REF(CHOICE_ABS),1,'A',1,1,'CHOICE_ABSORBER=',' ')
	      END DO
	      CALL MESSAGE('Enter the parameters for the absorbing galaxy')
C  The absorber distribution will be normalized to 1 at the center but this
C  must occur after any convolution.
	      IF (CHOICE_ABS.EQ.'H'.OR.CHOICE_ABS.EQ.'M') THEN
	        CALL USRINP(PARAMETRS_ABS,4,'F',4,0,'PARAMETERS_ABSORBER=',
	1       'Enter the central surface brightness (ADU) and the Hubble'//
	2       ' "a" (pixels):')
	      ELSE IF (CHOICE_ABS.EQ.'P') THEN
	        CALL MESSAGE('Enter the central surface brightness (ADU),'//
	1       ' the Hubble "a" (pixels),')
	        CALL USRINP(PARAMETRS_ABS,4,'F',4,0,'PARAMETERS_ABSORBER=',
	1       'and the first and second order coefficients for "1/a"
	2 (pixels):')
	      ELSE IF (CHOICE_ABS.EQ.'T') THEN
	        CALL MESSAGE('Enter the central surface brightness (ADU),'//
	1       ' the Hubble "a" (pixels),')
	        CALL USRINP(PARAMETRS_ABS,4,'F',4,0,'PARAMETERS_ABSORBER=',
	1       'and the taper "b" (pixels):')
	      ELSE IF (CHOICE_ABS.EQ.'V') THEN
	        CALL MESSAGE('Enter the surface brightness at 1/2 the'//
	1       ' total light (ADU) and the')
	        CALL USRINP(PARAMETRS_ABS,4,'F',4,0,'PARAMETERS_ABSORBER=',
	1       'corresponding radius (pixels)')
	      ELSE IF (CHOICE_ABS.EQ.'G') THEN
	    CALL USRINP(PARAMETRS_ABS,4,'F',4,0,'PARAMETERS_ABSORBER=',
	1   'Enter the central surface brightness (ADU) and the'//
	2   ' FWHM (pixels):')
C  Convert from FWHM to a scaling parameter
	        PARAMETRS_ABS(2)=PARAMETRS_ABS(2)/(2.0*SQRT(LOG(2.0)))
	      END IF
	      CALL MESSAGE('Enter the column and row of the absorber:'//
	1     ' (default = last position,')
	      CALL USRINP(POSITION_ABS,2,'F',4,1,'POSITION_ABSORBER=',
	1     'fractional values allowed)')
	      IF (FILE_TYPE.EQ.'F') THEN
	        COLUMN_ABS=POSITION_ABS(1)
	        ROW_ABS=POSITION_ABS(2)
	      END IF
	      INCLINA_ABS=0.0
	      CALL USRINP(INCLINA_ABS,1,'F',4,1,'INCLINATION_ABSORBER=',
	1     'Enter the ellipse inclination angle of the absorber:'//
	2     '  (default = 0.0 degrees)')
	      ECCENTR_ABS=0.
	      CALL USRINP(ECCENTR_ABS,1,'F',4,1,'ECCENTRICITY_ABSORBER=',
	1     'Enter the eccentricity of the ellipse for the absorber:'//
	2     '  (default = 0.0)')
	    END IF
	  END IF
C
C  Get the radius where the surface brightness is one millionth of the
C  the central surface brightness for the various choices
	  IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'P') RADIUSLIMIT=PARAMETRS(2)
	1 *999.0
	  IF (CHOICE.EQ.'M') RADIUSLIMIT=PARAMETRS(2)*280.9
	  IF (CHOICE.EQ.'T') RADIUSLIMIT=PARAMETRS(2)*999.0
	  IF (CHOICE.EQ.'V') RADIUSLIMIT=PARAMETRS(2)*16.38  ! Factor of
			! one five millionth;  10.54 is one millionth
	  IF (CHOICE.EQ.'G') RADIUSLIMIT=PARAMETRS(2)*5.257
	  IF (COMMAND.EQ.'MA'.OR.COMMAND.EQ.'MB') THEN
	    IF (CHOICE_ABS.EQ.'H'.OR.CHOICE_ABS.EQ.'P') RADIUSLIMIT_ABS
	1   =PARAMETRS_ABS(2)*999.0
	    IF (CHOICE_ABS.EQ.'M') RADIUSLIMIT_ABS=PARAMETRS_ABS(2)*280.9
	    IF (CHOICE_ABS.EQ.'T') RADIUSLIMIT_ABS=PARAMETRS_ABS(2)*999.0
	    IF (CHOICE_ABS.EQ.'V') RADIUSLIMIT_ABS=PARAMETRS_ABS(2)*10.54
	    IF (CHOICE_ABS.EQ.'G') RADIUSLIMIT_ABS=PARAMETRS_ABS(2)*5.257
	  END IF
	END IF
C
C  Now do the actual calculations
C
	IF (COMMAND(1:1).EQ.'M') THEN
	  RADIUSLIMIT=MIN(RADIUSLIMIT,MAXDIAG-1.0)
	  NPTS=INT(RADIUSLIMIT)
	  CONVOLVE=.FALSE.
	  IF (COMMAND.EQ.'M') THEN
	    CALL USRINP(CONVOLVE,1,'L',1,1,'CONVOLVE=',
	1   'Do you wish to convolve the distribution?  (default = N)')
	  ELSE IF (COMMAND.EQ.'MA'.OR.COMMAND.EQ.'MB') THEN
	    CALL USRINP(CONVOLVE,1,'L',1,1,'CONVOLVE=',
	1   'Do you wish to convolve the distributions?  (default = N)')
	  END IF
	  IF (CONVOLVE) THEN
	    RADIUSINCONVOLVE(1)=0.0
	    CALL MESSAGE('Open the input smearing file')
	    CALL OPENCF(23,1,'FILE_23=',0)
	    CALL USRINP(%REF(IDSMEAR),1,'A',6,1,'ID=',
	1   'Enter the ID of the object:  (default = last smearing'//
	2   ' entry)')
	    CALL ALPHA(%REF(IDSMEAR),1,6,NC,NF,NR,2)
	    CALL RDDATFL1B(23,IDSMEAR,SMEARREAD,SMEARRADIUS,MAXDIAG,NPOINTS_S)
	    DO I=1,NPOINTS_S
	      CALL INTERP1D(SMEARRADIUS,SMEARREAD,NPOINTS_S,REAL(I-1),
	1     SMEAR(I))
	    END DO
	    CALL USRINP(NPOINTS_C,1,'I',4,0,'RADIUS=',
	1   'Enter the outer radius to where convolution is to be done:')
	    NPOINTS_C=NPOINTS_C+1
	    I=1
	    IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'T'
	2   .OR.CHOICE.EQ.'P'.OR.CHOICE.EQ.'G') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	      DO WHILE (RADIUSINCONVOLVE(I).LT.RADIUSLIMIT+1)
					! One larger than outer radius
				        ! for interpolation in CONVLV2DG
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
C	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/PARAMETRS(2))**2*
C	1       PARAMETRS(2)**2/200.0))
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/PARAMETRS(2))**2*
	1       PARAMETRS(2)/100.0))
	        IF (I.GE.MAXDIAG) CALL ERROR(-4,'Step size too small')
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2))+10.0)/20.0)*
	1     96.0)
	    ELSE IF (CHOICE.EQ.'V') THEN
	      DO WHILE (RADIUSINCONVOLVE(I).LT.RADIUSLIMIT+1)
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
C	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/PARAMETRS(2))
C	1       **2*(PARAMETRS(2)/10.0)**2/200.0))
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/PARAMETRS(2))**2*
	1       (PARAMETRS(2)/10.0)/100.0))
	        IF (I.GE.MAXDIAG) CALL ERROR(-4,'Step size too small')
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2)/10.0)+10.0)/
	1     20.0)*96.0)
	    END IF
	    IRADIICONVOLVE(1)=1
	    IRADIICONVOLVE(2)=I-1
	    DO NN=IRADIICONVOLVE(1),IRADIICONVOLVE(2)
	      CALL FAKGALFUN(CHOICE,PARAMETRS,RADIUSINCONVOLVE(NN),
	1     GRADIENT,GRADPARAM,TERM)
	      ARRAYINCONVOLVE(NN)=TERM
	    END DO
	    CALL CONVLV2DG(ARRAYINCONVOLVE,RADIUSINCONVOLVE,MAXDIAG,
	1   SMEAR,NPOINTS_S,ARRAYOUT,NPTS,NPOINTS_C,
	2   NANGLES)
	  ELSE IF (.NOT.CONVOLVE.AND.FILE_TYPE.EQ.'C') THEN
	    DO NN=1,NPTS
	      CALL FAKGALFUN(CHOICE,PARAMETRS,(FLOAT(NN)-1.0),GRADIENT,
	1     GRADPARAM,TERM)
	      ARRAYOUT(NN)=TERM
	    END DO
C  The case of not convolving and writing to a SAD file does not need any
C  calculation at this time;  PARAMETRS is used in FAKGAL1 to write directly
C  to the file at each semimajor axis value.
	  END IF
	  IF (COMMAND.EQ.'MA'.OR.COMMAND.EQ.'MB') THEN
C  Note that a data file is never written to for these cases
	    RADIUSLIMIT=MIN(RADIUSLIMIT_ABS,MAXDIAG-1.0)
	    NPTS_ABS=INT(RADIUSLIMIT_ABS)
	    IF (CONVOLVE) THEN
	      RADIUSINCONVOLVE(1)=0.0
	      I=1
	      IF (CHOICE_ABS.EQ.'H'.OR.CHOICE_ABS.EQ.'M'.OR.CHOICE_ABS.EQ.'T'
	2     .OR.CHOICE_ABS.EQ.'P'.OR.CHOICE.EQ.'G') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the absorbing galaxy.
	        DO WHILE (RADIUSINCONVOLVE(I).LT.RADIUSLIMIT+1)
					! One larger than outer radius
				        ! for interpolation in CONVLV2DG
	          RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1         MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/PARAMETRS(2))**2*
	1         PARAMETRS(2)/100.0))
	          IF (I.GE.MAXDIAG) CALL ERROR(-4,'Step size too small')
	          I=I+1
	        END DO
	        NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2))+10.0)/20.0)*
	1       96.0)
	      ELSE IF (CHOICE_ABS.EQ.'V') THEN
	        DO WHILE (RADIUSINCONVOLVE(I).LT.RADIUSLIMIT+1)
	          RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1         MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/PARAMETRS(2))**2*
	1         (PARAMETRS(2)/10.0)/100.0))
	          IF (I.GE.MAXDIAG) CALL ERROR(-4,'Step size too small')
	          I=I+1
	        END DO
	        NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2)/10.0)+10.0)/
	1       20.0)*96.0)
	      END IF
	      IRADIICONVOLVE(1)=1
	      IRADIICONVOLVE(2)=I-1
	      DO NN=IRADIICONVOLVE(1),IRADIICONVOLVE(2)
	        CALL FAKGALFUN(CHOICE_ABS,PARAMETRS_ABS,RADIUSINCONVOLVE(NN),
	1       GRADIENT,GRADPARAM,TERM)
	        ARRAYINCONVOLVE(NN)=TERM
	      END DO
	      CALL CONVLV2DG(ARRAYINCONVOLVE,RADIUSINCONVOLVE,MAXDIAG,
	1     SMEAR,NPOINTS_S,ARRAYOUT_ABS,NPTS,NPOINTS_C,
	2     NANGLES)
C  Do not consider the case of a data file with an absorber
	    END IF
	  END IF
	  IF (FILE_TYPE.EQ.'C') THEN
	    CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1   'Enter the ID of the object to be written to the file:'//
	2   ' (default = last entry)')
	    CALL ALPHA(%REF(ID),1,6,NC,NF,NR,2)
	    CALL FAKGAL2(CHOICE,CONVOLVE,ID,POSITION,INCLINA,ECCENTR,
	1   PARAMETRS,SKY,ARRAYOUT,NPTS)
C  Skip ahead now for this case
	    GO TO 60
	  END IF
C  Load RADIUSOUT for use in POLINTERP1D in FAKGAL1 and FAKGAL3
	  IF (CONVOLVE) THEN
	    DO I=1,MAX(NPTS,NPTS_ABS)
	      RADIUSOUT(I)=I-1.0
	    END DO
	  END IF
	END IF  ! End of COMMAND='M' IF block
C
	IF (COMMAND.EQ.'PA'.OR.COMMAND.EQ.'PB'.OR.COMMAND.EQ.'MA'
	1.OR.COMMAND.EQ.'MB') THEN
	  OPTICALDEPTH=0.01
	  CALL USRINP(OPTICALDEPTH,1,'F',4,1,'OPTICAL_DEPTH=',
	1 'Enter the optical depth:  (default = 0.01)')
	END IF
C  Get the scale factor for all commands which write to a SAD file
	IF (FIRSTTIME) THEN
	  SCALE=1.0
	  CALL USRINP(SCALE,1,'F',4,1,'SCALE=',
	1 'Enter the intensity scale factor:  (default = 1.0)')
	END IF
C
	IF (COMMAND(1:1).EQ.'P') RADIUSLIMIT=NPTS-INDEXMINUSRADIUS+20
	MNCOLUMN=JINT(COLUMN-RADIUSLIMIT)
	IF (MNCOLUMN.LT.1) MNCOLUMN=1
	IF (MNCOLUMN.GT.NCOLS) MNCOLUMN=NCOLS
	MNROW=JINT(ROW-RADIUSLIMIT)
	IF (MNROW.LT.1) MNROW=1
	IF (MNROW.GT.NROWS) MNROW=NROWS
	MXCOLUMN=JINT(COLUMN+RADIUSLIMIT)
	IF (MXCOLUMN.LT.1) MXCOLUMN=1
	IF (MXCOLUMN.GT.NCOLS) MXCOLUMN=NCOLS
	MXROW=JINT(ROW+RADIUSLIMIT)
	IF (MXROW.LT.1) MXROW=1
	IF (MXROW.GT.NROWS) MXROW=NROWS
	LENGTHX=MXCOLUMN-MNCOLUMN+1
	LENGTHY=MXROW-MNROW+1
	LENGTH=LENGTHX*LENGTHY
	CALL DYNAMIC(ADDRESS1,LENGTH,'  R4',.TRUE.)
	CALL DYNAMIC(ADDRESS2,LENGTH,'  R4',.TRUE.)
	CALL FAKGAL1(LU_OUT,%VAL(ADDRESS1),%VAL(ADDRESS2),LENGTHX,
	1LENGTHY,COMMAND,NCOLS,NROWS,COLUMN,COLUMNS_ABS,ROW,ROW_ABS,
	2INDEXMINUSRADIUS,INDEXMINUSRADIUS_ABS,PARAM_PRF,PARAM_PRF_ABS,
	3NPTS,NPTS_ABS,MNCOLUMN,MNROW,MXCOLUMN,MXROW,INCLINA,
	4INCLINA_ABS,ECCENTR,ECCENTR_ABS,PARAMETRS,PARAMETRS_ABS,
	5SKY,SKY_ABS,GRADPARAM,SCALE,CONVOLVE,RADIUSOUT,ARRAYOUT,
	6ARRAYOUT_ABS,CHOICE,GRADIENT,OPTICALDEPTH)
	CALL DYNAMIC(ADDRESS1,LENGTH,'  R4',.FALSE.)
	CALL DYNAMIC(ADDRESS2,LENGTH,'  R4',.FALSE.)
  60	FIRSTTIME=.FALSE.
	MORE=.TRUE.
	IF (COMMAND.EQ.'B'.OR.COMMAND(1:1).EQ.'P') THEN
	  CALL USRINP(MORE,1,'L',1,1,'MORE=',
	1 'More galaxies to model from this character file?  (default = Y)')
	ELSE
	  CALL USRINP(MORE,1,'L',1,1,'MORE=',
	1 'More galaxies to model?  (default = Y)')
	END IF
	IF (MORE) THEN
	  IF (COMMAND.EQ.'B'.OR.COMMAND(1:1).EQ.'P') GO TO 12
	  IF (COMMAND(1:1).EQ.'M') GO TO 49
	END IF
	IF (FILE_TYPE.EQ.'F') THEN
	  CALL CLOSEFITS(LU_OUT)
	ELSE IF (FILE_TYPE.EQ.'C') THEN
	  CLOSE (UNIT=22)
	END IF
	IF (COMMAND.EQ.'B'.OR.COMMAND(1:1).EQ.'P') CLOSE(UNIT=21)
	IF (CONVOLVE) CLOSE(UNIT=23)
	IF (COMMAND.EQ.'PA'.OR.COMMAND.EQ.'PB'.OR.COMMAND.EQ.'MA'
	1.OR.COMMAND.EQ.'MB') CLOSE(UNIT=24)
	RETURN
	END
