	SUBROUTINE FAKSTAR
C
C       This subroutine allows one to create maps containing artificial
C  star images.  If the file already exists, the new images are added
C  to what is there, otherwise a new file is created with a size 
C  specified by the user.  The parameters for the stars may be input
C  manually whereby the user enters the central location to a fraction
C  of a pixel location, the inclination, the eccentricity, and the
C  central surface brightness in ADU.  Alternately, the parameters
C  may be read from a file prepared by option BB, FI, or PR.  For
C  the first three cases one must then specify the particular surface
C  brightness law.  A dynamic memory assignment is used in which the
C  area necessary to create the image is big enough to contain pixels
C  with one millionth the central intensity.  Allowance is made
C  the center of the image to be outside the boundaries of the map.
C		Donald H. Gudehus, November, 1982
C  		Version 2, Jim Haywood, August, 1983
C  		Added code to generate artificial speckle images
C		Version 3, Donald H. Gudehus, June, 1989
C		Added code to produce images from a file prepared by option
C		FI.
C		Version 4, Donald H. Gudehus, July 1, 1999
C		Fixed up questionable branches into blocks
C		Version 4U, Donald H. Gudehus, February 17, 2008
C		Converted to Unix and FITS
C	
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	BYTE DAYO(9)
C
	CHARACTER*1 CHOICE,GRADIENT,FILE
	CHARACTER*2 COMMAND
	CHARACTER*3 FILEFMT,PRGFMAT
	CHARACTER*6 IDREAD
	CHARACTER*40 FILEKEYWORD,COMMENT,DNAMEREAD
	CHARACTER*68 CHFITSMHV
	CHARACTER*80 FILENAME
C
	INTEGER MAPKEY,PLANEKEY,NMAPS,NPLNS,NROWS,NCOLS,STATUS,NC,NF,NR,
	1NPTS,NPTS_ABS,NSKY,IQ(4),NUMBER_OF_SPECKLES,LENGTHX,LENGTHY,LENGTH,ADDRESS1,
	1ADDRESS2,SEED(2),MNCOLUMN,MNROW,MXCOLUMN,MXROW,IMAGE,NUMSPEC,
	2LLO,MLO
C
	LOGICAL CONVOLVE,NEEDTOZERO,FIRST_TIME,MORE,ALL_OBJECTS,
	1ALL_OBJECTS_FIRST_TIME
C
	REAL*4 DATA(MAXSIZE),INDEXMINUSRADIUS,INDEXMINUSRADIUS_ABS,
	1PARAM_PRF(5,MAXDIAG),PARAM_PRF_ABS(5,MAXDIAG),POSITION(2),
	2COLUMN,COLUMN_ABS,ROW,ROW_ABS,COLUMNS,ROWS,INCLINA,INCLINA_ABS,
	3ECCENTR,ECCENTR_ABS,RADII,FWHMA,FWHMB,THETA,ADU,LOGINTADU,
	4PARAMETRS(4),PARAMETRS_ABS(4),GRADPARAM(2),PRIMARY,RATIO,OFFSET(2),
	5PEAK,SKY,SKY_ABS,SEEING,INCLIN_S,ECCENT_S,SCALE,SUM1,SUM2,X,Y,
	6COLUMN2,ROW2,RADIUSLIMIT,RADIUSLIMIT_ABS,RADIUSOUT(MAXDIAG),
	7ARRAYOUT(MAXDIAG),ARRAYOUT_ABS(MAXDIAG),OPTICALDEPTH
C
	REAL*8 DATADP(MAXSIZE)
C
	CHARACTER*6 ID,IDSMEAR,ID_ABS
	CHARACTER*27 IDBBEN
	COMMON /ID/ ID,IDSMEAR,ID_ABS,IDBBEN

	CHARACTER*80 DNAME
	LOGICAL EXTENDED
	COMMON /DFILE/ DNAME,EXTENDED
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	CALL MESSAGE('FAKSTAR, Version 4U')
	NEEDTOZERO=.FALSE.
	FIRST_TIME=.TRUE.
	CONVOLVE=.FALSE. !  Not used in FAKSTAR
	GRADIENT='N' !  Not used in FAKSTAR
	NUMBER_OF_SPECKLES=1
	RATIO=0.0
C
	LU_OUT=11
	FILEKEYWORD='FILE='
	FILENAME=' '
	IOFL=.FALSE. !  The file may or may not exist
	MAPKEY=0
	PLANEKEY=0
	NMAPS=0
	NPLNS=0
	NROWS=0
	NCOLS=0
	PRGFMAT=' '
	CALL MESSAGE('Open a FITS file to be written to')
	CALL OPENFITS(LU_OUT,IOFL,FILEKEYWORD,FILENAME,MAPKEY,PLANEKEY,NMAPS,
	1NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
	IF (.NOT.IOFL.OR.EXTENDED) NEEDTOZERO=.TRUE.
	FILEFMT=CHFITSMHV(LU_OUT,IMNUM,'BITPIX','R4')
	IF (NEEDTOZERO) THEN
	  IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1 FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
	    DO J=1,NROWS
	      CALL RDFITSROW(LU_OUT,IMNUM,IPNUM,DATA,MAXSIZE,J)
	      DO I=1,NCOLS
	        DATA(I)=0.0
	      END DO
	      CALL WRFITSROW(LU_OUT,IMNUM,IPNUM,DATA,MAXSIZE,J)
	    END DO
	  ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	    DO J=1,NROWS
	      CALL RDDPFITSROW(LU_OUT,IMNUM,IPNUM,DATADP,MAXSIZE,J)
	      DO I=1,NCOLS
	        DATADP(I)=0.0D0
	      END DO
	      CALL WRDPFITSROW(LU_OUT,IMNUM,IPNUM,DATADP,MAXSIZE,J)
	    END DO
	  END IF
	END IF
C  Get the command
	COMMAND=' '
	DO WHILE (COMMAND.NE.'B'.AND.COMMAND.NE.'F'.AND.COMMAND.NE.'M'
	1.AND.COMMAND.NE.'P')
	  COMMAND='M'
	  CALL MESSAGE('Enter a command:')
	  CALL MESSAGE('     B = Parameters contained in a file prepared'//
	1 ' by option BB')
	  CALL MESSAGE('     F = Parameters contained in a file prepared'//
	1 ' by option FI')
	  CALL MESSAGE('     P = Parameters contained in a file prepared'//
	1 ' by option PR')
	  CALL MESSAGE('     M = Parameters input manually (default)')
	  CALL USRINP(%REF(COMMAND),1,'A',2,1,'COMMAND=',' ')
	  CALL ALPHA(%REF(COMMAND),1,2,NC,NF,NR,2)
	END DO
	ALL_OBJECTS=.FALSE.
C  Open the file and get the ID
  	IF (COMMAND.EQ.'B'.OR.COMMAND.EQ.'F') THEN
	  CALL MESSAGE('Open the data file')
	  CALL OPENCF(21,1,'FILE_21=',0)
	  MORE=.FALSE.
	ELSE IF (COMMAND.EQ.'P') THEN
	  CALL OPENCF(21,1,'FILE_21=',0)
	END IF
  12	CONTINUE !  Return point for more objects under commands B, F, and P
  	IF (COMMAND.EQ.'B'.OR.COMMAND.EQ.'F') THEN
	  IF (.NOT.MORE) THEN
	    CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1   'Enter the ID of the object:  (* does all objects in file)')
	  ELSE IF (MORE) THEN
	    CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1   'Enter the ID of the object:')
	  END IF
	  CALL ALPHA(%REF(ID),1,6,NC,NF,NR,2)
	  IF (ID.EQ.'*') THEN
	    ALL_OBJECTS=.TRUE.
	    ALL_OBJECTS_FIRST_TIME=.TRUE.
	  END IF
	  REWIND 21
	ELSE IF (COMMAND.EQ.'P') THEN
	  CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1 'Enter the ID of the object:')
	  CALL ALPHA(%REF(ID),1,6,NC,NF,NR,2)
	  REWIND 21
	END IF
C  Read the data for a given ID
C  Return point for ALL_OBJECTS=.TRUE. and commands B and F
  20	IF (COMMAND.EQ.'B'.AND.ALL_OBJECTS) THEN
	  CALL MESSAGE('Sorry, not ready yet')
	ELSE IF (COMMAND.EQ.'B'.AND..NOT.ALL_OBJECTS) THEN
	  CALL MESSAGE('Sorry, not ready yet')
	  READ(21,1000,ERR=9991) COMMENT,DAYO
1000	  FORMAT(' ',A/' File measured:  ',9A)
	  READ(21,1001,ERR=9991)
	  READ(21,1002,ERR=9991)
1001	  FORMAT(' FILE      COLUMN      ROW      SKY(AVG)   NSKY ',
	1 'STDEV(SKY)  STAR(AV-AV)  NSTR STDEV(STAR) MAG(AV-AV) ',
	2 'RADIUS  FWHMA   FWHMB   THETA')
1002	  FORMAT(' ID                           SKY(CNTRD)        ',
	1 'QUADRANTS   STAR(AV-CT)                   MAG(AV-CT)')
	  IDREAD=' '
	  DO WHILE (ID.NE.IDREAD)
	    READ(21,1003) DNAMEREAD,COLUMN,ROW,SKY,NSKY,RMSK,STAR,
	1   NSTR,RMSR,AGA,RADII,FWHMA,FWHMB,THETA
	    READ(21,1004) IDREAD,SKYC,STARAC,AAC
1003	    FORMAT(' ',A7,F9.3,1X,F9.3,1X,F12.3,1X,I6,1X,F8.3,1X,
	1   F13.3,1X,I6,1X,F10.3,1X,F9.3,1X,F7.1,1X,F7.3,1X,F7.3,1X,
	2   F7.2)
1004	    FORMAT(' ',A27,F12.3,9X,I1,1X,I1,1X,I1,1X,I1,1X,F13.3,19X,
	1   F9.3)
	  END DO
	  ECCENTR=SQRT(1.0-(FWHMB/FWHMA)**2)
	ELSE IF (COMMAND.EQ.'F'.AND.ALL_OBJECTS) THEN
	  IF (ALL_OBJECTS_FIRST_TIME) THEN
C  Read the header and the first object
	    READ (21,1200)
1200	    FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',2X,'INCLINATION',
	1   1X,'ECCENTRICITY',1X,'FWHMA',5X,'FWHMB',1X,
	1   'QUADRANT CODE        PEAK             SKY')
	    READ(21,1201) ID,COLUMN,ROW,INCLINA,ECCENTR,
	1   FWHMA,FWHMB,(IQ(JJ),JJ=1,4),PEAK,SKY
1201	    FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,1X,F9.3,1X,
	1   F9.3,3X,4I2,5X,G16.8,1X,G16.8)
	  ELSE
C  Read the next object and when the end of the file is reached, return.
	    READ(21,1201,END=999) ID,COLUMN,ROW,INCLINA,ECCENTR,
	1   FWHMA,FWHMB,(IQ(JJ),JJ=1,4),PEAK,SKY
	  END IF
	ELSE IF (COMMAND.EQ.'F'.AND..NOT.ALL_OBJECTS) THEN
C  Read the header since the file was rewound when the next ID was prompted for
	  READ (21,1200)
	  DO WHILE (ID.NE.IDREAD)
	    READ(21,1201,END=22) IDREAD,COLUMN,ROW,INCLINA,ECCENTR,
	1   FWHMA,FWHMB,(IQ(JJ),JJ=1,4),PEAK,SKY
	  END DO
	  GO TO 23
  22	  CALL MESSAGE('Couldn''t find the object')
	  RETURN
  23	  CONTINUE
  	ELSE IF (COMMAND.EQ.'P') THEN
	  READ(21,1100,ERR=9991)
1100      FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',1X,'INCLINATION',
	1 1X,'ECCENTRICITY',1X,'RADIUS',5X,'ADU',3X,'LOG(INTEGRAL)')
	  IDREAD=' '
	  DO WHILE (ID.NE.IDREAD)
	    READ(21,1101,END=32) IDREAD,COLUMN,ROW,INCLINA,ECCENTR,R,
	1   ADU,LOGINTADU
	  END DO
	  IF (R.NE.0.0) CALL MESSAGE('Warning!  The file does not begin'//
	1 ' at the center of the star')
	  INDEXMINUSRADIUS=1.0-R
	  PARAM_PRF(1,1)=COLUMN
	  PARAM_PRF(2,1)=ROW
	  PARAM_PRF(3,1)=INCLINA
	  PARAM_PRF(4,1)=ECCENTR
	  PARAM_PRF(5,1)=ADU
	  GO TO 34
  32	  CALL MESSAGE('Couldn''t find the object')
	  RETURN
  34	  CONTINUE
	  NPTS=1
	  DO WHILE (ID.EQ.IDREAD)
	    READ(21,1101,END=42) IDREAD,COLUMN,ROW,INCLINA,ECCENTR,R,
	1   ADU,LOGINTADU
1101	    FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,1X,F7.2,1X,
	1   F13.6,1X,F11.8)
	    NPTS=NPTS+1
	    PARAM_PRF(1,NPTS)=COLUMN
	    PARAM_PRF(2,NPTS)=ROW
	    PARAM_PRF(3,NPTS)=INCLINA
	    PARAM_PRF(4,NPTS)=ECCENTR
	    PARAM_PRF(5,NPTS)=ADU
  	  END DO
	  NPTS=NPTS-1  ! Ignore the last number read
  42 	  CONTINUE
	  COLUMN=PARAM_PRF(1,NPTS)
	  ROW=PARAM_PRF(2,NPTS)
	  SKY=-999.999
	  CALL USRINP(SKY,1,'F',4,3,'SKY=',
	1 'Enter the sky to be subtracted from the data in the file:'//
	2 '  (default = minimum)')
	  IF (SKY.EQ.-999.999) THEN
	    SKY=PARAM_PRF(5,1)
	    DO I=1,NPTS
	      SKY=AMIN1(SKY,PARAM_PRF(5,I))
  	    END DO
	  END IF
	END IF
C  Get the surface brightness formula
C  Return point for more objects and COMMAND='M'
  49	IF (((COMMAND.EQ.'B'.OR.COMMAND.EQ.'F').AND.FIRST_TIME)
	1.OR.COMMAND.EQ.'M') THEN
	  CHOICE=' '
	  DO WHILE(CHOICE.NE.'G'.AND.CHOICE.NE.'W'.AND.CHOICE.NE.'S')
	    CHOICE='G'
	    CALL MESSAGE('Enter the surface brightness formula:')
	    CALL MESSAGE('     G = Gaussian  (default)')
	    CALL MESSAGE('     W = Gaussian with power law wings')
	    IF (COMMAND.EQ.'M') CALL MESSAGE('     S = Speckle images of a'//
	1   ' binary (gaussian profiles)')
	    CALL USRINP(%REF(CHOICE),1,'A',1,1,'CHOICE=',' ')
	  END DO
	END IF
C  Get the parameters of the surface brightness formulas
	IF (COMMAND.EQ.'B'.OR.COMMAND.EQ.'F') THEN
	  IF (CHOICE.EQ.'G') THEN
	    PARAMETRS(1)=PEAK-SKY
	    PARAMETRS(2)=FWHMA
C  COLUMN, ROW, INCLINA, and ECCENTR are set when the file is read
	  ELSE IF (CHOICE.EQ.'W') THEN
	    CALL MESSAGE('Sorry, not ready yet')
	  ELSE IF (CHOICE.EQ.'S') THEN
	    CALL MESSAGE('That choice is available only with command M')
	    RETURN
	  END IF
	ELSE IF (COMMAND.EQ.'M') THEN
	  IF (CHOICE.EQ.'G') THEN
	    CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1   'Enter the central surface brightness (ADU) and the'//
	2   ' FWHM (pixels):')
	  ELSE IF (CHOICE.EQ.'W') THEN
	    CALL MESSAGE('Sorry, not ready yet')
	    RETURN
C	    CALL MESSAGE('Enter the central surface brightness (ADU),
C	1 the FWHM (pixels),')
C	    CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
C	1   'and the power law scale (pixels):')
	  ELSE IF (CHOICE.EQ.'S') THEN
	    IF (NCOLS.EQ.320.AND.NROWS.EQ.512) THEN
	      CALL MESSAGE('A U of M formatted speckle file will be'//
	1     ' generated')
	      CALL MESSAGE('(ie., 32 64 X 64 images in a'//
	1     ' 320 X 512 file)')
	    END IF
	    CALL USRINP(SEEING,1,'F',4,0,'SEEING=',
	1   'Enter the FWHM (pixels) of the seeing:')
	    INCLIN_S=0.0
	    CALL USRINP(INCLIN_S,1,'F',4,1,'INCLINATION=',
	1   'Enter the ellipse inclination angle:  (default = 0.0'//
	2   ' degrees)')
	    ECCENT_S=0.0
	    CALL USRINP(ECCENT_S,1,'F',4,1,'ECCENTRICITY=',
	1   'Enter the eccentricity of the ellipse:  (default = 0.0)')
	    CALL MESSAGE('For the seeing disk')
	  END IF
	  CALL USRINP(POSITION,2,'F',4,1,'POSITION=',
	1 'Enter the column and row:  (default = last position,'//
	2 ' fractional values allowed)')
	  COLUMN=POSITION(1)
	  ROW=POSITION(2)
	  IF (CHOICE.EQ.'S') THEN
	    COLUMNS=COLUMN
	    ROWS=ROW
	    CALL MESSAGE('Enter the central surface brightness (ADU) of'//
	1   ' the primary, the FWHM of a ')
	    CALL USRINP(PARAMETRS,4,'F',4,0,'PARAMETERS=',
	1 'speckle, the ratio of the secondary to the primary, and'//
	2 ' the number of speckles:')
	    PRIMARY=PARAMETRS(1)
	    RATIO=PARAMETRS(3)
	    PARAMETRS(3)=0.0
	    NUMBER_OF_SPECKLES=PARAMETRS(4)
	    PARAMETRS(4)=0.0
	    CALL USRINP(OFFSET,2,'F',4,0,'OFFSET=',
	1   'Enter the X and Y offsets of the secondary from'//
	2   ' the primary:')
	    CALL USRINP(SEED,2,'I',4,0,'SEED=','Enter two large'//
	1   ' odd numbers to seed the random number generator')
	    CALL MESSAGE('for the individual speckles:')
	  END IF
	  INCLINA=0.0
	  CALL USRINP(INCLINA,1,'F',4,1,'INCLINATION=',
	1 'Enter the ellipse inclination angle:  (default = 0.0'//
	2 ' degrees)')
	  ECCENTR=0.0
	  CALL USRINP(ECCENTR,1,'F',4,1,'ECCENTRICITY=',
	1 'Enter the eccentricity of the ellipse:  (default = 0.0)')
	END IF
C  Convert from FWHM to a scaling parameter
	PARAMETRS(2)=PARAMETRS(2)/(2.0*SQRT(LOG(2.0)))
	SEEING=SEEING/(2.0*SQRT(LOG(2.0)))
C  If we are generating a speckle image, set the radius limit to something
C  larger than sigma, or else get the radius where the surface brightness is
C  one millionth of the central surface brightness.
	IF (CHOICE.EQ.'S') THEN
	  RADIUSLIMIT=PARAMETRS(2)*2.00
	ELSE
	  RADIUSLIMIT=PARAMETRS(2)*5.257
	END IF
C
	IF (FIRST_TIME) THEN
	  SCALE=1.0
	  CALL USRINP(SCALE,1,'F',4,1,'SCALE=',
	1 'Enter the intensity scale factor:  (default = 1.0)')
	END IF
	IF (COMMAND.EQ.'P') RADIUSLIMIT=NPTS-INDEXMINUSRADIUS+20
	IF (CHOICE.EQ.'S'.AND.NCOLS.EQ.320.AND.NROWS.EQ.512) THEN
	  IMAGE=0
	ELSE
	  IMAGE=31
	END IF
	DO WHILE (IMAGE.LT.32)
	  NUMSPEC=0
	  IMAGE=IMAGE+1
	  IF (NCOLS.EQ.320.AND.NROWS.EQ.512) THEN
	    LLO=1+JMOD((IMAGE-1),5)*64
	    MLO=449-((IMAGE-1)/5)*64
	  ELSE
	    LLO=0
	    MLO=0
	  END IF
	  DO WHILE (NUMSPEC.LT.NUMBER_OF_SPECKLES)
	    IF (CHOICE.EQ.'S') THEN
	      SUM1=0.0
	      SUM2=0.0
	      NUMSPEC=NUMSPEC+1
	      DO I=1,4
	        SUM1=SUM1+RAN(SEED(1))
	        SUM2=SUM2+RAN(SEED(2))
	      END DO
	      X=SQRT(2.0)*SEEING*(SUM1-2.0) !  ???
	      Y=SQRT(1.0-ECCENT_S**2)*SQRT(2.0)*SEEING*(SUM2-2.0)
	      COLUMN=COSD(INCLIN_S)*X-SIND(INCLIN_S)*Y+COLUMNS+LLO
	      ROW=SIND(INCLIN_S)*X+COSD(INCLIN_S)*Y+ROWS+MLO
	      IF (RATIO.NE.0.0) THEN
	        COLUMN2=COLUMN+OFFSET(1)
	        ROW2=ROW+OFFSET(2)
	      ELSE
	        COLUMN2=COLUMN
	        ROW2=ROW
	      END IF
	    ELSE
	      COLUMN2=COLUMN
	      ROW2=ROW
	    END IF
	    MNCOLUMN=JINT(MIN(COLUMN-RADIUSLIMIT,COLUMN2-RADIUSLIMIT))
	    IF (MNCOLUMN.LT.1) MNCOLUMN=1
	    IF (MNCOLUMN.GT.NCOLS) MNCOLUMN=NCOLS
	    MNROW=JINT(MIN(ROW-RADIUSLIMIT,ROW2-RADIUSLIMIT))
	    IF (MNROW.LT.1) MNROW=1
	    IF (MNROW.GT.NROWS) MNROW=NROWS
	    MXCOLUMN=JINT(MAX(COLUMN+RADIUSLIMIT,COLUMN2+RADIUSLIMIT))
	    IF (MXCOLUMN.LT.1) MXCOLUMN=1
	    IF (MXCOLUMN.GT.NCOLS) MXCOLUMN=NCOLS
	    MXROW=JINT(MAX(ROW+RADIUSLIMIT,ROW2+RADIUSLIMIT))
	    IF (MXROW.LT.1) MXROW=1
	    IF (MXROW.GT.NROWS) MXROW=NROWS
	    LENGTHX=MXCOLUMN-MNCOLUMN+1
	    LENGTHY=MXROW-MNROW+1
	    LENGTH=LENGTHX*LENGTHY*2
	    CALL DYNAMIC(ADDRESS1,LENGTH,'  R4',.TRUE.)
	    CALL DYNAMIC(ADDRESS2,LENGTH,'  R4',.TRUE.)
	    IF (CHOICE.EQ.'S') PARAMETRS(1)=PRIMARY
	    CALL FAKGAL1(LU_OUT,%VAL(ADDRESS1),%VAL(ADDRESS2),LENGTHX,
	1   LENGTHY,COMMAND,NCOLS,NROWS,COLUMN,COLUMNS_ABS,ROW,ROW_ABS,
	2   INDEXMINUSRADIUS,INDEXMINUSRADIUS_ABS,PARAM_PRF,PARAM_PRF_ABS,
	3   NPTS,NPTS_ABS,MNCOLUMN,MNROW,MXCOLUMN,MXROW,INCLINA,
	4   INCLINA_ABS,ECCENTR,ECCENTR_ABS,PARAMETRS,PARAMETRS_ABS,
	5   SKY,SKY_ABS,GRADPARAM,SCALE,CONVOLVE,RADIUSOUT,ARRAYOUT,
	6   ARRAYOUT_ABS,CHOICE,GRADIENT,OPTICALDEPTH)
	    IF (CHOICE.EQ.'S') THEN
	      IF (RATIO.NE.0.0) THEN
	        PARAMETRS(1)=RATIO*PRIMARY
	        COLUMN=COLUMN2
	        ROW=ROW2
	        CALL FAKGAL1(LU_OUT,%VAL(ADDRESS1),%VAL(ADDRESS2),LENGTHX,
	1       LENGTHY,COMMAND,NCOLS,NROWS,COLUMN,COLUMNS_ABS,ROW,ROW_ABS,
	2       INDEXMINUSRADIUS,INDEXMINUSRADIUS_ABS,PARAM_PRF,PARAM_PRF_ABS,
	3       NPTS,NPTS_ABS,MNCOLUMN,MNROW,MXCOLUMN,MXROW,INCLINA,
	4       INCLINA_ABS,ECCENTR,ECCENTR_ABS,PARAMETRS,PARAMETRS_ABS,
	5       SKY,SKY_ABS,GRADPARAM,SCALE,CONVOLVE,RADIUSOUT,ARRAYOUT,
	6       ARRAYOUT_ABS,CHOICE,GRADIENT,OPTICALDEPTH)
	      END IF
	    ELSE
	      NUMSPEC=NUMBER_OF_SPECKLES
	      IMAGE=32
	    END IF
	    CALL DYNAMIC(ADDRESS1,LENGTH,'  R4',.FALSE.)
	    CALL DYNAMIC(ADDRESS2,LENGTH,'  R4',.FALSE.)
	  END DO
	END DO
	FIRST_TIME=.FALSE.
	IF (ALL_OBJECTS) THEN
	  ALL_OBJECTS_FIRST_TIME=.FALSE.
	  IF (COMMAND.EQ.'B'.OR.COMMAND.EQ.'F') GO TO 20
	ELSE IF (.NOT.ALL_OBJECTS) THEN
	  MORE=.TRUE.
	  CALL USRINP(MORE,1,'L',4,1,'MORE=',
	1 'More stars to model?  (default = Y)')
	  IF (MORE) THEN
	    IF (COMMAND.EQ.'B'.OR.COMMAND.EQ.'F'.OR.COMMAND.EQ.'P') GO TO 12
	    IF (COMMAND.EQ.'M') GO TO 49
	  END IF
	END IF
 999	CLOSE(21)
	CALL CLOSEFITS(LU_OUT)
	RETURN
9991	CALL ERROR(-3,'Couldn''t read the header') 
	END
