	SUBROUTINE ANNULUS2(UNIT)
C
C       This subroutine helps ANNULUS, DISPANN, and PROFILE to allow display
C  of elliptical galaxy and stellar data files on a graphics terminal and/or 
C  other devices contained within the plot package.  The average surface
C  brightness and integrated average surface brightness within an annulus are 
C  plotted against the semi-major axis.  Both linear and log-log plots are
C  possible.  If UNIT=0 the name of the file is asked for;  if UNIT>0
C  it is assumed that the file has already been opened 
C  on the unit given as the argument.  The entry PROFILE2 is called by
C  PROFILE and DISPANN to allow display of the above quantities plus the
C  X and Y central locations, the inclination, and the eccentricity as a
C  function of the semi-major axis.
C			Donald H. Gudehus, August, 1982
C			Version 1U, Donald H. Gudehus, January 31, 2008
C			Converted to Unix
C
	INCLUDE 'miipspar.for'
C
	BYTE BCNTRLJ
	CHARACTER*1 CNTRLJ
	EQUIVALENCE (BCNTRLJ,CNTRLJ)
C
	CHARACTER*6 ID
	COMMON /ID/ ID
C
	CHARACTER*6 IDLAST
	CHARACTER*50 XLABEL,YLABEL,TITLE
C
	INTEGER*4 STATUS,UNIT,DUMUNIT
	INTEGER*4 QUITTEST
	INTEGER*4 COMMAND(25)
	INTEGER COLOR
C
	LOGICAL OFIL,MORE,EXTRA,INVERT(2),CROSSHAIRS,INTERACT,
	1PLOTDEVQUES
C
	REAL*4 PARAM_PRF(MAXDIAG,7),DAT(MAXDIAG),RADIUS(MAXDIAG),
	1XARRAY(MAXDIAG+4),YARRAY(MAXDIAG+4)
	REAL*4 RADII(2),INDEXMINUSRADIUS
	COMMON /REAL4_SCRATCH_BLOCK/ PARAM_PRF,DAT,RADIUS,XARRAY,
	1YARRAY
C
	EXTRA=.FALSE.
	GO TO 1
	ENTRY PROFILE2(UNIT)
	EXTRA=.TRUE.
  1	DUMUNIT=UNIT
	IF (UNIT.EQ.0) THEN
	  CALL MESSAGE('Open the data file')
	  CALL OPENCF(21,1,'FILE_21=',0)
	  DUMUNIT=21
	END IF
	IDLAST=' '
	BCNTRLJ='0A'X !  Sub- and superscript
C
  2	CONTINUE
	IF (UNIT.EQ.0) THEN
	  CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1 'Enter the ID of the object:  (default = last entry)')
	  CALL ALPHA(%REF(ID),1,6,LX,NF,NR,2)
	END IF
	COMMAND(1)=1
	DO I=2,25
	  J=ISETEND(COMMAND(I))
	END DO
	CALL MESSAGE('Enter one or more commands:')
	CALL MESSAGE('     1 = Displays average ADU vs. '//
	1'radius (default)')
	CALL MESSAGE('     2 = Displays log(average ADU) vs. '//
	1'log(radius)')
	CALL MESSAGE('     3 = Displays log(average ADU) vs. '//
	1'1/4 power of radius')
	CALL MESSAGE('     4 = Displays log(integrated average ADU) '//
	1'vs. log(radius)')
	IF (EXTRA) THEN
	  QUITTEST=9
	  CALL MESSAGE('     5 = Displays X of center vs. radius')
	  CALL MESSAGE('     6 = Displays Y of center vs. radius')
	  CALL MESSAGE('     7 = Displays inclination vs. radius')
	  CALL MESSAGE('     8 = Displays eccentricity vs. radius')
	  CALL MESSAGE('     9 = Quits')
	ELSE
	  QUITTEST=5
	  CALL MESSAGE('     5 = Quits')
	END IF
    	CALL USRINP(COMMAND,25,'I',4,3,'COMMAND(S)=',' ')
	NUMCOMMAND=INEL(COMMAND,25)
	DO I=1,NUMCOMMAND
	  IF (COMMAND(I).LT.1.OR.COMMAND(I).GT.QUITTEST) THEN
	    CALL MESSAGE('Invalid command number encountered')
	    GO TO 2
	  END IF
	END DO
	IF (COMMAND(1).EQ.QUITTEST) GO TO 999 !  Returns to calling program
C	IF (UNIT.NE.0.AND.COMMAND(NUMCOMMAND).NE.QUITTEST) THEN
C	PRINT *, '#2'
C	  CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
C	1 'Enter the ID of the object:  (default = last entry)')
C	  CALL ALPHA(%REF(ID),1,6,LX,NF,NR,2)
C	END IF
	IF (ID.NE.IDLAST) THEN
	  CALL RDDATFL1(DUMUNIT,ID,PARAM_PRF,MAXDIAG,NPTS,
	1 INDEXMINUSRADIUS,DELTARAD)
	END IF
	IDLAST=ID
	SKYDONE=.FALSE.
	DO I=1,NUMCOMMAND
	  IF (COMMAND(I).EQ.QUITTEST) THEN
	    MORE=.FALSE.
	    GO TO 999
	  END IF
	  IF (COMMAND(I).EQ.2.OR.COMMAND(I).EQ.4) THEN
	    PARAM_PRF(1,5)=0.46
	    CALL USRINP(PARAM_PRF(1,5),1,'F',4,1,'ZERO_RADIUS=',
	1   'Enter a substitute zero radius:  (default = 0.46)')
	  END IF
	  IF (COMMAND(I).EQ.2.OR.COMMAND(I).EQ.3) THEN
	    SKY=PARAM_PRF(1,6)
	    DO II=1,NPTS
	      SKY=MIN(SKY,PARAM_PRF(II,6))
	    END DO
	    WRITE (6,1400) SKY
1400	    FORMAT(' ','The minimum surface brightness is ',G18.7,'.')
	    WRITE (7,1400) SKY
	    CALL USRINP(SKY,1,'F',4,1,'SKY=','Enter a sky value:  '//
	1   '(default = above minimum)')
	  END IF
	  NPTS_WORKING=NPTS
	  DO II=1,NPTS
	    IF (COMMAND(I).EQ.2.OR.COMMAND(I).EQ.4) THEN
	      RADIUS(II)=LOG10(PARAM_PRF(II,5))
	    ELSE IF (COMMAND(I).EQ.3) THEN
	      RADIUS(II)=PARAM_PRF(II,5)**0.25
	    ELSE
	      RADIUS(II)=PARAM_PRF(II,5)
	    END IF
	    IF (COMMAND(I).EQ.1) DAT(II)=PARAM_PRF(II,6)
	    IF (COMMAND(I).EQ.2.OR.COMMAND(I).EQ.3) THEN
	      IF (PARAM_PRF(II,6)-SKY.GT.0.0) THEN
	        DAT(II)=LOG10(PARAM_PRF(II,6)-SKY)
	      ELSE
	        NPTS_WORKING=II-1
	        GO TO 22
	      END IF
	    END IF
	    IF (COMMAND(I).EQ.4) DAT(II)=PARAM_PRF(II,7)
	    IF (COMMAND(I).EQ.5) DAT(II)=PARAM_PRF(II,1)
	    IF (COMMAND(I).EQ.6) DAT(II)=PARAM_PRF(II,2)
	    IF (COMMAND(I).EQ.7) DAT(II)=PARAM_PRF(II,3)
	    IF (COMMAND(I).EQ.8) DAT(II)=PARAM_PRF(II,4)
	  END DO
  22	  IF (I.GT.1.AND.PLOTDEVQUES('G*',1)) CALL TEKPAU(0.0,4.0,0)
  25	  RADII(1)=PARAM_PRF(1,5)
	  RADII(2)=PARAM_PRF(NPTS,5)
	  WRITE (6,1500) RADII(1),RADII(2)
1500	  FORMAT(' ','The minimum and maximum radii are ',F9.3,' and ',
	1 F9.3,'.')
	  WRITE (7,1500) RADII(1),RADII(2)
	  CALL MESSAGE('Enter the start radius and end radius for '//
	1 'plotting:')
	  CALL USRINP(RADII,2,'F',4,1,'RADII=',
	1 '(default = entire range)')
	  IF (NPTS_WORKING.LT.NPTS) CALL MESSAGE('Maximum radius was '//
	1 'truncated')
	  IF (RADII(2).GT.(NPTS-1)*DELTARAD+1.0-INDEXMINUSRADIUS)
	1 GO TO 25
	  DO II=1,NPTS_WORKING
	    IF (PARAM_PRF(II,5).LE.RADII(1)) IXS=II
	    IF (PARAM_PRF(II,5).LE.RADII(2)) IXF=II
	  END DO
C  Load the plotting arrays
	  NPLPTS=IXF-IXS+1
	  DO II=1,NPLPTS
	    XARRAY(II)=RADIUS(II+IXS-1)
	    YARRAY(II)=DAT(II+IXS-1)
	  END DO	  
C  Initialize the ends of the arrays for automatic scaling each time around
	  DO IR=0,3
	    XARRAY(MAXDIAG-IR)=0.0
	    YARRAY(MAXDIAG-IR)=0.0
	  END DO
	  NPLOTS=1
	  LINVECT=0
	  INC=1
	  LINTYPE=0
	  NSYMBOL=0
	  INVERT(1)=.FALSE.
	  INVERT(2)=.FALSE.
	  COLOR=4080  ! Yellow
	  INTERACT=.FALSE.
	  CROSSHAIRS=.TRUE.
	  XLABEL='RADIUS (pixels)'
	  TITLE=ID
	  IF (COMMAND(I).EQ.1) THEN
	    YLABEL='SURFACE BRIGHTNESS (ADU/sq. pixel)'
	  ELSE IF (COMMAND(I).EQ.2) THEN
	    XLABEL='log(RADIUS (pixels))'
	    YLABEL='log(SURFACE BRIGHTNESS (ADU))'
	  ELSE IF (COMMAND(I).EQ.3) THEN
	    XLABEL='RADIUS'//CNTRLJ//CNTRLJ//'1/4'//CNTRLJ//' (pixels'//
	1   CNTRLJ//CNTRLJ//'1/4'//CNTRLJ//')'
	    YLABEL='log(SURFACE BRIGHTNESS (ADU))'
	  ELSE IF (COMMAND(I).EQ.4) THEN
	    XLABEL='log(RADIUS (pixels))'
	    YLABEL='log(INTEGRATED SURFACE BRIGHTNESS)'
	  ELSE IF (COMMAND(I).EQ.5) THEN
	    YLABEL='X POSITION (pixels)'
	  ELSE IF (COMMAND(I).EQ.6) THEN
	    YLABEL='Y POSITION (pixels)'
	  ELSE IF (COMMAND(I).EQ.7) THEN
	    YLABEL='INCLINATION (degrees)'
	  ELSE IF (COMMAND(I).EQ.8) THEN
	    YLABEL='ECCENTRICITY'
	  END IF
	  CALL XYPLOT(XARRAY,YARRAY,MAXDIAG,NPLOTS,NPLPTS,LINVECT,INC,
	1 LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,
	2 YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
 	END DO
 999	MORE=.FALSE.
	IF (COMMAND(1).NE.QUITTEST) THEN
	  CALL USRINP(MORE,1,'L',1,1,'MORE=',
	1 'More objects to plot?  (default = N)')
	END IF
	IF (MORE) GO TO 2
	IF (UNIT.EQ.0) CLOSE(DUMUNIT)
	CALL PLOT(0.0,0.0,9999)
	RETURN
	END
