	SUBROUTINE CENTROID(DAT,IXDIM,IYDIM,NX,NY,BIAS,X,Y,ITOT)
C
C       This subroutine computes the centroid in the X and Y directions.
C  If a pixel has been edited (value of 0) or if its value is less than
C  BIAS, that pixel is ignored.  If all the pixel values are zero then 
C  X and Y are returned as zero.  If 
C
C	DAT	(C)	The array containing the data
C	IXDIM	(C)	The dimension of DAT in the X direction
C	IYDIM	(C)	The dimension of DAT in the Y direction
C	NX	(C)	The number of points to be used in the 
C			X direction
C	NY	(C)	The number of points to be used in the 
C			Y direction
C	BIAS	(C)	The pixel intensity value below which, information
C			is ignored.
C	X	(R)	The location of centroid in the X direction
C	Y	(R)	The location of the centroid in the Y direction.
C			X and Y are set equal to 0 if no values above BIAS
C			exist.
C	ITOT	(R)	The total number of points used in the
C			determination
C
C				Susan M. Simkin, July, 1981
C
C			Version 2, Donald H. Gudehus, October, 1982
C			Fixed up so that the correct fractional pixel
C			location is given, added double precision, and
C			added documentation.
C			Version 3, Donald H. Gudehus, March, 1989
C			Inserted calculation of centroid for single nonzero
C			point
C
	DIMENSION DAT(IXDIM,IYDIM)
	REAL*8 SUMX,SUMY,TOT
C
	ITOT=0
	TOT=0.0
	SUMX=0.0
	SUMY=0.0
	DO M=1,NY
	  DO L=1,NX
	    DATP=DAT(L,M)-BIAS
C  Exclude edited areas (DAT=0 or DATP<0).  If there is only one nonzero point,
C  the average would equal BIAS.  In that case BIAS=DAT, DATP=0, and the 
C  point would not be excluded here, but TOT would equal 0.  We handle that
C  case separately below.
	    IF (DATP.GE.0.0) THEN
	      SUMX=SUMX+DATP*FLOAT(L)
	      SUMY=SUMY+DATP*FLOAT(M) 
	      TOT=TOT+DATP
	      ITOT=ITOT+1
	    END IF
	  END DO
	END DO
	IF (ITOT.EQ.1) THEN
	  DO M=1,NY
	    DO L=1,NX
	      IF (DAT(L,M)-BIAS.GE.0.0) THEN
	        SUMX=FLOAT(L)
	        SUMY=FLOAT(M) 
	        TOT=1.0
	        GO TO 25
	      END IF
	    END DO
	  END DO
	END IF
  25	IF (TOT.EQ.0.0) THEN
	  X=0.0
	  Y=0.0
	ELSE
	  X=SUMX/TOT+0.5
	  Y=SUMY/TOT+0.5
	END IF
	END
