	SUBROUTINE GET_LU_FROM_SAO(LU,NPIC,STATUS)
C
C	This subroutine obtains a logical unit for a FITS file that has
C  been displayed on SAOimage.  The picture number of the image on SAOimage
C  is used to look up the name of the file that was displayed.  The fitsfacts
C  common blocks are then searched to determine if the file is open or closed
C  (usual case).  If the file is open, then the current logical unit is returned.
C  If the file is closed, the file is opened with openfits in quiet mode, and the
C  logical unit returned.  In addition the map number and plane number are set
C  in their common blocks.
C
C			Donald H. Gudehus, February 5, 2008
C
C	LU	(R)	The logical unit corresponding to the selected image
C	NPIC	(C)	The picture number in SAOimage.  Up to 16 are allowed.
C	STATUS	(R)	The returned status.  Success is 0;  failure is 1;
C			and error is 2.
C
	INCLUDE 'saofacts.for'
	INCLUDE 'fitsfacts.for'
C
	INTEGER STATUS
C
	CHARACTER*3 PRGFMAT
	CHARACTER*40 FILEKEYWORD
	CHARACTER*80 FILENAME
C
	INTEGER PLANEKEY
C
	LOGICAL IOFL
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	STATUS=0
	IF (NPIC.LT.1.OR.NPIC.GT.16) THEN
	  CALL ERROR(-3,'Invalid picture number in  GET_LU_FROM_SAO')
	  STATUS=2
	  RETURN
	END IF
	FILENAME=TV_FILENAME(NPIC)
C  See if there is an assigned logical unit number to the name.  Usually
C  when an option is quit, the file is closed, and it's name is removed from
C  the array in fitsfacts, so this search will probably be unsuccessful.
	DO I=LUMIN,LUMAX
	  IF (FITSFILENAME(I).EQ.FILENAME) THEN
	    LU=I
	    IMNUM=MHED_SAO(1,NPIC)
	    IPNUM=MHED_SAO(2,NPIC)
	    RETURN
	  END IF
	END DO
C  Find an unused logical unit number and reopen the file.
	DO I=LUMIN,LUMAX
	  IF (FITSFILENAME(I).EQ.' ') THEN
	    LU=I
	    IOFL=.TRUE.
	    FILEKEYWORD=' '
	    MAPKEY=MHED_SAO(1,NPIC)
	    PLANEKEY=MHED_SAO(2,NPIC)
	    NMAPS=0
	    NPLNS=0
	    NCOLS=0
	    NROWS=0
	    PRGFMAT=' '
	    CALL OPENFITS(LU,IOFL,FILEKEYWORD,FILENAME,MAPKEY,PLANEKEY,
	1   NMAPS,NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
C  Get the map number and the plane number
	    IMNUM=MHED_SAO(1,NPIC)
	    IPNUM=MHED_SAO(2,NPIC)
	    RETURN
	  END IF
	END DO
	STATUS=1 !  Failure
	RETURN
	END
