	SUBROUTINE INTERP1D(ARRAY1,ARRAY2,NPOINTS,VALUE1,VALUE2)
C
C       This subroutine linearly interpolates to find a value within the
C  range of ARRAY2 corresponding to the value chosen within the range of
C  ARRAY1.  Each value at a given index in ARRAY1 corresponds to the value
C  in ARRAY2.  The interval between values is allowed to be arbitrary,
C  however, it is assumed that the values in ARRAY1 are
C  monotonically increasing or decreasing.
C        A common error in the use of this subroutine is not having
C  ARRAY1 completely filled.  When calling this subroutine, be sure
C  that NPOINTS is adjusted so that meaningful data is at every 
C  array location.
C					Donald H. Gudehus, May, 1984
C
C	NPOINTS	(C)	The dimension of ARRAY1 and ARRAY2 and the
C			number of valid data points.
C	VALUE1	(C)	The known value within the range of ARRAY1
C	VALUE2	(R)	The requested value within the range of ARRAY2
C
	DIMENSION ARRAY1(NPOINTS),ARRAY2(NPOINTS)
C
	IF (ARRAY1(NPOINTS).GT.ARRAY1(1)) THEN  ! Monotonically increasing
	  I=1
	  DO WHILE (ARRAY1(I).LE.VALUE1.AND.I.LE.NPOINTS)
	    I=I+1
	  END DO
	ELSE  ! Monotonically decreasing
	  I=1
	  DO WHILE(ARRAY1(I).GE.VALUE1.AND.I.LE.NPOINTS)
	    I=I+1
	  END DO
	END IF
	I=I-1
	IF (ARRAY1(I).EQ.VALUE1) THEN
	  VALUE2=ARRAY2(I)
	  RETURN
	ELSE
	  IF (I.LT.NPOINTS) THEN
	    VALUE2=ARRAY2(I)+(VALUE1-ARRAY1(I))/(ARRAY1(I+1)-
	1   ARRAY1(I))*(ARRAY2(I+1)-ARRAY2(I))
	  ELSE
	    VALUE2=ARRAY2(I)+(VALUE1-ARRAY1(I))/(ARRAY1(I)-
	1   ARRAY1(I-1))*(ARRAY2(I)-ARRAY2(I-1))
	  END IF
	END IF
C
	RETURN
	END
