	SUBROUTINE LINFIT1D(X,DATA,WEIGHT,NPOINTS,DEGREE,
	1CONSTANTTERMS,COEFF,COVARIANCE,MAXNUMPARAM,MEANERROR,
	2XPLOT,YPLOT,NPLPOINTS)
C
C       This subroutine performs a linear least squares fit of a 
C  polynomial to weighted data and returns the parameters of the fit and 
C  their errors.  If desired, certain coefficients may be held constant
C  in the solution.  There is no limit on the degree of the polynomial.
C  For plotting purposes, an array is filled with calculated values
C  corresponding to the values in the array XPLOT.  For spline fits,
C  this is the only meaningful returned value.  To prevent
C  a page owner violation when the user is prompted for the degree
C  (DEGREE=-1 in the call), the calling program should either contain a 
C  call to USRINP or open units 7 and 49.
C
C				Donald H. Gudehus, August, 1984
C
C	X	      (C) An array containing the values of the
C			  independent variable
C	DATA	      (C) An array containing the observations to
C			  be fitted
C	WEIGHT	      (C) An array for the weights of each point
C	NPOINTS	      (C) The dimension of the above three arrays
C			  and the number of points to be fit
C	DEGREE	    (C,R) The degree of the polynomial fit if .GE. 0.
C			  If DEGREE=-1, the degree of the polynomial fit is
C			  asked for;  if DEGREE=-2, the degree of the
C			  polynomial fit and the coefficients which are to
C			  be held constant are asked for;  if DEGREE=-3,
C			  the type of fit is asked for and if it is to be a
C			  polynomial, the degree and coefficients which are
C			  to be held constant are asked for;  if DEGREE=-99,
C			  a spline fit is done.  
C	CONSTANTTERMS (C) An array which is .TRUE. if that coefficient
C			  is to be held constant and not solved for.
C	COEFF	    (C/R) An array containing the coefficients of the
C			  polynomial of the fit.
C	COVARIANCE    (R) An array containing the covariances
C	MAXNUMPARAM   (C) The maximum number of parameters to be used
C			  in the calling program and the dimensions of
C			  COEFF and COVARIANCE in the calling program.
C	MEANERROR     (R) The mean error for an observation of
C			  average weight
C	XPLOT	      (C) An array containing points to be evaluated
C			  with the fitted function
C	YPLOT	      (R) An array containing the evaluated points from
C			  XPLOT
C	NPLPOINTS     (C) The dimension of the above two arrays and the
C			  number of points contained therein.
C
	INTEGER DEGREE,ADDRESS1,ADDRESS2,ADDRESS3,ADDRESS4,
	2ADDRESS5,ADDRESS6,ADDRESS7
	LOGICAL CONSTANTTERMS(MAXNUMPARAM)
	REAL XPLOT(NPLPOINTS),YPLOT(NPLPOINTS)
	REAL*8 X(NPOINTS),DATA(NPOINTS),WEIGHT(NPOINTS),
	1COEFF(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR
C
C  Create some dynamic memory
	LENGTH1=MAXNUMPARAM  ! NCONSTANT
	LENGTH2=MAXNUMPARAM  ! ISHIFT
	LENGTH3=MAXNUMPARAM  ! KPVT
	LENGTH4=MAXNUMPARAM  ! DUMCOEFF
	LENGTH5=MAXNUMPARAM*MAXNUMPARAM  ! LEFTSIDE
	LENGTH6=MAXNUMPARAM*(MAXNUMPARAM+1)/2  ! UPTRIPART
	LENGTH7=MAXNUMPARAM-1  ! AUX
	CALL DYNAMIC(ADDRESS1,LENGTH1,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS2,LENGTH2,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS3,LENGTH3,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS4,LENGTH4,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS5,LENGTH5,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS6,LENGTH6,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS7,LENGTH7,'  R8',.TRUE.)
	CALL LINFIT1D1(X,DATA,WEIGHT,NPOINTS,DEGREE,
	1CONSTANTTERMS,COEFF,COVARIANCE,MAXNUMPARAM,MEANERROR,
	2XPLOT,YPLOT,NPLPOINTS,%VAL(ADDRESS1),%VAL(ADDRESS2),
	3%VAL(ADDRESS3),%VAL(ADDRESS4),%VAL(ADDRESS5),%VAL(ADDRESS6),
	4%VAL(ADDRESS7))
	CALL DYNAMIC(ADDRESS1,LENGTH1,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS2,LENGTH2,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS3,LENGTH3,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS4,LENGTH4,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS5,LENGTH5,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS6,LENGTH6,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS7,LENGTH7,'  R8',.FALSE.)
	RETURN
	END
