	SUBROUTINE NLDER10(X,MAXNPOINTS,DIMENSN_IND,PARAMETRS,MAXNUMPARAM,
	1CHOICE,CONSTANTTERMS,OPERATE,IX,FUNCTN,DIMENSN_DATA,DERIV)
C
C       This subroutine helps NONLINFIT by selecting the chosen derivative for a
C  fitting function for a row of a SAD file or for an array of data, 
C  evaluating it taking into account a user defined operation if necessary, 
C  and returning the answer.  This subroutine is called by the SADFIT and
C  related subroutines (SADFIT_POWER_LAW and SADFIT_SPECTRUM), by FAKGAL1
C  and FAKGAL3, by PROBABLE2D, and by SUBSTAR.  If it is to be
C  used for other options, pay special attention to the use of COMMON blocks.
C  SCRATCH_BLOCK and SCRATCH_BLOCK2 are not available.
C			Donald H. Gudehus, January, 1987
C			Version 2, Donald H. Gudehus, October, 1989
C			Fixed error with CHOICE='G' and added CHOICE='GP'
C			Version 3, Donald H. Gudehus, February 18, 2008
C			Modified for FITS
C
C	X		(C)	The array containing the independent variable
C				values
C	MAXNPOINTS	(C)	The first dimension of the array X
C	DIMENSN_IND	(C)	The second dimension of the array X
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	CHOICE		(C)	The choice determining which function
C				to use within a category
C	CONSTANTTERMS	(C)	The terms which are to be held constant in the
C				solution.  Time is saved if these derivatives
C				are not calculated.
C	OPERATE		(C)	A logical key which determines what to do.
C				If .FALSE., the function is evaluated, if 
C				.TRUE., a user defined operation is carried
C				out, e.g., convolution.
C	IX		(C)	An array containing the starting and ending
C				indexes for the evauation of DERIV
C	FUNCTN		(C)	The output array for the function.
C	DIMENSN_DATA	(C)	The second dimension of the array FUNCTN
C	DERIV		(R)	The ouput array containing the derivative
C
	CHARACTER*(*) CHOICE
	CHARACTER*6 DUMCHOICE
	INTEGER DIMENSN_IND,DIMENSN_DATA,IX(2)
	DIMENSION PARAMETRS(MAXNUMPARAM),X(MAXNPOINTS,DIMENSN_IND),
	1FUNCTN(MAXNPOINTS,DIMENSN_DATA),
	2DERIV(MAXNPOINTS,MAXNUMPARAM,DIMENSN_DATA)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),OPERATE
	REAL*8 X,PARAMETRS,FUNCTN,DERIV
C
	REAL*8 TEMP
C
	INTEGER DEGREE2
	COMMON /FIT_SPECTRUM_BLOCK/ DEGREE2,NLINES
C
	IF (CHOICE.EQ.'G') THEN !  Gaussian
	  DO N=IX(1),IX(2)
	    DERIV(N,1,1)=(FUNCTN(N,1)-PARAMETRS(4))/PARAMETRS(1)
	    DERIV(N,2,1)=((X(N,1)-PARAMETRS(3))**2/PARAMETRS(2)**3)*
	1   (FUNCTN(N,1)-PARAMETRS(4))
	    DERIV(N,3,1)=((X(N,1)-PARAMETRS(3))/PARAMETRS(2)**2)*
	1   (FUNCTN(N,1)-PARAMETRS(4))
	    DERIV(N,4,1)=1.0
	  END DO
	ELSE IF (CHOICE.EQ.'I') THEN
	  DO N=IX(1),IX(2)
	    DERIV(N,1,1)=(FUNCTN(N,1)-PARAMETRS(3))/PARAMETRS(1)
	    DERIV(N,2,1)=(FUNCTN(N,1)-PARAMETRS(3))*LOG(X(N,1))
	    DERIV(N,3,1)=1.0
	  END DO
	ELSE IF (CHOICE.EQ.'GP') THEN !  Gaussian line profiles and polynomial
				      !  continuum
C  Calculate the derivative of the continuum by proceeding from parameter to
C  parameter
	  DO NP=1,DEGREE2+1
	    DO N=IX(1),IX(2)
C  The derivative calculation is different depending on whether an absorption
C  line or an emission line is present at the given wavelength location.
C  In order determine which case it is, we check whether FUNCTN is
C  greater than the continuum (emission) or less than it (absorption).  This
C  method would yield ambiguous results in the region of overlap of an emission
C  line and an absorption line, but because EXP(F) is approximately 1+F when
C  F is small (in the wings), little difficulty should arise.  Another approach
C  would be to compute the derivative numerically.
	      IF (FUNCTN(N,1).GE.POL(PARAMETRS,DEGREE2,X(N,1))) THEN  ! Emission
	        DERIV(N,NP,1)=X(N,1)**(NP-1)
	      ELSE  ! Absorption
	        DERIV(N,NP,1)=X(N,1)**(NP-1)*
	1       FUNCTN(N,1)/POL(PARAMETRS,DEGREE2,X(N,1))
	      END IF
	    END DO
	  END DO
C  Calculate the derivatives of the modeled lines by proceeding from line
C  to line
	  DO K=1,NLINES
C  Calculate the parameter numbers
	    N0=DEGREE2-1+K*3 !  The amplitude (emission) or optical depth
			     !  (absorption)
	    N1=N0+1 !  The sigma
	    N2=N0+2 !  The offset in position
	    DO N=IX(1),IX(2)
	      TEMP=(X(N,1)-PARAMETRS(N2))/PARAMETRS(N1)
	      IF (ABS(TEMP).GT.13.0) THEN
		 !  Don't do anything
	        DERIV(N,N0,1)=0.0
	        DERIV(N,N1,1)=0.0
	        DERIV(N,N2,1)=0.0
	      ELSE
	        IF (PARAMETRS(N0).GE.0.0) THEN !  Emission line
	          DERIV(N,N0,1)=1.0/EXP(0.5*TEMP**2)
	          DERIV(N,N1,1)=PARAMETRS(N0)*DERIV(N,N0,1)*(TEMP**2)/
	1         PARAMETRS(N1)
	          DERIV(N,N2,1)=PARAMETRS(N0)*DERIV(N,N0,1)*
	1         (X(N,1)-PARAMETRS(N2))/PARAMETRS(N1)**2
	        ELSE  ! Absorption line
	          DERIV(N,N0,1)=FUNCTN(N,1)/EXP(0.5*TEMP**2)
	          DERIV(N,N1,1)=DERIV(N,N0,1)*PARAMETRS(N0)*
	1         TEMP**2/PARAMETRS(N1)
	          DERIV(N,N2,1)=DERIV(N,N0,1)*PARAMETRS(N0)*
	1         TEMP/PARAMETRS(N1)
	        END IF
	      END IF
	    END DO
	  END DO
	END IF
C
	RETURN
	END
