	SUBROUTINE NLIT(FIRSTIME,PARAMETRS_SOLV,PARAMETRS_INIT,PARAMETRS_PRIOR,
	1PARAMETRS_BEST,NUMPARAM_SOLV,MAXNUMPARAM,AVREDCHISQ,SIGMAPARAM,
	2AVREDCHISQ_INIT,TOLERANCE,FRACINCR,ITERATION_SCHEME,CHOICE,
	3INTERMEDIATE,ITERATE)
C
C       This subroutine helps NONLINFIT by determining when to stop
C  iterating.  It calculates the value of the fractional increment based 
C  on the relative change in the sigma of the fit and readjusts the 
C  parameters to a value weighted by their former value if the sigma 
C  increases.
C			Donald H. Gudehus, March, 1984
C			Version 2, Donald H. Gudehus, December 1, 1994
C			Introduced ZERO_TEST
C			Version 3, Donlad H. Gudehus, February 26, 2004
C			Added section for AVREDCHISQ.EQ.AVREDCHISQ_PRIOR
C			and added COMMON block NLIT_REAL8 to ensure that
C			AVREDCHISQ_PRIOR retains its value upon successive
C			calls to this subroutine.  Improved the searching
C			logic.
C
	DIMENSION PARAMETRS_SOLV(MAXNUMPARAM),PARAMETRS_INIT(MAXNUMPARAM),
	1PARAMETRS_PRIOR(MAXNUMPARAM),PARAMETRS_BEST(MAXNUMPARAM),
	2SIGMAPARAM(MAXNUMPARAM)
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*6 CHOICE
	INTEGER*4 N_ITERATIONS
	LOGICAL FIRSTIME,INTERMEDIATE,ITERATE,ZERO_TEST
	REAL*8 PARAMETRS_SOLV,PARAMETRS_INIT,SIGMAPARAM,PARAMETRS_PRIOR,
	1PARAMETRS_BEST,AVREDCHISQ,AVREDCHISQ_INIT,AVREDCHISQ_PRIOR,
	2AVREDCHISQ_BEST

	COMMON /NLIT_REAL8/AVREDCHISQ_PRIOR
C
	IF (FIRSTIME) THEN
	  AVREDCHISQ_PRIOR=AVREDCHISQ_INIT
	  AVREDCHISQ_BEST=AVREDCHISQ_INIT
	  DO I=1,NUMPARAM_SOLV
	    PARAMETRS_PRIOR(I)=PARAMETRS_INIT(I)
	    PARAMETRS_BEST(I)=PARAMETRS_INIT(I)
	  END DO
	  N_ITERATIONS=0
	END IF
	IF (ITERATION_SCHEME.EQ.'A'.OR.ITERATION_SCHEME.EQ.'B'.OR.
	1ITERATION_SCHEME.EQ.'C') THEN
	  ITERATE=.FALSE.
	  DO I=1,NUMPARAM_SOLV
	    IF (ABS(PARAMETRS_SOLV(I)-PARAMETRS_PRIOR(I)).GE.
	1   (ABS(PARAMETRS_SOLV(I))+ABS(SIGMAPARAM(I)))*TOLERANCE)
	2   ITERATE=.TRUE.
	  END DO
C  If the parameters are actually zero and the fit is perfect, the above
C  test will fail.  Therefore check the sigmas.
	  ZERO_TEST=.TRUE.
	  DO I=1,NUMPARAM_SOLV
	   IF (ZERO_TEST.AND.SIGMAPARAM(I).NE.0.0) ZERO_TEST=.FALSE.
	  END DO
	  IF (ZERO_TEST) ITERATE=.FALSE.
C
	  IF (INTERMEDIATE.AND..NOT.ITERATE) 
	1 CALL MESSAGE('Solution has converged')
	END IF
	DO I=1,NUMPARAM_SOLV
	  PARAMETRS_PRIOR(I)=PARAMETRS_SOLV(I)
	END DO
C
	IF (AVREDCHISQ.LT.AVREDCHISQ_BEST) THEN
	  AVREDCHISQ_BEST=AVREDCHISQ
	  DO I=1,NUMPARAM_SOLV
	    PARAMETRS_BEST(I)=PARAMETRS_SOLV(I)
	  END DO
	END IF
	IF (ITERATION_SCHEME.EQ.'A'.OR.ITERATION_SCHEME.EQ.'B') THEN
	  IF (AVREDCHISQ.GT.0.98D0*AVREDCHISQ_PRIOR) THEN
	    FRACINCR=0.98*FRACINCR
	  END IF
	  IF (AVREDCHISQ.EQ.AVREDCHISQ_PRIOR) THEN
	    FRACINCR=0.9*FRACINCR
	  ELSE IF (AVREDCHISQ.GT.AVREDCHISQ_PRIOR) THEN
	    FRACINCR=0.67*FRACINCR
	  END IF
	  IF (AVREDCHISQ.GE.AVREDCHISQ_PRIOR) THEN
	    DO I=1,NUMPARAM_SOLV
	      IF (ITERATION_SCHEME.EQ.'A') THEN
	        PARAMETRS_SOLV(I)=PARAMETRS_BEST(I)*(1.0-FRACINCR)+
	1       PARAMETRS_SOLV(I)*FRACINCR
	      ELSE IF (ITERATION_SCHEME.EQ.'B') THEN
	        PARAMETRS_SOLV(I)=PARAMETRS_BEST(I)
	      END IF
	    END DO
	  END IF
	ELSE IF (ITERATION_SCHEME.EQ.'C') THEN
	  IF (AVREDCHISQ.GE.AVREDCHISQ_BEST) THEN
	    FRACINCR=0.67*FRACINCR
	    DO I=1,NUMPARAM_SOLV
	      PARAMETRS_SOLV(I)=PARAMETRS_BEST(I)
	    END DO
	  END IF
	ELSE IF (ITERATION_SCHEME.EQ.'M') THEN
	  ITERATE=.TRUE.
	  CALL USRINP(ITERATE,1,'L',4,1,'ITERATE=',
	1 'Do you wish to iterate 1 more time?  (default = Y)')
	  IF (ITERATE) THEN
	    DO I=1,NUMPARAM_SOLV
	      PARAMETRS_SOLV(I)=PARAMETRS_BEST(I)
	    END DO
	    FRACINCR=0.67*FRACINCR
	    CALL USRINP(FRACINCR,1,'F',4,1,
	1   'FRACTIONAL_INCREMENT=','Enter the fractional '//
	2   'increment:  (default = 0.67 x last value)')
	    CALL MESSAGE('Enter the new estimates for the parameters being')
	    CALL USRINP(PARAMETRS_SOLV,NUMPARAM_SOLV,'F',8,1,'PARAMETERS=',
	1   'solved for:  (default = best values)')
	  END IF
	END IF
C
	AVREDCHISQ_PRIOR=AVREDCHISQ
	N_ITERATIONS=N_ITERATIONS+1
	IF (.NOT.ITERATE) WRITE(7,1000) N_ITERATIONS
1000	FORMAT(' ','N_ITERATIONS = ',I10)
C	IF (MOD(N_ITERATIONS,100).EQ.1) THEN
C	END IF
C
	RETURN
	END
