	SUBROUTINE NONLINFIT(X,MAXNPOINTS,DIMENSN_IND,DATA,DIMENSN_DATA,WEIGHT,
	1IX,FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	2ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,MEANERROR,
	3MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,
	4XPLOT,DATPLOT,NPLPOINTS,RESIDUALS)
C
C       This subroutine does a nonlinear least squares fit to weighted data
C  and returns the parameters of the fit and their errors.  If desired,
C  certain coefficients may be held constant in the solution.  The dimensions
C  of the independent variable and the data are unrestricted.  The number of
C  points to be fit and the number of parameters to be solved for are also
C  unrestricted.  For plotting purposes, an array is filled with calculated
C  values corresponding to the values in the array XPLOT.
C       The weights can be relative or equal to the inverse square of the
C  standard deviation.  For relative weights however, the computed reduced
C  chi squared will be incorrect.
C       For each new function category, new subroutines which calculate
C  the function and its derivative must be created.  These must be referenced
C  in subroutines NLFUN and NLDER.  The object codes for subroutines NLFUN,
C  NLDER, NLFUN*, AND NLDER* belong in the NONLINFIT library.  In the calling
C  program, the plotting arrays XPLOT and DATPLOT must be dimensioned at least
C  as large as the number of points and the dimension of the data.  See the 
C  programs [MIIPS.FOR]NL_DEMO_1, NL_DEMO_2, NL_DEMO_4, NLTEST5,
C  NLTEST8, BINARY, DISPGAL, REDSHFTMAG, and SADFIT for further details on 
C  dimensioning and the subroutines NLFUN* and NLDER* for how to loop through 
C  the calculations.
C					Donald H. Gudehus, January, 1986
C			Version 2, D. Gudehus, April, 1994
C			Made MEANERROR multidimensional, and added
C			the arrays REDCHISQ and RESIDUALS
C			Version 3, D. Gudehus, May, 1995
C			Made CHOICE an arbitrary length
C
C	X		  (C)   An array containing the values of the
C			        independent variable.
C	MAXNPOINTS	  (C)   The dimension of the array X.  This is greater
C			        than or equal to the number of points to be fit.
C	DIMENSION_IND	  (C)	The dimension of the independent variable
C	DATA		  (C)   An array containing the observations to
C			        be fitted.
C	DIMENSION_DATA	  (C)	The dimension of the data.
C	WEIGHT		  (C)   An array for the weights of each point.
C	IX		  (C)	An array containing the starting and ending
C				indexes of the data for the fit.
C	FUNCAT		  (C)   The number of the function category;  ie., this
C				selects the NLFUN# and NLDER# that has
C				been written for a particular category of
C				solution.  If it is zero in the call, it is 
C				prompted for.
C	CHOICE		  (C/R) The choice of fitting function within a 
C				function category.  The position after the
C				last one in use by the calling program is used
C				as a flag by the NLFUN# when filling the plot 
C				array.  CHOICE should therefore be declared
C				large enough.
C	CONSTANTTERMS	  (C/R) An array which is .TRUE. if that coefficient
C			        is to be held constant and not solved for.
C				If NUMPARAM is .LE. zero in the call, the user
C				is prompted for which parameters to hold
C				constant.
C	PARAMETRS	  (C/R) An array containing the coefficients of the
C			        fitting function.
C	MAXNUMPARAM	  (C)	The maximum number of parameters desired to be
C				fit by the calling program.
C	NUMPARAM	  (C/R)	In the call, the absolute value gives the total
C				number of parameters in the function.  If it
C				is zero in the call, the user is prompted for
C				the number.  If it is less than zero in the 
C				call, the user is prompted for which parameters
C				to hold constant.  The absolute value is always
C				returned.
C	ITERATION_SCHEME  (C)	The scheme for iterating:  A is automatic with
C				maximum searching;  B is automatic with minimum
C				searching;  C is automatic with no searching;
C				M is manual.  If ITERATION_SCHEME
C				is blank in the call or M, the user is promted
C				for the value and also prompted for the initial
C				value of the fractional increment.
C	TOLERANCE	  (C/R)	The fractional tolerance for continuing the
C				iteration.  If it is zero in the call and the
C				ITERATION_SCHEME = 'A', 'B', or 'C', the user is
C				prompted for the value.  If the change in any
C				parameter is greater than TOLERANCE times the
C				sum of the parameter and its mean error,
C				iteration is continued.
C	INTERMEDIATE	  (C)	A logical variable which determnines whether
C				the intermediate results are printed on the
C				terminal.
C	OPERATE		  (C)	A logical variable which determines whether
C				to perform a user defined operation in the
C				subroutines NLFUN# and NLDER#.
C	COVARIANCE	  (R)   An array containing the covariances
C	MEANERROR	  (R)   An array giving the mean error for an 
C				observation of average weight for each of
C				the dependent variables
C	MEANERRORMEANERROR 
C			  (R)	An array giving the mean error of the mean
C				error for an observation of average weight
C				for each of the dependent variables
C	REDCHISQ	  (R)   An array giving the reduced chi squared
C				for each of the dependent variables
C	AVREDCHISQ	  (R)	The reduced chi squared averaged over
C				the dependent variables
C	XPLOT		  (C)   An array containing points from each dimension
C			        of the independent variable to be evaluated
C				with the fitted function.
C	DATPLOT		  (R)   An array containing the evaluated points from
C			        XPLOT.
C	NPLPOINTS	  (C)   The number of points to be plotted.  If it
C				equals zero in the call, the array DATPLOT is
C				not filled.  It must not be greater than
C				MAXNPOINTS
C	RESIDUALS	  (R)	An array containing the residuals for each of
C				the dependent variables.
C
	INTEGER DIMENSN_IND,DIMENSN_DATA,FUNCAT,IX(2),
	1ADDRESS1,ADDRESS2,ADDRESS3,ADDRESS4,
	2ADDRESS5,ADDRESS6,ADDRESS7,ADDRESS8,ADDRESS9,ADDRESS10,ADDRESS11,
	3ADDRESS12,ADDRESS13,ADDRESS14,ADDRESS15,ADDRESS16,ADDRESS17,
	4ADDRESS18,ADDRESS19,ADDRESS20,ADDRESS21
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,OPERATE
	REAL XPLOT(MAXNPOINTS,DIMENSN_IND),DATPLOT(MAXNPOINTS,DIMENSN_DATA)
	REAL*8 X(MAXNPOINTS,DIMENSN_IND),DATA(MAXNPOINTS,DIMENSN_DATA),
	1WEIGHT(MAXNPOINTS,DIMENSN_DATA),
	1COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),PARAMETRS(MAXNUMPARAM),
	6MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	7REDCHISQ(DIMENSN_DATA),AVREDCHISQ,
	7RESIDUALS(MAXNPOINTS,DIMENSN_DATA)
C
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*(*) CHOICE
C
C  Create some dynamic memory
	LENGTH1=MAXNPOINTS*DIMENSN_IND  ! XSAVE
	LENGTH2=MAXNUMPARAM  ! NCONSTANT
	LENGTH3=MAXNUMPARAM  ! ISHIFT
	LENGTH4=MAXNUMPARAM  ! KPVT
	LENGTH5=MAXNUMPARAM  ! PARAMETRS_SOLVE
	LENGTH6=MAXNUMPARAM  ! PARAMETRS_INIT
	LENGTH7=MAXNUMPARAM  ! PARAMETRS_PRIOR
	LENGTH8=MAXNUMPARAM  ! PARAMETRS_BEST
	LENGTH9=MAXNPOINTS*DIMENSN_DATA  ! FUNCT
	LENGTH10=MAXNPOINTS*MAXNUMPARAM*DIMENSN_DATA  ! DERIV
	LENGTH11=MAXNUMPARAM  ! SUMRIGHT
	LENGTH12=MAXNUMPARAM*MAXNUMPARAM  ! LEFTSIDE
	LENGTH13=MAXNUMPARAM*(MAXNUMPARAM+1)/2  ! UPTRIPART
	LENGTH14=MAXNUMPARAM-1  ! AUX
	LENGTH15=MAXNUMPARAM*MAXNUMPARAM  ! MINORARRAY
	LENGTH16=DIMENSN_DATA  ! NUM_DATA
	LENGTH17=DIMENSN_DATA  ! WEIGHT_DATA
	LENGTH18=DIMENSN_DATA  ! SUMSIGMA
	LENGTH19=MAXNUMPARAM  ! SIGMAPARAM
	LENGTH20=MAXNUMPARAM  ! SIGP_TEMP
	LENGTH21=MAXNUMPARAM*MAXNUMPARAM  ! COVARIANCE_TEMP
	CALL DYNAMIC(ADDRESS1,LENGTH1,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS2,LENGTH2,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS3,LENGTH3,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS4,LENGTH4,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS5,LENGTH5,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS6,LENGTH6,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS7,LENGTH7,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS8,LENGTH8,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS9,LENGTH9,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS10,LENGTH10,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS11,LENGTH11,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS12,LENGTH12,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS13,LENGTH13,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS14,LENGTH14,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS15,LENGTH15,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS16,LENGTH16,'  I4',.TRUE.)
	CALL DYNAMIC(ADDRESS17,LENGTH17,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS18,LENGTH18,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS19,LENGTH19,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS20,LENGTH20,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS21,LENGTH21,'  R8',.TRUE.)
	CALL NONLINFIT1(X,%VAL(ADDRESS1),MAXNPOINTS,DIMENSN_IND,DATA,
	1DIMENSN_DATA,WEIGHT,
	2IX,FUNCAT,CHOICE,CONSTANTTERMS,PARAMETRS,MAXNUMPARAM,NUMPARAM,
	3ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,COVARIANCE,
	4MEANERROR,MEANERRORMEANERROR,
	4XPLOT,DATPLOT,NPLPOINTS,%VAL(ADDRESS2),%VAL(ADDRESS3),%VAL(ADDRESS4),
	5%VAL(ADDRESS5),%VAL(ADDRESS6),%VAL(ADDRESS7),
	6%VAL(ADDRESS8),%VAL(ADDRESS9),%VAL(ADDRESS10),%VAL(ADDRESS11),
	7%VAL(ADDRESS12),%VAL(ADDRESS13),%VAL(ADDRESS14),%VAL(ADDRESS15),
	8%VAL(ADDRESS16),%VAL(ADDRESS17),%VAL(ADDRESS18),
	9REDCHISQ,AVREDCHISQ,%VAL(ADDRESS19),RESIDUALS,%VAL(ADDRESS20),
	1%VAL(ADDRESS21))
	CALL DYNAMIC(ADDRESS1,LENGTH1,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS2,LENGTH2,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS3,LENGTH3,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS4,LENGTH4,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS5,LENGTH5,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS6,LENGTH6,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS7,LENGTH7,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS8,LENGTH8,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS9,LENGTH9,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS10,LENGTH10,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS11,LENGTH11,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS12,LENGTH12,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS13,LENGTH13,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS14,LENGTH14,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS15,LENGTH15,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS16,LENGTH16,'  I4',.FALSE.)
	CALL DYNAMIC(ADDRESS17,LENGTH17,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS18,LENGTH18,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS19,LENGTH19,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS20,LENGTH20,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS21,LENGTH21,'  R8',.FALSE.)
C
	RETURN
	END
