	SUBROUTINE CONTOURPLOT(ARRAY,NCOLS,NROWS,MAPSCOL,MAPSROW,
	1KIND,C,XYOFF,XYSCALE,CONTOUR,PENWIDTH,NSYMBOL,COLOR,
	2NCONTOURS,DELV,HEIGHT,INTERACT,CROSSHAIRS,DEVICES,ADDENDA)
C
C  Note:  This is the Unix version.
C
C       This subroutine produces contour plots of the array ARRAY.
C  Contour plots can produced by several different methods;  1) line contours
C  of specified intensity levels, 2) symbol contours of specified intensities,
C  3) line contours spaced equidistant in intensity, 4) scanning 5) fast
C  and simple scanning.
C       This subroutine should be placed in the PAN library.
C
C			Donald H. Gudehus, June, 1988
C			Based on ACTPLT by Arnold Rots
C			Version 2, D. Gudehus, July, 1993
C			Prevented erasing of screen when using H
C			command, if a graphics device with a separate
C			TEK window exists.
C			Version 3, D. Gudehus, February 26, 2000
C			Changed FORMAT from G12.5 to G15.7 to avoid
C			output conversion error.  Various other fixes.
C			Version 3U, D. Gudehus, February 27, 2000
C			Converted to Unix
C			Version 4, D. Gudehus, May 17, 2001
C			Improved logic of plot device request
C			Version 5, D. Gudehus, March 18, 2004
C			Improved interactive section
C			Version 6, D. Gudehus, February 2, 2008
C			Allowed a file name to be displayed if using
C			SAOimage
C
C	ARRAY      (C)	The array containing the data to be contour plotted.
C	NCOLS      (C)	The number of columns of ARRAY
C	NROWS      (C)	The number of rows of ARRAY
C	MAPSCOL    (C)	The starting column in the full array of which ARRAY
C			is a subset.  This is used for readback of coordinates.
C	MAPSROW    (C)	The starting row in the full array	
C	KIND       (C)	The kind of contour plot:  1) contour follower,
C			2) symbol plotter, 3) equidistant contours,
C			4) line scanner, 5) fast and simple line scanner
C	C          (C)	An array of the contour intensity levels
C	XYOFF      (C)	An array giving the x and y offsets of the plot, 
C			in inches, from the lower left corner of the plotting
C			device.
C	XYSCALE    (C)	An array giving the scale in the x and y directions
C			in units of inches per pixel.
C	CONTOUR	   (C)	A character variable which determines whether 
C			every other contour is dashed, 'D', or whether the
C			user determines the dashing, 'S'.
C	PENWIDTH   (C)	The penwidth (only for the Lasergrafix and Apple 
C			Laser Writer) in thousandths of an inch.  If the 
C			pen width number is not present, a default pen width
C			(equal to 6 pixels on the Apple and 7 pixels on the 
C			Lasergrafix) is selected.
C	NSYMBOL    (C)	The symbol type for KIND=2
C	COLOR	   (C)	An array containing the color for each contour.
C	NCONTOURS  (C)	The number of contours
C	DELV       (C)	The intensity range of a symbol for KIND = 2.
C	HEIGHT     (C)	The height of a symbol in inches for KIND = 2.
C	INTERACT   (C)	A logical variable controlling whether the user 
C			will be prompted for plotting parameters.  If
C			.TRUE. the user is asked for the penwidths
C			and colors.
C	CROSSHAIRS (C)	A logical variable controlling whether the
C			crosshairs can be turned on and used to read
C			data values from the plot.
C			If .TRUE., then the following is true:  Typing
C			(not entering) a allows the user to enter
C			annotation at the location of the crosshairs
C			with the color most recently selected.  
C			CNTRL I (CNTRL I on Mac OS) toggles to the Greek
C			alphabet and special symbols and back to English,
C			CNTRL H (CNTRL B on Mac OS) gives a backspace, and
C			CNTRL K toggles to subscript, superscript, and back to
C			normal.  Refer to SYMGENGR.FOR for
C			the locations of the Greek characters and special
C			symbols on the keyboard.
C			Typing C turns on the crosshairs, pressing the
C			<CR> prints the values at that location
C			and typing Q turns off the crosshairs.
C			Typing c allows the user to be prompted for the
C			color.
C			Typing F allows the user to produce a frame around the
C			plot.
C			Typing P allows the user to plot a point from the
C			physical coordinates.
C			Typing Q will return.
C			Typing V allows cursor input for drawing a solid
C			vector of the color and line style most recently
C			selected.
C			Typing v allows the user to be prompted for the
C			starting and ending physical values of a vector.
C	DEVICES	   (C)	The plotting device(s).  If it is blank in the
C			in the call, the user is prompted for it.  See
C			PLOTDEVICES.FOR  for the available devices.
C	ADDENDA	   (C)	An argument which if .TRUE. will
C			prevent a call to PLOT with IPEN = 999, thus
C			allowing further plotting and annotation by
C			the calling program.  The calling program should
C			call  PLOT with IPEN = 999 when all additional
C			plotting is completed.  All coordinates for
C			annotation are measured in inches from the lower
C			left corner of the plot.
C
	INCLUDE 'saofacts.for'
C
	PARAMETER MAXSIZEDAT=2048
	DIMENSION ARRAY(NCOLS,NROWS),C(NCONTOURS),XYOFF(2),XYSCALE(2),DM(2),
	1PLOTPOS(2),PAR(2),VECTPARAM(4)
	INTEGER PENWIDTH(NCONTOURS),NSYMBOL(NCONTOURS),
	1COLOR(NCONTOURS),TEMPCOLOR,FRAMECOLOR,ADDRESS1,ADDRESS2,ADDRESS3
C	INTEGER MASK,STATUS
	LOGICAL INTERACT,CROSSHAIRS,ADDENDA,PLOTDEVQUES
C
	CHARACTER*1 CONTOUR
	CHARACTER*(*) DEVICES
	CHARACTER*3 DUMDEVICES(10)
	CHARACTER*30 DUMDEVICES2
C	CHARACTER*73 GRINSTRING
	CHARACTER*132 STRING
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	COMMON /PLOT_CHAN/ NCHAN,NDEV
C
	LOGICAL GRINPROMPT
	COMMON /GRINPROMPT/ GRINPROMPT
C
	DIMENSION DATL(MAXSIZEDAT),DATU(MAXSIZEDAT)
	COMMON /CONTOUR_PLOT/ DATL,DATU,DEGPPIXX,DEGPPIXY,RAC,DECC
C
	COMMON /PLOT_BLOCK/ XCOORD(1000),YCOORD(1000)
C
	INCLUDE 'plotpar.for'
C
	DATA THETA/1.570796326/,RADEG/57.2957795/
C
	DO I=1,1000
	  J=SETEND(XCOORD(I))
	  J=SETEND(YCOORD(I))
	END DO
C  Give some numbers nonzero values to guard against divide by zero
	IF (DEGPPIXX.EQ.0.0) DEGPPIXX=0.01
	IF (DEGPPIXY.EQ.0.0) DEGPPIXY=0.01
C
	IF (INTERACT) THEN
	  KIND=1
	  CALL MESSAGE('Enter the kind of plot desired:')
	  CALL MESSAGE('     1 = Contour follower (default)')
	  CALL MESSAGE('     2 = Symbol plotter')
	  CALL MESSAGE('     3 = Equidistant (in intensity) contour plotter')
	  CALL MESSAGE('     4 = Line scanner')
	  CALL USRINP(KIND,1,'I',4,1,'KIND=',
	1 '     5 = "Fast and simple line scanner')
	  IF (KIND.EQ.1.OR.KIND.EQ.2.OR.KIND.EQ.4) THEN
	    CALL USRINP(C,NCONTOURS,'F',4,0,'CONTOURS=',
	1   'Enter the intensity levels for the contours:  (up to 10 allowed)')
	    NCONTOURS=NEL(C,NCONTOURS)
	  ELSE IF (KIND.EQ.3) THEN
	    CALL MESSAGE('Contours will be equispaced in intensity;  large'//
	1   ' intensity gradients')
	    CALL MESSAGE('may delete a contour')
	    C(3)=0
	    DO WHILE (C(3).LE.1)
	      CALL USRINP(C,NCONTOURS,'F',4,0,'COUNTOURS=',
	1     'Enter the minimun, maximum, and number of levels for the'//
	2     ' contours:')
	    END DO
	    NCONTOURS=C(3)
	  ELSE IF (KIND.EQ.5) THEN
	    CALL USRINP(C,NCONTOURS,'F',4,0,'CONTOURS=',
	1   'Enter a single intensity level for the scan:')
	    NCONTOURS=1
	  END IF
	  IF (KIND.EQ.2) THEN
	    CALL USRINP(PAR,2,'F',4,0,'RANGES=',
	1   'Enter the intensity range of a symbol and its height in'//
	2   ' inches:')
	    DELV=PAR(1)
	    HEIGHT=PAR(2)
	    DO I=1,NCONTOURS
	      NSYMBOL(I)=MOD(I,14)-1
	    END DO
	    CALL USRINP(NSYMBOL,NCONTOURS,'I',4,3,'SYMBOL=',
	1   'Enter the symbol types:  (default = 0 1 2, etc.)')
	  END IF
C  Ask about offsets
	  CALL USRINP(XYOFF,2,'F',4,1,'OFFSETS=',
	1 'Enter the X and Y offsets in inches:  (defaults = 1.4 0.3)')  
C  Ask about scaling
	  XYSCALE(1)=(6.1-XYOFF(1))/FLOAT(NCOLS-1)
	  XYSCALE(2)=(5.3-XYOFF(2))/FLOAT(NROWS-1)
	  XYSCALE(1)=MIN(XYSCALE(1),XYSCALE(2))
	  XYSCALE(2)=XYSCALE(1)
	  CALL MESSAGE('Enter the X and Y scales in inches/pixel:')
	  CALL USRINP(XYSCALE,2,'F',4,1,'SCALES=',
	1 '(defaults = maximum while preserving aspect ratio)')
	  XAXIS_LENGTH=XYSCALE(1)*NCOLS
	  YAXIS_LENGTH=XYSCALE(2)*NROWS
	END IF
C  Take care of the plotting devices
	DUMDEVICES2=DEVICES
	LL=1
	DO J=1,10
	  DO I=1,3
	    DUMDEVICES(J)(I:I)=DUMDEVICES2(LL:LL)
	    LL=LL+1
	  END DO
	END DO
	CALL ALPHA(%REF(DUMDEVICES),10,3,NC,NF,NR,4)
	NUMBERDEVICES=NF
	IF (NUMBERDEVICES.EQ.0.OR.INTERACT) THEN
	  DUMDEVICES(1)=' '
	  CALL PLOTDEVICES(DUMDEVICES)
	END IF
C
	IF (INTERACT) THEN
	  DO I=1,NCONTOURS
	    PENWIDTH(I)=23
	  END DO
	  IF (.NOT.PLOTDEVQUES('-S',1)) THEN !  Normal colors
	    DO I=1,NCONTOURS
	      IF (MOD(I-1,7)+1.EQ.1) COLOR(I)=16776960 !  Yellow
	      IF (MOD(I-1,7)+1.EQ.2) COLOR(I)=16711935 !  Magenta
	      IF (MOD(I-1,7)+1.EQ.3) COLOR(I)=65280 !  Green
	      IF (MOD(I-1,7)+1.EQ.4) COLOR(I)=255 !  Blue
	      IF (MOD(I-1,7)+1.EQ.5) COLOR(I)=65535 !  Cyan
	      IF (MOD(I-1,7)+1.EQ.6) COLOR(I)=16770229 !  Moccasin
	      IF (MOD(I-1,7)+1.EQ.7) COLOR(I)=16711680 !  Red
	    END DO
	  ELSE !  Overlay colors on SAOimage
	    DO I=1,NCONTOURS
	      IF (MOD(I-1,5)+1.EQ.1) COLOR(I)=16777422  ! Yellow, 16777215+207
	      IF (MOD(I-1,5)+1.EQ.2) COLOR(I)=16777419  ! Red
	      IF (MOD(I-1,5)+1.EQ.3) COLOR(I)=16777420  ! Green
	      IF (MOD(I-1,5)+1.EQ.4) COLOR(I)=16777421  ! Blue
	      IF (MOD(I-1,5)+1.EQ.5) COLOR(I)=16777418  ! White
	    END DO
	  END IF
	  CONTOUR='N'
	  IF ((KIND.EQ.1.OR.KIND.EQ.3).AND.NCONTOURS.GT.1) THEN
	    CALL MESSAGE('Enter the desired contour:')
	    CALL MESSAGE('     N = No dashing (default)')
	    CALL USRINP(%REF(CONTOUR),1,'A',1,1,'CONTOUR=',
	1   '     D = Dashing alternating with solid (first is dashed)')
	  END IF
C  Ask about penwidths
	  IF (NCONTOURS.GT.1) THEN
	    CALL MESSAGE('Enter the penwidths in thousandths of an inch for'//
	1   ' the contour(s):')
	    CALL USRINP(PENWIDTH,NCONTOURS,'I',4,3,'PENWIDTHS=',
	1   '(defaults = 23, 23, etc.)')
C  Ask about color
	    IF (.NOT.PLOTDEVQUES('-S',1)) THEN
	      CALL MESSAGE('Enter the colors for the contour(s):  (defaults =')
	      CALL MESSAGE('yellow, magenta, green, blue, cyan, moccasin, red')
	      CALL MESSAGE('with rollover')
	    ELSE !  For making overlays with SAOimage, it is better to use
	         !  the overlay colors
	      CALL MESSAGE('When using SAOimage in overlay mode, a '//
	1     'restricted set of colors is available')
	      CALL MESSAGE('Enter the colors for the contour(s):  (defaults =')
	      CALL MESSAGE('yellow, red, green, blue, white with rollover')
	    END IF	  
	    CALL USRINP(COLOR,NCONTOURS,'I',4,3,'COLORS=',' ')
	  END IF
	END IF
 100	CALL PLOTDEVICES2(DUMDEVICES)
C  Display the name of a file on SAOimage if possible
	IF (PLOTDEVQUES('S*',1)) THEN
	  CALL ALPHA(%REF(DNAME),1,80,NC,NF,NR,2)
	  IF (DNAME.NE.' ') THEN
	    TV_FILENAME(1)=DNAME
	    CALL SAOCAPTION
	  END IF
	END IF
C
	CALL PLOT(XYOFF(1),XYOFF(2),-3)
C	WW=XYSCALE(1)*FLOAT(MAPFCOL-MAPSCOL)
C	HH=XYSCALE(2)*FLOAT(MAPFROW-MAPSROW)
	WW=XYSCALE(1)*FLOAT(NCOLS-1)
	HH=XYSCALE(2)*FLOAT(NROWS-1)
	WWHH=WW/HH
	IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,14) !  Delays display of 
				!  vectors for certain plot devices
	IF (KIND.EQ.1) THEN
	  NDIMEN1=(NCOLS+NROWS)*50 !  The factor was increased from 5 to 10
				   !  to prevent Access Violations on long
				   !  contours.  DHG, April, 1993
				   !  Increased to 50.  March, 1995
	  NDIMEN2=NDIMEN1
	  NDIMEN3=NDIMEN1
C  IRC is given by ADDRESS1, XX by ADDRESS2, YY by ADDRESS3
	  CALL DYNAMIC(ADDRESS1,NDIMEN1,'  I4',.TRUE.)
	  CALL DYNAMIC(ADDRESS2,NDIMEN2,'  R4',.TRUE.)
	  CALL DYNAMIC(ADDRESS3,NDIMEN3,'  R4',.TRUE.)
	  D=0.0
 	  DO I=1,NCONTOURS
	    IF (CONTOUR.EQ.'D') D=I-I/2*2
	    CINT=C(I)
	    CALL VECTMODE(0)
	    CALL COLORMODE(COLOR(I)) !  Should follow VECTMODE for Versaterm
				     !  so that the colors will be seen
	    CALL PENMODE(PENWIDTH(I))
	    CALL SCAN(ARRAY,NCOLS,NROWS,NCOLS,NROWS,CINT,D,WWHH,THETA,HH,IZY,
	1   %VAL(ADDRESS1),NDIMEN1,%VAL(ADDRESS2),NDIMEN2,
	2   %VAL(ADDRESS3),NDIMEN3)
	    IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,15) !  Displays the vectors
	  END DO
	  CALL DYNAMIC(ADDRESS1,NDIMEN1,'  I4',.FALSE.)
	  CALL DYNAMIC(ADDRESS2,NDIMEN2,'  R4',.FALSE.)
	  CALL DYNAMIC(ADDRESS3,NDIMEN3,'  R4',.FALSE.)
	ELSE IF (KIND.EQ.2) THEN
	  DELV2=0.5*DELV
	  DO 55 I=1,NCOLS
	    XP=XYSCALE(1)*FLOAT(I-1)
	    DO 51 J=1,NROWS
	      YP=XYSCALE(2)*FLOAT(J-1)
	      X=ARRAY(I,J)
	      IF (X.LT.-990.0) GO TO 51 !  ?
	      DO 41 K=1,NCONTOURS
	        Y=X-C(K)
	        IF (Y.LT.0.0) GO TO 45
   41	      CONTINUE
	      K=NCONTOURS !  The level is higher than the highest contour
   45	      IF (ABS(Y).LE.DELV2) GO TO 49
	      K=K-1
	      IF (K.LT.0) GO TO 51
	      Y=X-C(K)
	      IF (ABS(Y).LE.DELV2) GO TO 49
	      GO TO 51
   49	      K=K-1
	      IF (K.LT.0) GO TO 51
	      CALL COLORMODE(COLOR(K+1))
	      CALL PENMODE(PENWIDTH(K+1))
	      CALL SYMBOL(XP,YP,HEIGHT,NSYMBOL(K+1),0.0,-1)
   51	    CONTINUE
	    IF (PLOTDEVQUES('D*',0).AND.MOD(I,100).EQ.0) CALL PLOT(0,0,15)
   55	  CONTINUE
	  IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,15)
	ELSE IF (KIND.EQ.3) THEN
	  CBEG=C(1)
	  CLIM=C(2)
	  CSTP=(CLIM-CBEG)/(NCONTOURS-1)
	  CALL CPLOTC(ARRAY,NCOLS,NROWS,NCOLS,NROWS,PENWIDTH,COLOR,
	1 NCONTOURS,CBEG,CLIM,CSTP,XYSCALE(1),XYSCALE(2),DEGPPIXX,DEGPPIXY)
	ELSE IF (KIND.EQ.4) THEN
	  I0=0
	  L0=MIN(NCOLS,MAXSIZEDAT)
	  IS=1
	  DO I=1,L0
	    DATL(I)=ARRAY(I,1)
	  END DO
	  DO J=1,MIN(NROWS-1,MAXSIZEDAT-1)
	    DO I=1,L0
	      DATU(I)=ARRAY(I,J+1)
	    END DO
	    JJ=J-1
	    CALL SCANSL(C,PENWIDTH,COLOR,NCONTOURS,XYSCALE(1),XYSCALE(2),
	1   I0,NCOLS,JJ,IS,DATL,DATU)
	    IF (PLOTDEVQUES('D*',0).AND.MOD(J,100).EQ.0) CALL PLOT(0,0,15)
	  END DO
	  IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,15)
	ELSE IF (KIND.EQ.5) THEN
	  CALL SCAN_SIMPLE(ARRAY,NCOLS,NROWS,XYOFF,XYSCALE,C(1),PENWIDTH(1),
	1 COLOR(1))
	END IF
	CALL VECTMODE(0)
C
C  Display any vectors which haven't yet been displayed, for those devices
C  which are using block mode, such as SAOimage.
	IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,15)
C  Flush the buffer, as required for certain displays such as xterm
C  and xgterm under Unix.
	CALL PLOT(0,0,18)
	ANNOTATION_HEIGHT=0.14
	LINECOUNT=1
C  Activate the crosshairs if they were requested and readback is
C  supported on the plot device
	IF (CROSSHAIRS.AND.PLOTDEVQUES('G*',1)) THEN
	  ICHAR=0
	  IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	    CALL KEYBOARD(ICHAR)
	  ELSE
	    CALL WHERE(XNOW,YNOW,ICHAR)
	  END IF
C  Set the display mode to immediate for those devices which can switch
C  between immediate and delayed, such as SAOimage
	  IF (PLOTDEVQUES('D*',0)) CALL PLOT(0,0,13)
C  Start the alphagraphics mode in the upper left corner.  For SAOimage
C  this would however create occasional garbage characters.
	  IF (.NOT.PLOTDEVQUES('GD*',1)) CALL TEKPAU(0.0,5.44,0)
	  DO WHILE (ICHAR.NE.10.AND.ICHAR.NE.13.AND.ICHAR.NE.81.AND.
	1 ICHAR.NE.113)
	! A <LF>, <CR>, or the letter Q or q for quit.  Unix gives a
	! <LF> for a carriage return press except in SAOimage where a
	! <CR> is produced
	    IF (ICHAR.EQ.97) THEN !  The letter a for annotation
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Annotation requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Annotation requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
C  For a graphics terminal connected to a Unix system, the call to TEKPAU
C  is essential.
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      STRING=' '
	      CALL USRINP(%REF(STRING),1,'A',132,0,'STRING=',
	1     'Enter a string:')
	      CALL ALPHA(%REF(STRING),1,132,NC,NF,NR,1)
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL SYMBOL(XNOW-XYOFF(1),YNOW-XYOFF(2),ANNOTATION_HEIGHT,
	1     %REF(STRING),0.0,NC)
	    ELSE IF (ICHAR.EQ.1) THEN !  Control A for height of annotation
	      ANNOTATION_HEIGHT=0.14
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Annotation height requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Annotation height requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(ANNOTATION_HEIGHT,1,'F',4,1,'HEIGHT=',
	1     'Enter the annotation height in inches:  (default = 0.14)')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	    ELSE IF (ICHAR.EQ.98) THEN !  The letter b for background color
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Backgound color requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Backgound color requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL COLOR_INPUT(TEMPCOLOR,'BACKGROUND_COLOR=',
	1     'Enter the background color (singlet or triplet), e.g., '//
	2     'for SAOimage:')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL COLORMODE_BACKGROUND(TEMPCOLOR)
	    ELSE IF (ICHAR.EQ.67) THEN !  The letter C for coordinates
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Coordinates requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Coordinates requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      ICHAR1=0
	      I=1
	      DO WHILE(ICHAR1.NE.81.AND.ICHAR1.NE.113)
				!  The letter Q or q for quitting coordinates
	        CALL WHERE(XP,YP,ICHAR1)
	        IF (ICHAR1.NE.81.AND.ICHAR1.NE.113) THEN
	          XP=XP-XYOFF(1)
	          YP=YP-XYOFF(2)
	          XL=XP/XYSCALE(1)+FLOAT(MAPSCOL)
	          YL=YP/XYSCALE(2)+FLOAT(MAPSROW)
	          IF (ICHAR1.NE.45) THEN !  If not deleting the last
					 !  values
	            XCOORD(I)=XL
	            YCOORD(I)=YL
	            IF (GRINPROMPT) THEN
C	              ENCODE(28,5019,GRINSTRING) XL,YL
5019	              FORMAT('X=',G12.6,'Y=',G12.6)
C	              CALL GMESSAGE(GRINSTRING(1:28),0)
	            ELSE
	              IF (PLOTDEVQUES('2WT',1)) THEN
	                CALL TEK(0,0,4)
	              ELSE IF (PLOTDEVQUES('1WT',1)) THEN
C  Compute the y position for output so that non-VT100/Retrographics
C  terminals will have the correct position.  For example, on terminals
C  such as the Visual 550 and GraphOn, the alphagraphics cursor is hardware
C  controlled by the crosshair location.  Successive FORTRAN writes then
C  will result in overlapping numbers.
	                IF (NDEV.EQ.301) THEN
	                  CALL TEKPAU(0.0,5.50-0.16*LINECOUNT,0)
	                ELSE
	                  CALL TEKPAU(0.0,5.60-0.16*LINECOUNT,0)
	                END IF
	              END IF
	              WRITE (6,5020) XL,YL
5020	              FORMAT(' ','X=',G15.7,'Y=',G15.7)
	              WRITE(7,5020) XL,YL
	              LINECOUNT=LINECOUNT+1
	            END IF
		    IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	              IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	              IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	            END IF
	          ELSE !  A minus sign for deleting the last value
	   	    J=SETEND(XCOORD(I))
	            J=SETEND(YCOORD(I))
	            I=I-1
	          END IF
	        END IF
	        I=I+1
	      END DO
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Coordinates request canceled')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Coordinates request canceled')
	        CALL TRANSPARENT_TO_TEK
	      END IF  
	    ELSE IF (ICHAR.EQ.99) THEN !  The letter c for color
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Color requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Color requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL COLOR_INPUT(TEMPCOLOR,'COLOR=',
	1     'Enter the color (singlet or triplet):')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL COLORMODE(TEMPCOLOR) !  Must call this after return 
				        !  to graphics mode
	    ELSE IF (ICHAR.EQ.70) THEN !  The letter F for frame
              IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Frame requested')
C	      IF (PLOTDEVQUES('-S',1)) THEN
C	        FRAMECOLOR=16777421 !  Blue
C	      ELSE
C	        FRAMECOLOR=255
C	      END IF
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Frame requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL COLOR_INPUT(FRAMECOLOR,'COLOR=',
	1     'Enter the color of the frame (singlet or triplet):')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL COLORMODE(FRAMECOLOR)
C
C  Plot and label frame, set constants
C
c	      A0=HED(5)
c	      D0=HED(6)
c	      XC=HED(7)-FLOAT(MAPSCOL)
c	      YC=HED(8)-FLOAT(MAPSROW)
	      DM(1)=60
	      DM(2)=600
C  This will be improved to allow annotation in various units
C	      IF (GRINPROMPT) THEN
C	        CALL GMESSAGE('ENTER X (RA) INTERVAL IN SEC. AND Y (DEC)
C	1 INTERVAL IN ":',0)
C	        CALL USRINPG(DM,2,'F',4,1,'TICKS=','(DEFAULT = 60 600)')
C	      ELSE
C	        CALL MESSAGE('Enter X (RA) interval in sec. and Y (DEC)'//
C	1       ' interval in ":')
C	        CALL USRINP(DM,2,'F',4,1,'TICKS=','(default = 60 600)')
C	      END IF
	      DMX=DM(1)
	      DMY=DM(2)
	      IF (DMX.EQ.0) GO TO 150
	      IMX=DM(1)
	      IMY=DM(2)
	      COSD0=COSD(D0)
	      SIND0=SIND(D0)
	      B=COSD0+YC*DEGPPIXY/RADEG*SIND0
	      T=COSD0+(YC+FLOAT(NROWS-1))*DEGPPIXY/RADEG*SIND0
	      X=-XC*DEGPPIXX/RADEG
	      Y=-YC*DEGPPIXY/RADEG
	      RA=(A0-ATAN(X/(COSD0-Y*SIND0))*RADEG)*240.0
	      DEC=ASIN((SIND0+Y*COSD0)/SQRT(1.0+X*X+Y*Y))*RADEG*3600.0
	      X1=0.0
	      X2=0.2
	      X3=-1.4
	      X4=W
	      X5=W-0.2
	      SCD0=SIND0*COSD0
	      CD0S=COSD0*COSD0
	      XL=-XC*DEGPPIXX/RADEG
	      XL=XL*XL
	      XR=(FLOAT(NCOLS-1)-XC)*DEGPPIXX/RADEG
	      XR=XR*XR
	      HT=XYSCALE(2)*(NROWS-1)
	      W=XYSCALE(1)*(NCOLS-1)
	      Y1=0.0
	      Y2=0.2
	      Y3=-0.3
	      Y4=-0.2
	      Y5=HT
	      Y6=HT-0.2
	      Z1=0.14
	      Z2=0.07
C  BOTTOM
	      IRA=RA/DMX
	      SRA=IRA*DMX
 120	      DA=(A0-SRA/240.0)/RADEG
	      X=(TAN(DA)*B*RADEG/DEGPPIXX+XC)*XYSCALE(1)
	      IF (X.GT.W) GO TO 130
C	      CALL PLOT (X,Y1,3)
C	      CALL PLOT (X,Y2,2)
	      IRAH=SRA/3600
	      IRAM=SRA/60-IRAH*60
	      DRAS=SRA-IRAM*60-IRAH*3600
	      X=X-0.6
C  This is commented out for now
C	      CALL NUMBER (X,Y3,Z1,FLOAT(IRAH),0.0,-1)
C	      CALL SYMBOL (999.0,Y4,Z2,2HH ,0.0,2)
C	      CALL NUMBER (999.0,Y3,Z1,FLOAT(IRAM),0.0,-1)
C	      CALL SYMBOL (999.0,Y4,Z2,2HM ,0.0,2)
C	      CALL NUMBER (999.0,Y3,Z1,DRAS,0.0,1)
C	      CALL SYMBOL (999.0,Y4,Z2,1HS,0.0,1)
	      SRA=SRA-DMX
	      GO TO 120
C  RIGHT
 130	      IDEC=DEC/DMY
	      IDEC=IDEC*IMY
	      IF (IDEC.LT.0) IDEC=IDEC-IMY
 140	      IDEC=IDEC+IMY
	      D=FLOAT(IDEC)/(3600.0*RADEG)
	      SIN_D=SIN(D)
	      SDS=SIN_D*SIN_D
	      COS_D=COS(D)
	      CDS=COS_D*COS_D
	      Y=(SCD0-SIN_D*SQRT(XR*(CD0S-SDS)+CDS))/(SDS-CD0S)
	      Y=(Y*RADEG/DEGPPIXY+YC)*XYSCALE(2)
	      IF (Y.LT.0.0) GO TO 140
	      IF (Y.GT.HT) GO TO 150
C	      CALL PLOT (X4,Y,3)
C	      CALL PLOT (X5,Y,2)
	      GO TO 140
C  Draw the top and left of the frame
 150	      CALL PLOT (W,HT,3)
	      CALL PLOT (0.0,HT,2)
	      CALL PLOT (0.0,0.0,2)
	      IF (DMX.EQ.0) GO TO 190
C  LEFT
	      IDEC=DEC/DMY
	      IDEC=(IDEC+1)*IMY
	      IF (IDEC.LT.0) IDEC=IDEC-IMY
 160	      D=FLOAT(IDEC)/(3600.0*RADEG)
	      SIN_D=SIN(D)
	      SDS=SIN_D*SIN_D
	      COS_D=COS(D)
	      CDS=COS_D*COS_D
	      Y=(SCD0-SIN_D*SQRT(XL*(CD0S-SDS)+CDS))/(SDS-CD0S)
	      Y=(Y*RADEG/DEGPPIXY+YC)*XYSCALE(2)
	      IF (Y.GT.HT) GO TO 170
	      Y1=Y-0.1
C	      CALL PLOT (X1,Y,3)
C	      CALL PLOT (X2,Y,2)
	      KDEC=IDEC
	      IF (IDEC.LT.0) KDEC=-IDEC
	      IDECD=KDEC/3600
	      IDECM=KDEC/60-IDECD*60
	      IDECS=KDEC-IDECM*60-IDECD*3600
	      IF (IDEC.LT.0) IDECD=-IDECD
C	      CALL NUMBER (X3,Y1,Z1,FLOAT(IDECD),0.0,-1)
C	      CALL SYMBOL (999.0,Y,Z2,2H0 ,0.0,2)
C	      CALL NUMBER (999.0,Y1,Z1,FLOAT(IDECM),0.0,-1)
C	      CALL SYMBOL (999.0,Y,Z2,2H/ ,0.0,2)
C	      CALL NUMBER (999.0,Y1,Z1,FLOAT(IDECS),0.0,-1)
C	      CALL SYMBOL (999.0,Y,Z2,2H//,0.0,2)
	      IDEC=IDEC+IMY
	      GO TO 160
C  TOP
 170	      IRA=RA/DMX
	      SRA=(IRA+1)*DMX
 180	      SRA=SRA-DMX
	      DA=(A0-SRA/240.0)/RADEG
	      X=(TAN(DA)*T*RADEG/DEGPPIXX+XC)*XYSCALE(1)
	      IF (X.LT.0.0) GO TO 180
	      IF (X.GT.W) GO TO 190
C	      CALL PLOT (X,Y5,3)
C	      CALL PLOT (X,Y6,2)
	      GO TO 180
C  Draw the right and bottom of frame
 190	      CALL PLOT (W,HT,3)
	      CALL PLOT (W,0.0,2)
	      CALL PLOT (0.0,0.0,2)
C
	    ELSE IF (ICHAR.EQ.72.OR.ICHAR.EQ.104) THEN
				!  The letters H or h for help
	      IF (PLOTDEVQUES('GD*',1)) CALL MESSAGE('Help requested')
C  If a graphics terminal having a separate TEK window exists, don't erase
C  the screen;  otherwise erase in order to display the help commands.
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Help requested')
	      ELSE IF (PLOTDEVQUES('GT',1)) THEN
	        CALL TEK(0,0,0)
	        CALL TEK(0,0,4)
	      END IF
	      CALL MESSAGE('a = Allows the user to enter annotation at the'//
	1     ' location of the crosshairs')
	      CALL MESSAGE('    with the color most recently selected')
	      CALL MESSAGE('    ^I (^L on Mac OS) toggles to the Greek'//
	1     ' alphabet and special symbols')
	      CALL MESSAGE('    and back to English')
	      CALL MESSAGE('    ^H (^B on Mac OS) gives a backspace')
	      CALL MESSAGE('    ^K toggles to subscript, superscript, and'//
	1     ' back to normal')
	      CALL MESSAGE('^A = Prompts user for the height of the '//
	1     'annotation')
	      CALL MESSAGE('b = Allows the user to be prompted for the '//
	1     'background color')
	      CALL MESSAGE('C = Turns on the crosshairs and by pressing the'//
	1     ' <CR>, prints the values at')
	      CALL MESSAGE('    that location and puts the values in a'//
	1     ' common block;  typing - deletes the')
	      CALL MESSAGE('    last pair of values from the common block;'//
	1     ' and typing Q turns off the')
	      CALL MESSAGE('    crosshairs.')
	      CALL MESSAGE('c = Allows the user to be prompted for the'//
	1     ' color')
	      CALL MESSAGE('      Overlay colors for SAOimage:')
	      CALL MESSAGE('      Yellow = 16777422, Red = 16777419, '//
	1     'Green  = 16777420')
	      CALL MESSAGE('      Blue = 16777421, White  = 16777418')
	      CALL MESSAGE('F = Draws a frame around the plot')
	      CALL MESSAGE('H = Gives help')
	      CALL MESSAGE('h = Gives help')
	      CALL MESSAGE('O = Redoes the original plot')
	      CALL MESSAGE('P = Plots a cross at a given position')
	      CALL MESSAGE('Q = Returns.')
	      CALL MESSAGE('q = Returns.')
	      CALL MESSAGE('V = Draws a vector of the color and vector'//
	1     ' style most recently selected.')
	      CALL MESSAGE('v = Allows the user to be prompted for the'//
	1     ' starting and ending physical')
	      CALL MESSAGE('    values of a vector.')
C  Return control to the TEK window if a multi-window TEK device is being 
C  used.
	      IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	      IF (PLOTDEVQUES('1WT',1))
	1     CALL MESSAGE('Press "O" to continue')
	    ELSE IF (ICHAR.EQ.79) THEN !  The letter O for redo
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Original plot requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Original plot requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      GO TO 100
	    ELSE IF (ICHAR.EQ.80) THEN !  The letter P for plotting a position
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Plot of position requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Plot of position requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      J=SETEND(PLOTPOS)
	      J=SETEND(PLOTPOS(2))
	      IF (GRINPROMPT) THEN
C	        CALL USRINPG(PLOTPOS,2,'F',4,0,'POSITION=',
C	1       'ENTER THE OBJECT''S POSITION IN PIXELS:')
	      ELSE
	        CALL USRINP(PLOTPOS,2,'F',4,0,'POSITION=',
	1       'Enter the object''s position in pixels:')
	      END IF
	      I=I+1
	      WRITE (7,6040) PLOTPOS(1),PLOTPOS(2)
6040	      FORMAT (5H A,D:,2F12.5)
	      X=(PLOTPOS(1)-MAPSCOL)*XYSCALE(1)
	      Y=(PLOTPOS(2)-MAPSROW)*XYSCALE(2)
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      CALL SYMBOL (X,Y,0.5334,3,0.0,-1)
	    ELSE IF (ICHAR.EQ.86) THEN !  The letter V for vector
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Vectors from cursor requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Vectors from cursor requested')
	        CALL TRANSPARENT_TO_TEK
	      END IF
	      CALL WHERE(XNOW,YNOW,ICHAR1)
	      CALL WHERE(XNOW1,YNOW1,ICHAR1)
	      CALL PLOT(XNOW-XYOFF(1),YNOW-XYOFF(2),3)
	      CALL PLOT(XNOW1-XYOFF(1),YNOW1-XYOFF(2),2)
C  Turn off the cursor so that the vector will be unobstructed.  No
C  problem with xterm displaying garbage characters here.
	      IF (.NOT.PLOTDEVQUES('GD*',1)) CALL TEKCON
	    ELSE IF (ICHAR.EQ.118) THEN !  The letter v for vector in
					!  physical units
	      IF (PLOTDEVQUES('GD*',1))
	1     CALL MESSAGE('Vectors from physical values requested')
	      IF (PLOTDEVQUES('2WT',1)) THEN
	        CALL TEK(0,0,4)
	        CALL MESSAGE('Vectors from physical values requested')
	      ELSE IF (PLOTDEVQUES('1WT',1)) THEN
	        CALL TEKPAU(0.0,0.0,1)
	      END IF
	      CALL USRINP(VECTPARAM,4,'F',4,0,'PARAMETERS=',
	1     'Enter start x, y and end x, y in units of pixels:')
	      IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	        IF (PLOTDEVQUES('2WT',1)) CALL TRANSPARENT_TO_TEK
	        IF (PLOTDEVQUES('1WT',1)) CALL TEKCON
	      END IF
	      XNOW=(VECTPARAM(1)-MAPSCOL)*XYSCALE(1)
	      YNOW=(VECTPARAM(2)-MAPSROW)*XYSCALE(2)
	      CALL PLOT(XNOW,YNOW,3)
	      XNOW=(VECTPARAM(3)-MAPSCOL)*XYSCALE(1)
	      YNOW=(VECTPARAM(4)-MAPSROW)*XYSCALE(2)
	      CALL PLOT(XNOW,YNOW,2)
	    END IF
C  Flush any remaining information
	    CALL PLOT(0.0,0.0,18)
	    IF (.NOT.PLOTDEVQUES('GD*',1)) THEN
	      CALL KEYBOARD(ICHAR)
	    ELSE
	      CALL WHERE(XNOW,YNOW,ICHAR)
	    END IF
	  END DO
	END IF
C  On Unix, avoid garbage vectors by flushing the buffer.  Going to
C  alphagraphics with TEKPAU or calling RESETTEK before
C  quitting also prevents this.
	CALL PLOT(0.0,0.0,18)
C
C  LABEL PLOT
C
 200	NLA=.TRUE.
C	IF (GRINPROMPT) THEN
C	  CALL USRINPG(NLA,1,'L',4,1,'LABEL=','DO YOU WISH TO HAVE
C	1 LABELS WITH THE PLOT?  (DEFAULT = Y)')
C	ELSE
C	  CALL USRINP(NLA,1,'L',4,1,'LABEL=','Do you wish to have
C	1 labels with the plot?  (default = Y)')
C	END IF
C	IF (.NOT.NLA) RETURN
C	XLABEL=XBIN
C	YLABEL=YBIN
C	IF (GRINPROMPT) THEN
C	  CALL USRINPG(%REF(XLABEL),1,'A',40,1,'X_LABEL=',
C	1 'ENTER THE X LABEL:')
C	ELSE
C	  CALL USRINP(%REF(XLABEL),1,'A',40,1,'X_LABEL=',
C	1 'Enter the X label:')
C	END IF
C	CALL ALPHA(%REF(XLABEL),40,1,LX,NF,NR,1)
C	IF (GRINPROMPT) THEN
C	  CALL USRINPG(%REF(YLABEL),1,'A',40,1,'Y_LABEL=',
C	1 'ENTER THE Y LABEL:')
C	ELSE
C	  CALL USRINP(%REF(YLABEL),1,'A',40,1,'Y_LABEL=',
C	1 'Enter the Y label:')
C	END IF
C	CALL ALPHA(%REF(YLABEL),40,1,LY,NF,NR,1)
C	X=W+1.5*0.3937
C	Y=AMAX1(HT-1.0,15.0)
C	Y=Y*0.3937
C	Z=0.5334*0.3937
C 99	CONTINUE
C	CALL SYMBOL (X,Y,Z,4HRA= ,0.0,4)
C	CALL NUMBER (999.0,Y,Z,RAC,0.0,6)
C	CALL SYMBOL (999.0,Y,Z,5H DEG.,0.0,5)
C	Y=Y-1.0*0.3937
C	CALL SYMBOL (X,Y,Z,4HDEC=,0.0,4)
C	CALL NUMBER (999.0,Y,Z,DECC,0.0,6)
C	CALL SYMBOL (999.0,Y,Z,5H DEG.,0.0,5)
C	Y=Y-1.0*0.3937
C	CALL SYMBOL (X,Y,Z,4HCH= ,0.0,4)
C	CALL NUMBER (999.0,Y,Z,FLOAT(IHED(3)),0.0,-1)
C	Y=Y-1.0*0.3937
C	CALL SYMBOL (X,Y,Z,6HVHEL= ,0.0,6)
C	CALL NUMBER (999.0,Y,Z,HED(4),0.0,1)
C	Y=Y-1.5*0.3937
C	Z=0.3556*0.3937
C	CALL SYMBOL (X,Y,Z,12HCONT INTVS :,0.0,12)
C	DO I=1,NCONTOURS,2
C	  Y=Y-0.7*0.3937
C	  CALL NUMBER (X,Y,Z,C(I),0.0,2)
C	  CALL SYMBOL(999.0,Y,Z,5H     ,0.0,5)
C	  CALL NUMBER (999.0,Y,Z,C(I+1),0.0,2)
C	END DO
C
C  Produce an end-of-plot command
	IF (.NOT.ADDENDA) THEN
	  CALL PLOT(-XYOFF(1),-XYOFF(2),-3)
	  CALL PLOT(0.0,0.0,999)
	END IF
	IF (NDEV.EQ.10) LASERPRINT=.TRUE.
	RETURN
	END
