	SUBROUTINE RADGALOLD
C
C       This subroutine reads a data file containing a galaxy's surface
C  brightness as a function of radius and then allows one to plot a
C  preliminary Hubble "a" against the radius and solve for the 
C  coefficients describing a chosen surface brightness law.  A nonlinear
C  least squares solution with weights is made and the mean errors of 
C  each parameter are calculated.  One may convolve the surface
C  brightness law by a seeing function obtained from a data file before
C  doing the fit.  The total light of the galaxy is next
C  calculated and the residuals of the solution plotted.
C  Note:  The code called by NLTEST1, using NONLINFIT will eventually
C  be installed here.
C				Donald H. Gudehus, December, 1982
C				Version 2, Donald H. Gudehus, March, 1995
C				Made slight refinement in calculation of
C				convolution step size.
C
	CHARACTER*1 WEIGHT_SCHEME,ITERATION_SCHEME
	CHARACTER*6 IDLAST,IDREAD,CHOICE
	CHARACTER*40 FILENAMESMEAR,FILENAMEWEIGHTS,FILENAMERESULTS
	CHARACTER*50 XLABEL,YLABEL,TITLE
	COMMON /CHAR_SCRATCH_BLOCK/ WEIGHT_SCHEME,ITERATION_SCHEME,
	1IDLAST,IDREAD,CHOICE,FILENAMESMEAR,FILENAMEWEIGHTS,FILENAMERESULTS,
	2XLABEL,YLABEL,TITLE
C
	CHARACTER*6 ID,ID2,ID3
	CHARACTER*27 IDBBEN
	COMMON /ID/ ID,ID2,ID3,IDBBEN
C
	LOGICAL OFIL
	COMMON /OLDFILE/ OFIL
C
	CHARACTER*80 FILENAME
	COMMON /CFILE/ FILENAME
C
	PARAMETER MAXSIZE=5000
	REAL*8 SUM(6,6),SUMRIGHT(6),UPTRIPART(21),AUX(5),DET(2),
	1MINOR(2),MINORARRAY(6,6),ADUINTEGRAL1,ADUINTEGRAL2,
	2SIGMASUM(2),SUMWEIGHT,WEIGHT,SUMLOGWEIGHT,LOGWEIGHT,
	3INVERSE(6,6),SIGN,NUMER,DENOM
	REAL*4 COLUMN,ROW,INCLINA,LOGINTADU,NUMRADII
	INTEGER COLOR
	INTEGER*4 SIZE,STATUS,IRADII(2),IRADIICONVOLVE(2),KPVT(6)
	LOGICAL PLOTA,INVERT(2),INTERACT,CROSSHAIRS,ADJUST,
	1MORE,SOLVESKY,ITERATE,FIRSTIME,PLOTD,CONVOLVE,
	1EVALUATE,SOLVE,PARAMETRSCHANGED,GO,STORERESULTS,STOREOPENED

	COMMON /SCRATCH_BLOCK/ RADIUS(MAXSIZE),ECCENTRICITY(MAXSIZE),
	1SURFBRIGHTOB(MAXSIZE),YARRAY(MAXSIZE),PARAMETRS(6),PARAMETRS_INIT(6),
	2PARAMETRS_PRIOR(6),SMEAR(MAXSIZE),SMEARREAD(MAXSIZE),
	3SMEARRADIUS(MAXSIZE),SURFBCONVOLVE(MAXSIZE),
	4ARRAYINCONVOLVE(MAXSIZE),RADIUSINCONVOLVE(MAXSIZE),
	5USERPARAMETRS(6),READOUT(2),COVARIANCE(15),FILEWEIGHT(MAXSIZE),
	6DERIV(MAXSIZE,6),RADII(2),NPOINTS_MAX,SUM,SUMRIGHT,
	7UPTRIPART,AUX,DET,MINOR,MINORARRAY,
	8ADUINTEGRAL1,ADUINTEGRAL2,SIGMASUM,
	9SUMWEIGHT,WEIGHT,SUMLOGWEIGHT,LOGWEIGHT,INVERSE,SIGN,
	1NUMER,DENOM,SIGMA,SIGMA_INIT,SIGMAPARAM(6),
	1OBMINUSCALC(MAXSIZE,2),COLUMN,ROW,INCLINA,ECCENTR,LOGINTADU,
	2COLOR,SIZE,STATUS,IRADII,IRADIICONVOLVE,KPVT,
	3PI,TWOPI,LX,NF,NR,INDEXMINUSRADIUS,R,ADU,SKY,
	4PLOTA,INVERT,INTERACT,CROSSHAIRS,ADJUST,MORE,SOLVESKY,
	5ITERATE,FIRSTIME,PLOTD,CONVOLVE,EVALUATE,SOLVE,
	6PARAMETRSCHANGED,GO,STORERESULTS,STOREOPENED,TOLERANCE,FRACINCR,
	6CONVOLVEFACTOR,NUMPARAM,NPLOTS,NPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,
	7NPOINTS_S,RADIUS_C,NPOINTS_C,NANGLES,NUMRADII,NTIMES,IER,
	8EPS,ADUMAGINTEGRAL,AVWEIGHT,AVLOGWEIGHT,REDCHISQR,EXPCT_SIGMA,
	9EXPCT_SIGMASIGMA
C
	DATA PI /3.14159265/,TWOPI /6.2831853/
C
	CALL MESSAGE('RADGAL, Version 2')
	STOREOPENED=.FALSE.
C  Get the data to be fitted
	IDLAST=' '
	CALL MESSAGE('Open the data file')
	CALL OPENCF(21,1,'FILE_21=',0)
   2	REWIND 21
	CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1'Enter the ID of the object:  (default = last galaxy entry)')
	CALL ALPHA(%REF(ID),1,6,LX,NF,NR,2)
	IF (ID.EQ.IDLAST) GO TO 42
	READ(21,1100,ERR=14)
	GO TO 16
  14	CALL MESSAGE('Couldn''t read the header')
	RETURN
  16	CONTINUE
1100	FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',1X,'INCLINATION',
	11X,'ECCENTRICITY',1X,'RADIUS',5X,'ADU',3X,'LOG(INTEGRAL)')
	IDREAD=' '
	DO 20 WHILE (ID.NE.IDREAD)
	  READ(21,1101,END=22) IDREAD,COLUMN,ROW,INCLINA,ECCENTR,R,
	1 ADU,LOGINTADU
  20	CONTINUE
	IF (R.NE.0.0) CALL MESSAGE('Warning!  The file does not begin 
	1at the center of the galaxy')
	RADIUS(1)=R
	INDEXMINUSRADIUS=1-JINT(R)
	ECCENTRICITY(1)=ECCENTR
	SURFBRIGHTOB(1)=ADU
	GO TO 24
  22	CALL MESSAGE('Couldn''t find the object')
	RETURN
  24	CONTINUE
	NPOINTS_MAX=1
	DO 40 WHILE (ID.EQ.IDREAD)
	  READ(21,1101,END=42) IDREAD,COLUMN,ROW,INCLINA,ECCENTR,R,
	1 ADU,LOGINTADU
1101	  FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,F8.3,1X,
	1 F13.6,1X,F11.8)
	  NPOINTS_MAX=NPOINTS_MAX+1
	  RADIUS(NPOINTS_MAX)=R
	  ECCENTRICITY(NPOINTS_MAX)=ECCENTR
	  SURFBRIGHTOB(NPOINTS_MAX)=ADU
  40	CONTINUE
	NPOINTS_MAX=NPOINTS_MAX-1  ! Ignores the last number read
  42	CONTINUE
C  See if a first estimate of the Hubble "a" is desired
	CALL MESSAGE('Do you wish to plot the first estimate of the '//
	1'Hubble "a" against radius?')
	PLOTA=.FALSE.
	CALL USRINP(PLOTA,1,'L',1,1,'PLOT=','(default = N)')
C  Set up the plotting parameters
	SIZE=MAXSIZE
	NPLOTS=1
	LINVECT=0
	INC=1
	LINTYPE=0
	NSYMBOL=0
	INVERT(1)=.FALSE.
	INVERT(2)=.FALSE.
	COLOR=16776960
	INTERACT=.FALSE.
	CROSSHAIRS=.TRUE.
	XLABEL='RADIUS (pixels)'
	YLABEL='HUBBLE A (pixels)'
	TITLE='FIRST ESTIMATE'
	SKY=SURFBRIGHTOB(1)
	DO I=1,NPOINTS_MAX
	  SKY=AMIN1(SKY,SURFBRIGHTOB(I))
	END DO
	IF (PLOTA) THEN
	  CALL USRINP(SKY,1,'F',4,1,'SKY=',
	1 'Enter the sky level to be subtracted from the data: '//
	2 '(default = minimum)')
	  DO 50 I=2,NPOINTS_MAX  ! We must start at 2 to avoid division
				 ! by 0
	    DENOM=SURFBRIGHTOB(I)-SKY
	    SIGN=1.0D0
	    IF (DENOM.EQ.0.0) THEN
	      YARRAY(I)=0.0
	      GO TO 50
	    ELSE IF (DENOM.LT.0.0) THEN
	      DENOM=-DENOM
	      SIGN=-1.0D0
	    END IF
	    FACTOR=(SURFBRIGHTOB(1)-SKY)/DENOM
	    YARRAY(I)=SIGN*RADIUS(I)/(SQRT(FACTOR)-1.0)
  50	  CONTINUE
	  CALL XYPLOT(RADIUS(2),YARRAY(2),SIZE-1,NPLOTS,NPOINTS_MAX-1,
	1 LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,
	2 XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
	END IF
	CHOICE=' '
	DO WHILE (CHOICE.NE.'G'.AND.CHOICE.NE.'H'.AND.CHOICE.NE.'M'
	1.AND.CHOICE.NE.'MC'.AND.CHOICE.NE.'ML'.AND.CHOICE.NE.'MR'
	2.AND.CHOICE.NE.'P'.AND.CHOICE.NE.'T'.AND.CHOICE.NE.'V'
	3.AND.CHOICE.NE.'VC'.AND.CHOICE
	4.NE.'K'.AND.CHOICE.NE.'L'.AND.CHOICE.NE.'HV'.AND.CHOICE.NE.'I'
	5.AND.CHOICE.NE.'SI'.AND.CHOICE.NE.'E3')
	  CHOICE='M'
	  CALL MESSAGE('Enter the surface brightness law:')
	  CALL MESSAGE('     G  = Gaussian')
	  CALL MESSAGE('     H  = Hubble')
	  CALL MESSAGE('     HV = Hubble with variable power in
	1 denominator')
	  CALL MESSAGE('     I  = Inverse radius')
	  CALL MESSAGE('     K  = Constant')
	  CALL MESSAGE('     L  = Straight line')
	  CALL MESSAGE('     M  = Modified Hubble (default)')
	  CALL MESSAGE('     MC = Modified Hubble plus modified Hubble for '//
	1 'cluster')
	  CALL MESSAGE('     ML = Fixed modified Hubble with color '//
	1 'gradient, linear in log(r)')
	  CALL MESSAGE('     MR = Fixed modified Hubble with color '//
	1 'gradient, linear in r')
C	  CALL MESSAGE('     P  = Power series for "1/a"')
	  CALL MESSAGE('     SI = Sum of K/r and K''/r**2')
	  CALL MESSAGE('     T  = Tapered Hubble')
	  CALL MESSAGE('     V  = de Vaucouleurs')
	  CALL MESSAGE('     VC = de Vaucouleurs plus de Vaucouleurs '//
	1 'for cluster')
	  CALL MESSAGE('     E3 = Extrafocal photometry of E3 galaxies')
	  CALL USRINP(%REF(CHOICE),1,'A',6,1,'CHOICE=',' ')
	  CALL ALPHA(%REF(CHOICE),1,6,LX,NF,NR,2)
	END DO
	IF (CHOICE.EQ.'MC'.OR.CHOICE.EQ.'VC') THEN
	  SOLVE=.FALSE.
	  CALL USRINP(SOLVE,1,'L',1,1,'SOLVE=',
	1 'Do you wish to solve for the radius parameters?  (default
	2 = N)')
	  IF (SOLVE) THEN
	    IF (CHOICE.EQ.'MC') CHOICE='MCX'
	    IF (CHOICE.EQ.'VC') CHOICE='VCX'
	  ELSE
	    CALL USRINP(USERPARAMETRS(1),2,'F',4,0,'RADIUS_PARAMETERS=',
	1   'Enter the radius parameters:')
	  END IF
	ELSE IF (CHOICE.EQ.'I') THEN
	  SOLVE=.FALSE.
	  CALL USRINP(SOLVE,1,'L',1,1,'SOLVE=',
	1 'Do you wish to solve for the power of the radius?  (default
	2 = N)')
	  IF (SOLVE) THEN
	    CHOICE='IX'
	  ELSE
	    TPOWER=1.0
	    CALL USRINP(TPOWER,1,'F',4,1,'POWER=',
	1   'Enter the power of the radius:  (default = 1.0)')
	    USERPARAMETRS(1)=TPOWER  ! User supplied constant
	  END IF
	ELSE IF (CHOICE.EQ.'T') THEN
	  SOLVE=.FALSE.
	  CALL USRINP(SOLVE,1,'L',1,1,'SOLVE=',
	1 'Do you wish to solve for the power of the taper?  (default
	2 = N)')
	  IF (SOLVE) THEN
	    CHOICE='TX'
	  ELSE
	    TPOWER=2.0
	    CALL USRINP(TPOWER,1,'F',4,1,'POWER=',
	1   'Enter the power of the taper:  (default = 2.0)')
	    USERPARAMETRS(1)=TPOWER  ! User supplied constant
	  END IF
	END IF
	SOLVESKY=.TRUE.
	IF (CHOICE.EQ.'K'.OR.CHOICE.EQ.'L'.OR.CHOICE.EQ.'E3'.OR.CHOICE.EQ.
	1'ML'.OR.CHOICE.EQ.'MR') THEN
	  SOLVESKY=.FALSE.
	  SKY=0.0
	ELSE
	  CALL USRINP(SOLVESKY,1,'L',1,1,'SOLVE=',
	1 'Do you wish to solve for the sky level also?  (default = Y)')
	  IF (.NOT.SOLVESKY.AND..NOT.PLOTA) THEN
	    SKY=SURFBRIGHTOB(1)
	    DO I=1,NPOINTS_MAX
	      SKY=AMIN1(SKY,SURFBRIGHTOB(I))
	    END DO
	    CALL USRINP(SKY,1,'F',4,1,'SKY=',
	1   'Enter the sky level to be subtracted from the data: 
	2 (default = minimum)')
	  END IF
	END IF
C  Find out if the user wishes to convolve the surface brightness law
	CONVOLVE=.TRUE.
	CALL MESSAGE('Do you wish to convolve the surface brightness 
	1law with a smearing distribution?')
	CALL USRINP(CONVOLVE,1,'L',1,1,'CONVOLVE=','(default = Y)')
	IF (CONVOLVE) THEN
	  CALL MESSAGE('Open the file containing the smearing function')
	  FILENAME=FILENAMESMEAR  !  Sets up default in COMMON block
	  CALL OPENCF(22,1,'FILE_22=',0)
	  FILENAMESMEAR=FILENAME
	  CALL USRINP(%REF(ID2),1,'A',6,1,'ID=',
	1 'Enter the ID of the smearing object:  (default = last
	2 smearing entry)')
	  CALL ALPHA(%REF(ID2),1,6,LX,NF,NR,2)
	  CALL RDDATFL1B(22,ID2,SMEARREAD,SMEARRADIUS,MAXSIZE,NPOINTS_S)
C  Set up SMEAR such that the index equals the corresponding radius plus 1.
C  Allow for the possibility that the points are unevenly spaced.
	  DO I=1,NPOINTS_S
	    CALL INTERP1D(SMEARRADIUS,SMEARREAD,NPOINTS_S,REAL(I-1),SMEAR(I))
	  END DO
	  CONVOLVEFACTOR=75.0
	  CALL USRINP(CONVOLVEFACTOR,1,'F',4,1,'CONVOLVE_FACTOR=',
	1 'Enter the convolving factor:  (default = 75.0)')
	END IF
C  Now get the range of data to be fit
	CALL MESSAGE('Enter the starting and ending radii for the 
	1least squares fit:')
	RADII(1)=RADIUS(1)
	RADII(2)=RADIUS(NPOINTS_MAX)
	CALL USRINP(RADII,2,'F',4,1,'RADII=','(default = the entire 
	1range of the data file)')
	IF (.NOT.CONVOLVE.AND.RADII(1).EQ.0.0) THEN
	  CALL MESSAGE('Do you wish to adjust the first data point to
	1 a radius of')
	  ADJUST=.TRUE.
	  CALL USRINP(ADJUST,1,'L',1,1,'ADJUST=',
	1 '0.25 pixels?  (default = Y)')
	END IF
C  Get the number of points, allowing for RADIUS to be variably spaced.
	NPOINTS=0
	DO I=1,NPOINTS_MAX
	  IF (RADIUS(I).GE.RADII(1).AND.RADIUS(I).LE.RADII(2)) NPOINTS
	1 =NPOINTS+1
	END DO
C  Get the range of smearing if convolution is desired
	IF (CONVOLVE) THEN
	  RADIUS_C=RADII(2)
	  CALL MESSAGE('Enter the outer radius to where convolution is
	1 to be done:')
	  CALL USRINP(RADIUS_C,1,'F',4,1,'RADIUS=',
	1 '(default = previous outer fitting radius')
	  NPOINTS_C=RADIUS_C+1  ! Corrects for the point at zero radius
C  This should be ammended in case the data file does not start at the center
	END IF
C  Now convert the radii to index
	DO I=1,NPOINTS
	  IF (RADIUS(I).LE.RADII(1)) IRADII(1)=I
	  IF (RADIUS(I).LE.RADII(2)) IRADII(2)=I
	END DO
	IF (ADJUST) RADIUS(IRADII(1))=0.25
	IF (CHOICE.EQ.'G') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the sigma')
	    CALL MESSAGE('(pixels), and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU) and')
	    CALL MESSAGE('the sigma (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU) and the Hubble "a" (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'MC') THEN
	  IF (SOLVESKY) THEN
C	    CALL MESSAGE('Enter the estimates for the central surface
C	1 brightness (ADU), the Hubble "a"')
C	    CALL MESSAGE('(pixels), the cluster central surface brightness,
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU),')
	    CALL MESSAGE('the cluster central surface brightness,
	1 (ADU),')
	    CALL MESSAGE('and the sky level (ADU):')
	  ELSE
C	    CALL MESSAGE('Enter the estimates for the central surface
C	1 brightness (ADU), the Hubble "a"')
C	    CALL MESSAGE('(pixels), and the cluster central surface brightness,
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU)')
	    CALL MESSAGE('and the cluster central surface brightness,
	1 (ADU)')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'MCX') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the cluster central surface brightness,
	1 (ADU), the cluster Hubble "a"')
	    CALL MESSAGE('(pixels), and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the cluster central surface brightness,
	1 (ADU), and the cluster')
	    CALL MESSAGE('Hubble "a" (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'ML') THEN
	  CALL MESSAGE('Enter the fixed values for the central surface
	1 brightness (ADU), the Hubble "a"')
	  CALL MESSAGE('(pixels), and the sky level (ADU):')
	  CALL USRINP(USERPARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	  CALL MESSAGE('Enter the estimates for the fixed and linear terms')
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1 'for the color gradient:')
	ELSE IF (CHOICE.EQ.'MR') THEN
	  CALL MESSAGE('Enter the fixed values for the central surface
	1 brightness (ADU), the Hubble "a"')
	  CALL MESSAGE('(pixels), and the sky level (ADU):')
	  CALL USRINP(USERPARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	  CALL MESSAGE('Enter the estimates for the fixed and linear terms')
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1 'for the color gradient:')
	ELSE IF (CHOICE.EQ.'HV') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the power and the sky
	1 level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), and the power:')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'I') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the multiplying factor
	1 (ADU pixels) and the')
	    CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1   'sky level (ADU):')
	  ELSE
	    CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1   'Enter the estimate for the multiplying factor:')
	  END IF
	ELSE IF (CHOICE.EQ.'IX') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the multiplying
	1 factor (ADU pixels), the power of the')
	    CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1   'radius, and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the multiplying
	1 factor (ADU pixels) and the power of the')
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=','radius:')
	  END IF
	ELSE IF (CHOICE.EQ.'K') THEN
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1 'Enter the estimate for the constant value:')
	ELSE IF (CHOICE.EQ.'L') THEN
	  CALL MESSAGE('Enter the estimates for the constant and the
	1 slope:')
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'P') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the first and second order
	1 coefficients for "1/a" (pixels), and')
	    CALL MESSAGE('the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU) and the Hubble "a"')
	    CALL MESSAGE('(pixels), and the first and second order
	1 coefficients for "1/a" (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'SI') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the two multiplying
	1 factors (ADU pixels) and the')
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1 'sky level (ADU):')
	  ELSE
	    CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',
	1   'Enter the estimates for the two multiplying factors
	2 (ADU pixels):')
	  END IF
	ELSE IF (CHOICE.EQ.'T') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the taper "b" (pixels), and the
	1 sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), and the taper "b" (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'TX') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the taper "b" (pixels), the taper
	1 power, and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the Hubble "a"')
	    CALL MESSAGE('(pixels), the taper "b" (pixels), and the
	1 taper power:')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'V') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the surface brightness
	1 at 1/2 the total light')
	    CALL MESSAGE('(ADU), the corresponding radius (pixels), and
	1 the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the surface brightness
	1 at 1/2 the total light')
	    CALL MESSAGE('(ADU) and the corresponding radius (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'VC') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the surface brightness
	1 at 1/2 the total light')
C	    CALL MESSAGE('(ADU), the corresponding radius (pixels), the
	    CALL MESSAGE('(ADU), the
	1 cluster surface')
	    CALL MESSAGE('brightness, (ADU), and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the surface brightness
	1 at 1/2 the total light')
C	    CALL MESSAGE('(ADU), the corresponding radius (pixels), and the
	    CALL MESSAGE('(ADU), and the
	1 cluster surface')
	    CALL MESSAGE('brightness, (ADU)')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'VCX') THEN
	  IF (SOLVESKY) THEN
	    CALL MESSAGE('Enter the estimates for the surface brightness
	1 at 1/2 the total light')
	    CALL MESSAGE('(ADU), the corresponding radius (pixels), the
	1 cluster surface')
	    CALL MESSAGE('brightness, (ADU), the cluster radius
	1 (pixels), and the sky level (ADU):')
	  ELSE
	    CALL MESSAGE('Enter the estimates for the surface brightness
	1 at 1/2 the total light')
	    CALL MESSAGE('(ADU), the corresponding radius (pixels), the
	1 cluster surface')
	    CALL MESSAGE('brightness, (ADU), and the cluster radius
	1 (pixels):')
	  END IF
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'E3') THEN
	  CALL MESSAGE('Enter the estimates for the total magnitude and
	1 the Hubble a')
	  CALL USRINP(PARAMETRS,6,'F',4,0,'PARAMETERS=',' ')
	END IF
	DO I=1,6
	  PARAMETRS_INIT(I)=PARAMETRS(I)
	END DO
C  Get the weighting scheme
  67	WEIGHT_SCHEME='R'
	CALL MESSAGE('Select the weighting scheme:')
	CALL MESSAGE('     R = Weight equal to circumference/(noise)**2 
	1(default)')
	CALL MESSAGE('     S = Weight equal to the square root of
	1 the circumference')
	CALL MESSAGE('     N = Weight equal to 1/(noise)**2')
	CALL MESSAGE('     P = Weight equal to weight from file/
	1(constant*noise**2)')
	CALL USRINP(%REF(WEIGHT_SCHEME),1,'A',1,1,'WEIGHT_SCHEME=',' ')
	IF (WEIGHT_SCHEME.NE.'R'.AND.WEIGHT_SCHEME.NE.'S'.AND.
	1WEIGHT_SCHEME.NE.'N'.AND.WEIGHT_SCHEME.NE.'P') GO TO 67
C  Open the file for the weights
	IF (WEIGHT_SCHEME.EQ.'P') THEN
	  CALL MESSAGE('Open the file containing the weights')
	  FILENAME=FILENAMEWEIGHTS
	  CALL OPENCF(23,1,'FILE_23=',0)
	  FILENAMEWEIGHTS=FILENAME
	  REWIND 23
	  READ(23,1000)
1000	  FORMAT(' RADIUS(PIXELS) EXPOSURE(SECONDS)')
	  DO I=1,MAXSIZE
	    READ(23,1002,END=68) R,FILEWEIGHT(I)
1002	    FORMAT(' ',3X,F7.2,8X,F9.3)
	  END DO
  68	  CALL MESSAGE('Enter the constant to normalize the weights
	1 from the file;  this should be')
	  CALL MESSAGE('the exposure of the initial file from which the
	1 weights were calculated:')
	  CONSTANT=2700.0
	  CALL USRINP(CONSTANT,1,'F',4,1,'CONSTANT=',
	1 '(default = 2700')
	END IF
C  Get the readout noise in order to compute the weights
	READOUT(1)=63.9
	READOUT(2)=21.3
	IF (WEIGHT_SCHEME.NE.'S') THEN
	  CALL MESSAGE('Enter the readout noise in electrons and the
	1 gain in electrons per ADU:')
	  CALL USRINP(READOUT,2,'F',4,1,'NOISE_PARAMETERS=',
	1 '(defaults = 63.9 21.3)')
	END IF
C  Get the iteration scheme
	ITERATION_SCHEME=' '
	DO WHILE (ITERATION_SCHEME.NE.'M'.AND.ITERATION_SCHEME.NE.'A'.AND.
	1ITERATION_SCHEME.NE.'B'.AND.ITERATION_SCHEME.NE.'C')
	  ITERATION_SCHEME='M'
	  CALL MESSAGE('Select the iteration scheme:')
	  CALL MESSAGE('     M = Manual (default)')
	  CALL MESSAGE('     A = Automatic selection of fractional
	1 increment with')
	  CALL MESSAGE('         maximum searching')
	  CALL MESSAGE('     B = Automatic selection of fractional
	1 increment with')
	  CALL MESSAGE('         minimum searching')
	  CALL MESSAGE('     C = Automatic selection of fractional
	1 increment with')
	  CALL MESSAGE('         no searching')
	  CALL USRINP(%REF(ITERATION_SCHEME),1,'A',1,1,
	1 'ITERATION_SCHEME=',' ')
	END DO
C  Get the tolerance for stopping the iteration for scheme A or B.
	IF (ITERATION_SCHEME.EQ.'A'.OR.ITERATION_SCHEME.EQ.'B'.OR.
	1ITERATION_SCHEME.EQ.'C') THEN
	  TOLERANCE=.0001
	  CALL USRINP(TOLERANCE,1,'F',4,1,'TOLERANCE=',
	1 'Enter the tolerance for continuing the iteration: 
	2 (default = 0.0001)')
	END IF
C  Get the fractional increment for the first iteration
	FRACINCR=1.0
	IF (ITERATION_SCHEME.EQ.'M'.AND.CHOICE.NE.'K'.AND.CHOICE.NE.'L')
	1THEN
	  CALL USRINP(FRACINCR,1,'F',4,1,'FRACTIONAL_INCREMENT=',
	1 'Enter the fractional increment:  (default = 1.0)')
	END IF
	VFACTOR=3.33*LOG(10.0)
	FIRSTIME=.TRUE.
	NTIMES=0
C  Start of the loop for solving the normal equations which have been
C  linearized by expansion in a Taylor series.
  70	NTIMES=NTIMES+1
C  Perform the convolution if this was asked for
	IF ((FIRSTIME.AND.CONVOLVE).OR.(CONVOLVE.AND.(.NOT.FIRSTIME)
	1.AND.PARAMETRSCHANGED)) THEN
	  IRADIICONVOLVE(1)=IRADII(1)
	  EVALUATE=.TRUE.
	  RADIUSINCONVOLVE(1)=RADII(1)
	  IF (RADIUSINCONVOLVE(1).NE.0.0) THEN
	    CALL ERROR(-1,'The object to be convolved does not
	1 begin at its center')
	  END IF
	  I=1
	  IF (CHOICE.EQ.'G'.OR.CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'
C	1 .OR.CHOICE.EQ.'MC'.OR.CHOICE.EQ.'MCX'.OR.CHOICE.EQ.'T'.OR.CHOICE
	1 .OR.CHOICE.EQ.'MCX'.OR.CHOICE.EQ.'T'.OR.CHOICE
	2 .EQ.'TX'.OR.CHOICE.EQ.'HV'.OR.CHOICE.EQ.'P') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	    DO WHILE ((RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1   +1.0).AND.(I.LT.MAXSIZE))  ! One larger than outer radius
				      ! for interpolation in CONVLV2DG
	      RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1     MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/PARAMETRS(2))**3*
	1     PARAMETRS(2)/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	      I=I+1
	    END DO
	    NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2))+10.0)/20.0)*
	1   96.0)
	  ELSE IF (CHOICE.EQ.'MC') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	    DO WHILE ((RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1   +1.0).AND.(I.LT.MAXSIZE))  ! One larger than outer radius
				      ! for interpolation in CONVLV2DG
	      RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1     MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/USERPARAMETRS(1))**3*
	1     USERPARAMETRS(1)/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	      I=I+1
	    END DO
	    NANGLES=24+INT((1.0-(MIN(10.0,USERPARAMETRS(1))+10.0)/20.0)*
	1   96.0)
	  ELSE IF (CHOICE.EQ.'ML'.OR.CHOICE.EQ.'MR') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	    DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1   +1.0)  ! One larger than outer radius
		   ! for interpolation in CONVLV2DG
	      RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1     MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/USERPARAMETRS(2))**3*
	1     USERPARAMETRS(2)/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	      I=I+1
	    END DO
	    NANGLES=24+INT((1.0-(MIN(10.0,USERPARAMETRS(2))+10.0)/20.0)*
	1   96.0)
C	  ELSE IF (CHOICE.EQ.'V'.OR.CHOICE.EQ.'VC'.OR.CHOICE.EQ.'VCX') THEN
	  ELSE IF (CHOICE.EQ.'V'.OR.CHOICE.EQ.'VCX') THEN
	    DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1   +1.0)
	      RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1     MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/PARAMETRS(2))**3*
	1     PARAMETRS(2)/10.0/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	      I=I+1
	    END DO
	    NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2)/10.0)+10.0)/20.0)
	1   *96.0)
	    ELSE IF (CHOICE.EQ.'VC') THEN
	      DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1     +1.0)
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/USERPARAMETRS(1))**3*
	2       USERPARAMETRS(1)/10.0/(RADIUSINCONVOLVE(I)+1.0)/
	3       CONVOLVEFACTOR))
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,USERPARAMETRS(1)/10.0)+10.0)
	1     /20.0)*96.0)
	  ELSE
	    DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1   +1.0)
	      RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+1.0
	      I=I+1
	    END DO
	    NANGLES=24
	  END IF
	  IRADIICONVOLVE(2)=I-1
	  DO NN=IRADIICONVOLVE(1),IRADIICONVOLVE(2)
C  Evaluate the formula at the precomputed radii
	    CALL RADGALFUN(RADIUSINCONVOLVE,NN,PARAMETRS,CHOICE,
	1   EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	    ARRAYINCONVOLVE(NN)=TERM
	  END DO
	  CALL CONVLV2DG(ARRAYINCONVOLVE,RADIUSINCONVOLVE,MAXSIZE,
	1 SMEAR,NPOINTS_S,SURFBCONVOLVE,MAX(NPOINTS,NPOINTS_C),
	2 NPOINTS_C,NANGLES)
C	XLABEL='RADIUS'
C	YLABEL='ADU'
C	TITLE='CONVOLUTION-ORIGINAL'
C	DO II=1,NPOINTS
C	  CALL RADGALFUN(RADIUS,II,PARAMETRS,CHOICE,
C	1 EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
C	  YARRAY(II)=TERM-SURFBCONVOLVE(II)
C	END DO
C	CALL XYPLOT(RADIUS,YARRAY,
C	1MAXSIZE,1,NPOINTS,0,1,
C	10,0,INVERT,COLOR,.FALSE.,.TRUE.,XLABEL,
C	2YLABEL,TITLE,' ',.FALSE.)
	END IF
	EVALUATE=.TRUE.
	IF (CONVOLVE) EVALUATE=.FALSE.
	DO 500 ITERATION=1,1
C  Initialize the sums for the normal equations
	  DO 80 J=1,5
	    SUMRIGHT(J)=0.0
	    DO 80 I=1,5
	      SUM(J,I)=0.0
  80	  CONTINUE
C  Initialize the mean error
	  SIGMASUM(1)=0.0
	  SIGMASUM(2)=0.0
	  SUMWEIGHT=0.0
C  Compute the derivatives for the normal equations.
C  Convolution is in some cases ignored in computing the derivatives
C  in order to save time
	  CALL RADGALDER(SURFBCONVOLVE,MAXSIZE,6,CHOICE,
	1 PARAMETRS,USERPARAMETRS,CONVOLVE,SOLVESKY,ARRAYINCONVOLVE,
	2 RADIUSINCONVOLVE,RADIUS,IRADII,IRADIICONVOLVE,SMEAR,
	3 NPOINTS_S,NPOINTS,NPOINTS_C,NANGLES,DERIV)
C  Compute the sums for the normal equations
	  DO 200 N=IRADII(1),IRADII(2)
	    IF (CHOICE.EQ.'G'.OR.CHOICE.EQ.'H'.OR.CHOICE.EQ.'IX'
C	1   .OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'SI'.OR.CHOICE.EQ.'V') THEN
	1   .OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'SI'.OR.CHOICE.EQ.'V'.OR.CHOICE.EQ.
	2   'MC'.OR.CHOICE.EQ.'VC') THEN
	      NUMPARAM=2
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	      IF (SOLVESKY) THEN
	        NUMPARAM=NUMPARAM+1
	        SURFBRIGHT=PARAMETRS(3)+SURFBRIGHT
	      END IF
	    ELSE IF (CHOICE.EQ.'MCX'.OR.CHOICE.EQ.'VCX') THEN
	      NUMPARAM=4
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	      IF (SOLVESKY) THEN
	        NUMPARAM=NUMPARAM+1
	        SURFBRIGHT=PARAMETRS(5)+SURFBRIGHT
	      END IF
	    ELSE IF (CHOICE.EQ.'ML'.OR.CHOICE.EQ.'MR') THEN
	      NUMPARAM=2
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
C	    ELSE IF (CHOICE.EQ.'HV'.OR.CHOICE.EQ.'MC'.OR.CHOICE.EQ.'VC') THEN
	    ELSE IF (CHOICE.EQ.'HV') THEN
	      NUMPARAM=3
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	      IF (SOLVESKY) THEN
	        NUMPARAM=NUMPARAM+1
	        SURFBRIGHT=PARAMETRS(4)+SURFBRIGHT
	      END IF
	    ELSE IF (CHOICE.EQ.'I') THEN
	      NUMPARAM=1
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	      IF (SOLVESKY) THEN
	        NUMPARAM=NUMPARAM+1
	        SURFBRIGHT=PARAMETRS(2)+SURFBRIGHT
	      END IF
	    ELSE IF (CHOICE.EQ.'K') THEN
	      NUMPARAM=1
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	    ELSE IF (CHOICE.EQ.'L') THEN
	      NUMPARAM=2
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	    ELSE IF (CHOICE.EQ.'T') THEN
	      NUMPARAM=3
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	      IF (SOLVESKY) THEN
	        NUMPARAM=NUMPARAM+1
	        SURFBRIGHT=PARAMETRS(4)+SURFBRIGHT
	      END IF
	    ELSE IF (CHOICE.EQ.'TX') THEN
	      NUMPARAM=4
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	      IF (SOLVESKY) THEN
	        NUMPARAM=NUMPARAM+1
	        SURFBRIGHT=PARAMETRS(5)+SURFBRIGHT
	      END IF
	    ELSE IF (CHOICE.EQ.'E3') THEN
	      NUMPARAM=2
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      SURFBRIGHT=TERM
	    END IF
C  Compute the weights for the normal equations and the initial sigmas
C  based on readout noise and shot noise and the number of pixels in
C  each contour.
	    IF (WEIGHT_SCHEME.EQ.'N'.OR.WEIGHT_SCHEME.EQ.'P'.OR.
	1   WEIGHT_SCHEME.EQ.'R') THEN
c	      WEIGHT=1.0/(READOUT(1)**2+
c	1     SURFBRIGHTOB(N)*READOUT(2))
C  Weight in units of 1/sigma(ADU)**2
	      WEIGHT=READOUT(2)**2/
	1     (READOUT(1)**2+SURFBRIGHTOB(N)*READOUT(2))
	      IF (WEIGHT_SCHEME.EQ.'R') THEN
	        WEIGHT=WEIGHT*AMAX1(1.0,TWOPI*RADIUS(N))
	      ELSE IF (WEIGHT_SCHEME.EQ.'N') THEN
	        WEIGHT=WEIGHT*320.0  ! Assume a full width of the CCD
	      ELSE IF (WEIGHT_SCHEME.EQ.'P') THEN
	        WEIGHT=WEIGHT*320.0*FILEWEIGHT(N)/CONSTANT
	      END IF
	    ELSE IF (WEIGHT_SCHEME.EQ.'S') THEN  ! Square root of radius
						 ! only
	      WEIGHT=SQRT(RADIUS(N)+1.0)
	    END IF
	    SUMWEIGHT=WEIGHT+SUMWEIGHT
	    IF (SOLVESKY) THEN
	      OBSURFBRIGHT=SURFBRIGHTOB(N)
	    ELSE 
	      OBSURFBRIGHT=SURFBRIGHTOB(N)-SKY
	    END IF
	    DO 190 J=1,NUMPARAM  ! Loop for rows in matrix
	      SUMRIGHT(J)=WEIGHT*DERIV(N,J)*(OBSURFBRIGHT-SURFBRIGHT)+
	1     SUMRIGHT(J)
	      DO 190 I=1,NUMPARAM  ! Loop for columns
	        SUM(J,I)=WEIGHT*DERIV(N,J)*DERIV(N,I)+SUM(J,I)
	        SUMRIGHT(J)=WEIGHT*DERIV(N,J)*DERIV(N,I)*PARAMETRS(I)+
	1       SUMRIGHT(J)
 190	    CONTINUE
	    IF (FIRSTIME.AND.ITERATION.EQ.1) THEN
	      SIGMASUM(1)=(SURFBRIGHT-OBSURFBRIGHT)**2*WEIGHT
	1     +SIGMASUM(1)
	    END IF
 200	  CONTINUE
	  NUMRADII=IRADII(2)-IRADII(1)+1
C  The first time through, calculate the mean error for the
C  initial guess
	  IF (FIRSTIME.AND.ITERATION.EQ.1) THEN
	    SIGMAADU=SQRT(SIGMASUM(1)*NUMRADII/
	1   ((NUMRADII-NUMPARAM)*SUMWEIGHT))
	    SIGMA_INIT=SIGMAADU
	    SIGMASIGMA=SIGMAADU/SQRT(2.0*(NUMRADII-NUMPARAM))
	    CALL MESSAGE('For the initial guess:')
	    WRITE(6,1300) SIGMAADU,SIGMASIGMA
	    WRITE(7,1300) SIGMAADU,SIGMASIGMA
	  END IF
	  IF (ITERATION.EQ.1) THEN
	  END IF
C  Load the array UPTRIPART with the upper right part of SUM in order
C  to solve the equations
	  K=0
	  DO 210 J=1,NUMPARAM
	    DO 210 I=1,J
	      K=K+1
	      UPTRIPART(K)=SUM(I,J)
 210	  CONTINUE
	  IER=1.0E-14
	  CALL SYMINV(SUMRIGHT,UPTRIPART,NUMPARAM,1,EPS,IER,AUX)
	  IF (IER.GT.0) WRITE(6,1200) IER+1
	  IF (IER.GT.0) WRITE(7,1200) IER+1
1200	  FORMAT(' Warning:  Possible loss of significance indicated at 
	1 elimination step',I4/' in SYMINV routine, where pivot element
	1 was less than the internal tolerance'/ 'EPS=1.E-14 set up in 
	1 RADGAL')
	  IF (IER.EQ.-1) CALL MESSAGE('Attention:  SYMINV failed to
	1 solve system of linear equations')
	  IF (IER.EQ.-1) RETURN
C  Adjust the solution by the fractional increment
	  DO I=1,NUMPARAM
	    PARAMETRS(I)=PARAMETRS(I)+(SUMRIGHT(I)-PARAMETRS(I))*
	1   FRACINCR
C	PRINT *, SUMRIGHT(I),PARAMETRS(I),FRACINCR,'SUMR,PAR,FR'
C  Avoid unnatural solutions
	    IF (CHOICE.EQ.'TX') THEN
	      PARAMETRS(I)=ABS(PARAMETRS(I))
	    END IF
	  END DO
	  IF (CHOICE.EQ.'MH') THEN
	    IF (PARAMETRS(2).LT.0.0) PARAMETRS(2)=0.5*ABS(PARAMETRS(2))
	  END IF
C  Compute the mean error for an observation of average weight
C  for the solution
	  SIGMASUM(1)=0.0
	  SUMWEIGHT=0.0
	  SUMLOGWEIGHT=0.0
	  IF (CONVOLVE) THEN
	    IRADIICONVOLVE(1)=IRADII(1)
	    EVALUATE=.TRUE.
	    RADIUSINCONVOLVE(1)=RADII(1)
	    IF (RADIUSINCONVOLVE(1).NE.0.0) THEN
	      CALL ERROR(-1,'The object to be convolved does not
	1   begin at its center')
	    END IF
	    I=1
	    IF (CHOICE.EQ.'G'.OR.CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'
C	1   .OR.CHOICE.EQ.'MC'.OR.CHOICE.EQ.'MCX'.OR.CHOICE.EQ.'T'.OR.CHOICE
	1   .OR.CHOICE.EQ.'MCX'.OR.CHOICE.EQ.'T'.OR.CHOICE
	2   .EQ.'TX'.OR.CHOICE.EQ.'HV'.OR.CHOICE.EQ.'P') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	      DO WHILE ((RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1     +1.0).AND.(I.LT.MAXSIZE))  ! One larger for interpolation in
				         ! CONVLV2DG
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/PARAMETRS(2))**3*
	1       PARAMETRS(2)/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2))+10.0)/20.0)*
	1     96.0)
	  ELSE IF (CHOICE.EQ.'MC') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	    DO WHILE ((RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1   +1.0).AND.(I.LT.MAXSIZE))  ! One larger than outer radius
				      ! for interpolation in CONVLV2DG
	      RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1     MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/USERPARAMETRS(1))**3*
	1     USERPARAMETRS(1)/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	      I=I+1
	    END DO
	    NANGLES=24+INT((1.0-(MIN(10.0,USERPARAMETRS(1))+10.0)/20.0)*
	1   96.0)
	    ELSE IF (CHOICE.EQ.'ML'.OR.CHOICE.EQ.'MR') THEN
C  Set up RADIUSINCONVOLVE with smaller increments near the center of
C  the galaxy.
	      DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1     +1.0)  ! One larger for interpolation in
		     ! CONVLV2DG
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)/USERPARAMETRS(2))**3*
	1       USERPARAMETRS(2)/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,USERPARAMETRS(2))+10.0)/20.0)*
	1     96.0)
C	    ELSE IF (CHOICE.EQ.'V'.OR.CHOICE.EQ.'VC'.OR.CHOICE.EQ.'VCX') THEN
	    ELSE IF (CHOICE.EQ.'V'.OR.CHOICE.EQ.'VCX') THEN
	      DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1     +1.0)
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/PARAMETRS(2))**3*
	1       PARAMETRS(2)/10.0/(RADIUSINCONVOLVE(I)+1.0)/CONVOLVEFACTOR))
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,PARAMETRS(2)/10.0)+10.0)
	1     /20.0)*96.0)
	    ELSE IF (CHOICE.EQ.'VC') THEN
	      DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1     +1.0)
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+MIN(1.0,
	1       MAX(0.001,(1.0+RADIUSINCONVOLVE(I)*10.0/USERPARAMETRS(1))**3*
	2       USERPARAMETRS(1)/10.0/(RADIUSINCONVOLVE(I)+1.0)/
	3       CONVOLVEFACTOR))
	        I=I+1
	      END DO
	      NANGLES=24+INT((1.0-(MIN(10.0,USERPARAMETRS(1)/10.0)+10.0)
	1     /20.0)*96.0)
	    ELSE
	      DO WHILE (RADIUSINCONVOLVE(I).LT.MAX(RADII(2),RADIUS_C)
	1     +1.0)
	        RADIUSINCONVOLVE(I+1)=RADIUSINCONVOLVE(I)+1.0
	        I=I+1
	      END DO
	      NANGLES=24
	    END IF
	    IRADIICONVOLVE(2)=I-1
	    DO NN=IRADIICONVOLVE(1),IRADIICONVOLVE(2)
C  Evaluate the formula
	      CALL RADGALFUN(RADIUSINCONVOLVE,NN,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      ARRAYINCONVOLVE(NN)=TERM
	    END DO
	    CALL CONVLV2DG(ARRAYINCONVOLVE,RADIUSINCONVOLVE,MAXSIZE,
	1   SMEAR,NPOINTS_S,SURFBCONVOLVE,MAX(NPOINTS,NPOINTS_C),
	2   NPOINTS_C,NANGLES)
C	XLABEL='RADIUS'
C	YLABEL='ADU'
C	TITLE='CONVOLUTION-ORIGINAL'
C	DO II=1,NPOINTS
C	  CALL RADGALFUN(RADIUS,II,PARAMETRS,CHOICE,
C	1 EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
C	  YARRAY(II)=TERM-SURFBCONVOLVE(II)
C	END DO
C	CALL XYPLOT(RADIUS,YARRAY,
C	1MAXSIZE,1,NPOINTS,0,1,
C	10,0,INVERT,COLOR,.FALSE.,.TRUE.,XLABEL,
C	2YLABEL,TITLE,' '.FALSE>)
	  END IF
	  IF (SOLVESKY) THEN
	    IF (CHOICE.EQ.'G'.OR.CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'
	1   .OR.CHOICE.EQ.'V') SKY=PARAMETRS(3)
	  END IF
	  EVALUATE=.TRUE.
	  IF (CONVOLVE) EVALUATE=.FALSE.
	  DO 400 N=IRADII(1),IRADII(2)
	    CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1   EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	    SURFBRIGHT=TERM
	    IF (SOLVESKY) THEN
	      SURFBRIGHT=PARAMETRS(NUMPARAM)+SURFBRIGHT
	    END IF
C  Compute the weights for the sigmas
	    IF (WEIGHT_SCHEME.EQ.'N'.OR.WEIGHT_SCHEME.EQ.'P'.OR.
	1   WEIGHT_SCHEME.EQ.'R') THEN
	      WEIGHT=READOUT(2)**2/(READOUT(1)**2+
	1     SURFBRIGHTOB(N)*READOUT(2))
	      IF (WEIGHT_SCHEME.EQ.'R') THEN
	        WEIGHT=WEIGHT*AMAX1(1.0,TWOPI*RADIUS(N))
	      ELSE IF (WEIGHT_SCHEME.EQ.'N') THEN
	        WEIGHT=WEIGHT*320
	      ELSE IF (WEIGHT_SCHEME.EQ.'P') THEN
	        WEIGHT=WEIGHT*320.0*FILEWEIGHT(N)/CONSTANT
	      END IF
	    ELSE IF (WEIGHT_SCHEME.EQ.'S') THEN  ! Square root of radius
						 ! only
	      WEIGHT=SQRT(RADIUS(N))
	    END IF
	    LOGWEIGHT=LOG10(WEIGHT)
	    SUMWEIGHT=WEIGHT+SUMWEIGHT
	    SUMLOGWEIGHT=LOGWEIGHT+SUMLOGWEIGHT
	    IF (SOLVESKY) THEN
	      OBSURFBRIGHT=SURFBRIGHTOB(N)
	    ELSE 
	      OBSURFBRIGHT=SURFBRIGHTOB(N)-SKY
	    END IF
	    OBMINUSCALC(N,1)=OBSURFBRIGHT-SURFBRIGHT
C  Calculate the residuals in magnitudes
	    IF (SOLVESKY) THEN
	      IF (ABS(OBSURFBRIGHT-SKY).GT.0.0.AND.SURFBRIGHT-SKY.GT.0.0)
	1     THEN
	        OBMINUSCALC(N,2)=ABS(2.5*LOG10(ABS((OBSURFBRIGHT-SKY)/
	1       (SURFBRIGHT-SKY))))
	      ELSE
	        OBMINUSCALC(N,2)=0.0
	      END IF
	    ELSE
	      OBMINUSCALC(N,2)=ABS(2.5*LOG10(ABS(OBSURFBRIGHT/
	1     SURFBRIGHT)))
	    END IF
	    SIGMASUM(1)=OBMINUSCALC(N,1)**2*WEIGHT
	1   +SIGMASUM(1)
	    SIGMASUM(2)=OBMINUSCALC(N,2)**2*ABS(LOGWEIGHT)
	1   +SIGMASUM(2)
 400	  CONTINUE
	  AVWEIGHT=SUMWEIGHT/NUMRADII
	  AVLOGWEIGHT=SUMLOGWEIGHT/NUMRADII
	  REDCHISQR=(SIGMASUM(1)/(NUMRADII-NUMPARAM))
	  CALL MESSAGE(' ')
	  SIGMA=SQRT(SIGMASUM(1)/((NUMRADII-NUMPARAM)*AVWEIGHT))
	  SIGMAADU=SIGMA
	  SIGMASIGMA=SIGMA/SQRT(2*(NUMRADII-NUMPARAM))
	  CALL MESSAGE('For the solution:')
	  IF (ITERATION_SCHEME.EQ.'A'.OR.ITERATION_SCHEME.EQ.
	1 'B'.OR.ITERATION_SCHEME.EQ.'C') THEN
	    WRITE (6,1301) FRACINCR
	    WRITE (7,1301) FRACINCR
1301	    FORMAT(' The fractional increment = ',F15.8)
	  END IF
	  WRITE(6,1300) SIGMA,SIGMASIGMA
1300	  FORMAT(' Mean error for an observation of average weight
	1 = '/G18.6,  '   +-',G18.6)
	  WRITE(7,1300) SIGMA,SIGMASIGMA
	  IF (CHOICE.NE.'E3') THEN
	    SIGMA=SQRT(SIGMASUM(2)/((NUMRADII-NUMPARAM)*AVLOGWEIGHT))
	    SIGMASIGMA=SIGMA/SQRT(2*(NUMRADII-NUMPARAM))
	    WRITE(6,1305) SIGMA,SIGMASIGMA
1305	    FORMAT('                         and in magnitudes it is
	1 = '/G18.6,  '   +-',G18.6)
	    WRITE(7,1305) SIGMA,SIGMASIGMA
	  END IF
C  Compute the mean errors of each parameter
	  CALL DETERMNT(SUM,6,NUMPARAM,DET,KPVT)
	  DENOM=DET(1)
	  IF (DENOM.EQ.0.0) THEN
	    CALL ERROR(-1,'Determinant was zero')
	    DENOM=DENOM+1.0
	  END IF
	  DO 460 I=1,NUMPARAM
	    CALL COFACTOR(SUM,MINORARRAY,6,I,I,NUMPARAM,MINOR,SIGN,
	1   KPVT)
	    NUMER=MINOR(1)*SIGN
	    IF (ABS(MINOR(2)-DET(2)).GT.74) THEN
	      CALL ERROR(-1,'Determinant and/or cofactor beyond
	1     allowable range')
	      SIGMAPARAM(I)=0.0
	    ELSE
	      SIGMAPARAM(I)=SIGMAADU*SQRT(AVWEIGHT*
	1     ABS(NUMER/DENOM))*10.0**(0.5*(MINOR(2)-DET(2)))
	    END IF
 460	  CONTINUE
	  IF (CHOICE.EQ.'G') THEN
1309	    FORMAT(' Central surface brightness = ',F16.5,'   +-',F16.5/
	1   ' Sigma = ',F16.5,'   +-',F16.5)
	    WRITE (6,1309) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1309) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(3)  ! For the integration below
1310	      FORMAT(' Sky surface brightness = ',F16.5,'   +-',F16.5)
	      WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	      WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	    END IF
	  ELSE IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'MCX') THEN
C	  ELSE IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'.OR.CHOICE(1:2).EQ.'MC') THEN
1311	    FORMAT(' Central surface brightness = ',F16.5,'   +-',F16.5/
	1   ' Hubble "a" = ',F19.5,'   +-',F19.5)
	    WRITE (6,1311) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1311) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    IF (CHOICE.EQ.'MCX') THEN
1312	      FORMAT(' Cluster central surface brightness = ',F15.6,'  +-',
	1     F15.6/' Cluster Hubble "a" = ',F16.4,'   +-',F16.4)
	      WRITE (6,1312) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	      WRITE (7,1312) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(5)  ! For the integration below
	        WRITE (6,1310) PARAMETRS(5),SIGMAPARAM(5)
	        WRITE (7,1310) PARAMETRS(5),SIGMAPARAM(5)
	      END IF
C	    ELSE IF (CHOICE.EQ.'MC') THEN
C1313	      FORMAT(' Cluster central surface brightness = ',F15.6,'  +-',
C	1     F15.6)
C	      WRITE (6,1313) PARAMETRS(3),SIGMAPARAM(3)
C	      WRITE (7,1313) PARAMETRS(3),SIGMAPARAM(3)
C	      IF (SOLVESKY) THEN
C	        SKY=PARAMETRS(4)  ! For the integration below
C	        WRITE (6,1310) PARAMETRS(4),SIGMAPARAM(4)
C	        WRITE (7,1310) PARAMETRS(4),SIGMAPARAM(4)
C	      END IF
	    ELSE
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(3)  ! For the integration below
	        WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	        WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	      END IF
	    END IF
	  ELSE IF (CHOICE.EQ.'MC') THEN
	    WRITE (6,1414) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
1414	    FORMAT(' Central surface brightness = ',F16.5,'  +-',G16.5/
	1   ' Cluster central surface brightness = ',F16.5,'  +-',G16.5)
	    WRITE (7,1414) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(3)  ! For the integration below
	        WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	        WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	      END IF
	  ELSE IF (CHOICE.EQ.'ML'.OR.CHOICE.EQ.'MR') THEN
1314	    FORMAT(' Constant term = ',F16.5,'   +-',F16.5/
	1   ' Linear term = ',F16.5,'   +-',F16.5)
	    WRITE (6,1314) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1314) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  ELSE IF (CHOICE.EQ.'HV') THEN
1315	    FORMAT(' Central surface brightness = ',F16.5,'   +-',G16.5/
	1   ' Hubble "a" = ',F16.5,'   +-',G16.5/
	2   ' Power = ',F16.5,'  +-',G16.5)
	    WRITE (6,1315) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	    WRITE (7,1315) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(4)  ! For the integration below
	      WRITE (6,1310) PARAMETRS(4),SIGMAPARAM(4)
	      WRITE (7,1310) PARAMETRS(4),SIGMAPARAM(4)
	    END IF
	  ELSE IF (CHOICE.EQ.'I') THEN
1320	    FORMAT(' Multiplying factor = ',F16.5,'  +-',G16.5)
	    WRITE (6,1320) PARAMETRS(1),SIGMAPARAM(1)
	    WRITE (7,1320) PARAMETRS(1),SIGMAPARAM(1)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(2)
	      WRITE (6,1310) PARAMETRS(2),SIGMAPARAM(2)
	      WRITE (7,1310) PARAMETRS(2),SIGMAPARAM(2)
	    END IF
	  ELSE IF (CHOICE.EQ.'IX') THEN
1321	    FORMAT(' Multiplying factor = ',F16.5,'  +-',F16.5/
	1   ' Power of radius = ',F16.5,'  +-',F16.5)
	    WRITE (6,1321) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1321) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(3)
	      WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	      WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	    END IF
	  ELSE IF (CHOICE.EQ.'K') THEN
1330	    FORMAT(' Constant = ',F15.6,'   +-',G15.6)
	    WRITE (6,1330) (PARAMETRS(I),SIGMAPARAM(I),I=1,1)
	    WRITE (7,1330) (PARAMETRS(I),SIGMAPARAM(I),I=1,1)
	  ELSE IF (CHOICE.EQ.'L') THEN
1331	    FORMAT(' Constant = ',F15.6,'   +-',G15.6/' Slope = ',G15.6,
	1   '   +-',G15.6)
	    WRITE (6,1331) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1331) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  ELSE IF (CHOICE.EQ.'SI') THEN
1322	    FORMAT(' First order multiplying factor = ',F16.5,' +-',
	1   F16.5/' Second order multiplying factor = ',F16.5,' +-',
	2   F16.5)
	    WRITE (6,1322) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1322) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(3)
	      WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	      WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	    END IF
	  ELSE IF (CHOICE.EQ.'T') THEN
1340	    FORMAT(' Central surface brightness = ',F16.5,'   +-',F16.5/
	1   ' Hubble "a" = ',F16.5,'   +-',F16.5/' Taper = ',F16.5,
	2   '   +-',F16.5)
	    WRITE (6,1340) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	    WRITE (7,1340) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(4)
	      WRITE (6,1310) PARAMETRS(4),SIGMAPARAM(4)
	      WRITE (7,1310) PARAMETRS(4),SIGMAPARAM(4)
	    END IF
	  ELSE IF (CHOICE.EQ.'TX') THEN
1341	    FORMAT(' Central surface brightness = ',F16.5,'   +-',F16.5/
	1   ' Hubble "a" = ',F16.5,'   +-',F16.5/' Taper = ',F16.5,
	2   '   +-',F16.5/' Taper power = ',F16.5,'   +-',F16.5)
	    WRITE (6,1341) (PARAMETRS(I),SIGMAPARAM(I),I=1,4)
	    WRITE (7,1341) (PARAMETRS(I),SIGMAPARAM(I),I=1,4)
	    IF (SOLVESKY) THEN
	      SKY=PARAMETRS(5)
	      WRITE (6,1310) PARAMETRS(5),SIGMAPARAM(5)
	      WRITE (7,1310) PARAMETRS(5),SIGMAPARAM(5)
	    END IF
	  ELSE IF (CHOICE.EQ.'V'.OR.CHOICE.EQ.'VCX') THEN
1350	    FORMAT(' Effective surface brightness = ',F16.5,'   +-',
	1   F16.5/' Effective radius = ',F16.5,'   +-',F16.5)
	    WRITE (6,1350) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1350) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    IF (CHOICE.EQ.'VCX') THEN
1351	      FORMAT(' Cluster effective surface brightness = ',F15.6,
	1     '   +-',F15.6/' Cluster effective radius = ',F16.4,'   +-',
	2     F16.4)
	      WRITE (6,1351) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	      WRITE (7,1351) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(5)  ! Used below in integration
	        WRITE (6,1310) PARAMETRS(5),SIGMAPARAM(5)
	        WRITE (7,1310) PARAMETRS(5),SIGMAPARAM(5)
	      END IF
	    ELSE IF (CHOICE.EQ.'VC') THEN
1352	      FORMAT(' Cluster effective surface brightness = ',F16.6,
	1     '   +-',F16.6)
	      WRITE (6,1352) PARAMETRS(3),SIGMAPARAM(3)
	      WRITE (7,1352) PARAMETRS(3),SIGMAPARAM(3)
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(4)  ! Used below in integration
	        WRITE (6,1310) PARAMETRS(4),SIGMAPARAM(4)
	        WRITE (7,1310) PARAMETRS(4),SIGMAPARAM(4)
	      END IF
	    ELSE
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(3)  ! Used below in integration
	        WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	        WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	      END IF
	    END IF
	  ELSE IF (CHOICE.EQ.'VC') THEN
1450	    FORMAT(' Effective surface brightness = ',F16.5,'   +-',
	1   F16.5/' Cluster effective surface brightness = ',F16.5,'
	2   +-',F16.5)
	    WRITE (6,1450) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1450) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	      IF (SOLVESKY) THEN
	        SKY=PARAMETRS(3)  ! Used below in integration
	        WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	        WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	      END IF
	  ELSE IF (CHOICE.EQ.'E3') THEN
1363	    FORMAT(' Total magnitude = ',F9.4,'   +-',
	1   F9.4/' Radius-parameter = ',F9.5,'   +-',F9.5)
	    WRITE (6,1363) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	    WRITE (7,1363) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  END IF
 500	CONTINUE
C  Get the fractional increment for the next pass and readjust the
C  parameters as necessary depending on the iteration scheme.
	DO I=1,NUMPARAM
	  PARAMETRS_PRIOR(I)=PARAMETRS(I)
	END DO
	CALL RADGALIT(FIRSTIME,PARAMETRS,PARAMETRS_INIT,NUMPARAM,
	1SIGMAADU,SIGMA_INIT,TOLERANCE,FRACINCR,ITERATION_SCHEME,
	2CHOICE,ITERATE)
	IF (NTIMES.GE.50) ITERATE=.FALSE.
	FIRSTIME=.FALSE.
	PARAMETRSCHANGED=.FALSE.
	DO I=1,NUMPARAM
	  IF (PARAMETRS_PRIOR(I).NE.PARAMETRS(I)) PARAMETRSCHANGED
	1 =.TRUE.
	END DO
	IF (ITERATE) GO TO 70
C  Compute the expected mean error for a perfect fit
	IF (WEIGHT_SCHEME.EQ.'R'.OR.WEIGHT_SCHEME.EQ.'N'.OR.
	1WEIGHT_SCHEME.EQ.'P') THEN
	  EXPCT_SIGMA=SQRT(NUMRADII/SUMWEIGHT)
	  EXPCT_SIGMASIGMA=EXPCT_SIGMA/SQRT(2.0*(NUMRADII-NUMPARAM))
	  CALL MESSAGE('For a perfect fit:')
	  WRITE (6,1300) EXPCT_SIGMA,EXPCT_SIGMASIGMA
	  WRITE (7,1300) EXPCT_SIGMA,EXPCT_SIGMASIGMA
	END IF
C  Calculate the covariance 
	CALL MATRIXINV(SUM,6,NUMPARAM,INVERSE,KPVT)
	K=0
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	    COVARIANCE(K)=SIGMAADU**2*AVWEIGHT*INVERSE(I,J)
	  END DO
	END DO
	K=0
	CALL MESSAGE(' ')
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	      WRITE (6,1400) I,J,COVARIANCE(K)
	      WRITE (7,1400) I,J,COVARIANCE(K)
1400	      FORMAT(' Covariance(',I1,',',I1,') = ',G20.8)
	  END DO
	END DO
C  Print the average weight
	WRITE (6,1410) AVWEIGHT
	WRITE (7,1410) AVWEIGHT
1410	FORMAT(' The average weight = ',F15.6)
C  Print the reduced chi squared
	IF (WEIGHT_SCHEME.NE.'S') THEN
	  WRITE (6,1415) REDCHISQR
	  WRITE (7,1415) REDCHISQR
1415	  FORMAT(' The reduced chi squared = ',F10.4)
	END IF
C
C  Compute the integral of the surface brightness distribution
C  using the actual data out to the ending input radius and the 
C  function just solved for from there to infinity (actually to
C  where the answer doesn't change by more than 1 part in 10(10)
C  for a one pixel increase in radius).  The integration in the center
C  of the object will be accurate if the data file was prepared from
C  a two-dimensional array by options EA or PR, but not so accurate if
C  the data file was created from a formula as from option FG.
C
C  Reset SKY for the correct integration for those choices where it was
C  not being solved for and where it was set equal to zero because 
C  USERPARAMETRS was taking over its task, ie., choices ML and MR.
	IF (CHOICE.EQ.'ML'.OR.CHOICE.EQ.'MR') SKY=USERPARAMETRS(3)
C
	IF (CHOICE.NE.'E3') THEN
	  ADUINTEGRAL1=0.0  ! The integration must be zero at the start
	  FORESHORT=SQRT(1.-ECCENTRICITY(2)**2)
	  ADUINTEGRAL1=(SURFBRIGHTOB(1)-SKY)/PI+(SURFBRIGHTOB(2)-
	1 SKY)*2*FORESHORT
	  DO 520 I=3,IRADII(2)
	    FORESHORT=SQRT(1.-ECCENTRICITY(I)**2)
	    ADUINTEGRAL1=ADUINTEGRAL1+(SURFBRIGHTOB(I)-SKY)*2*(I-1)*
	1   FORESHORT
 520	  CONTINUE
	  ADUINTEGRAL1=ADUINTEGRAL1*PI
	  ADUMAGINTEGRAL=30.0-2.5*SNGL(DLOG10(ADUINTEGRAL1))
1420	  FORMAT(' The integral from radius = ',F6.1,' to radius = ',
	1 F6.1,' = ',F20.6)
	  N=IRADII(2)+1  ! Next available array location to be used
		         ! in loop for integration
	  ISTART=IRADII(1)  ! To be used in plotting residuals
	  NPOINTS=NUMRADII
	  CALL MESSAGE(' ')
	  WRITE(6,1420) RADII(1),RADII(2),ADUINTEGRAL1
	  WRITE(7,1420) RADII(1),RADII(2),ADUINTEGRAL1
1430	  FORMAT(' In magnitudes it is ',F11.6)
	  WRITE(6,1430) ADUMAGINTEGRAL
	  WRITE(7,1430) ADUMAGINTEGRAL
	  RADII(1)=RADII(2)+1
	  RADIUS(N)=RADII(1)  ! The radius at which the integration
			      ! begins
	  STARTRADIUS=RADIUS(N)
	  TEST=1.0  ! Initialization
	  ADUINTEGRAL2=0.0
	  IF (CHOICE.NE.'H'.AND.CHOICE.NE.'ML'.AND.CHOICE.NE.'MR'.AND.CHOICE
	1 .NE.'I'.AND.CHOICE.NE.'IX'.AND.CHOICE
	2 .NE.'K'.AND.CHOICE.NE.'L'.AND.CHOICE.NE.'SI') THEN
C  Now integrate using the formula.  Note that N is constant here
	    EVALUATE=.TRUE.
	    GO=.TRUE.
C  Integrate only the galaxy for the following choices
	    IF (CHOICE(1:2).EQ.'MC') CHOICE='M'
	    IF (CHOICE(1:2).EQ.'VC') CHOICE='V'
	    DO WHILE (GO)
	      CALL RADGALFUN(RADIUS,N,PARAMETRS,CHOICE,
	1     EVALUATE,SURFBCONVOLVE,TERM,USERPARAMETRS)
	      DELTARADIUS=1.0+0.01*(RADIUS(N)/STARTRADIUS)
	      TEST=TWOPI*TERM*(RADIUS(N)-1.0+(DELTARADIUS+1.0)/2.0)*
	1     DELTARADIUS*FORESHORT
	      ADUINTEGRAL2=ADUINTEGRAL2+TEST
	      RADIUS(N)=RADIUS(N)+DELTARADIUS
	      IF (TEST/(ADUINTEGRAL1+ADUINTEGRAL2).LT.1E-10) GO=.FALSE.
	    END DO
	    RADII(2)=RADIUS(N)
1440	    FORMAT(' The integral from radius =',F6.1,' to the limit of',
	1   F12.1,' =',F17.6)
	    WRITE(6,1440) RADII(1),RADII(2),ADUINTEGRAL2
	    WRITE(7,1440) RADII(1),RADII(2),ADUINTEGRAL2
	    IF (ADUINTEGRAL2.GT.0.0) THEN
	      ADUMAGINTEGRAL=30.0-2.5*SNGL(DLOG10(ADUINTEGRAL2))
	    ELSE
	      ADUINTEGRAL2=0.0
	      ADUMAGINTEGRAL=100.0
	    END IF
	    WRITE(6,1430) ADUMAGINTEGRAL
	    WRITE(7,1430) ADUMAGINTEGRAL
	    ADUINTEGRAL1=ADUINTEGRAL1+ADUINTEGRAL2
	    ADUMAGINTEGRAL=30.0-2.5*SNGL(DLOG10(ADUINTEGRAL1))
1460	    FORMAT(' The total is ',F20.6,' and in magnitudes it is ',
	1   F11.6)
	    WRITE(6,1460) ADUINTEGRAL1,ADUMAGINTEGRAL
	    WRITE(7,1460) ADUINTEGRAL1,ADUMAGINTEGRAL
	    CALL MESSAGE(' ')
	  END IF
	END IF
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',1,1,'PLOT=',
	1'Do you wish to plot the residuals?  (default = Y)')
	IF (PLOTD) THEN
	  XLABEL='RADIUS (pixels)'
	  YLABEL='OSBSERVED-CALCULATED (ADU)'
	  TITLE='RESIDUALS'
C  Reset the array
	  DO I=SIZE-ISTART+1-3,SIZE-ISTART+1
	    RADIUS(I)=0
	  END DO
	  CALL XYPLOT(RADIUS(ISTART),OBMINUSCALC(ISTART,1),SIZE-ISTART
	1 +1,NPLOTS,NPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2 INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  IF (CHOICE.NE.'E3') THEN
C  Reset the array
	    DO I=SIZE-ISTART+1-3,SIZE-ISTART+1
	      RADIUS(I)=0
	    END DO
	    YLABEL='OBSERVED-CALCULATED (MAGNITUDES)'
	    CALL XYPLOT(RADIUS(ISTART),OBMINUSCALC(ISTART,2),SIZE-ISTART
	1   +1,NPLOTS,NPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,
	2   INTERACT,CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  END IF
	  CALL RESETTEK
	END IF
	STORERESULTS=.FALSE.
	IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'V') THEN
	  CALL USRINP(STORERESULTS,1,'L',1,1,'STORE=','Do you wish to store
	1 the results?  (default = N)')
	END IF
	IF (STORERESULTS.AND..NOT.STOREOPENED) THEN
	  FILENAME=FILENAMERESULTS
	  CALL OPENCF(24,0,'FILE_24=',0)
	  FILENAMERESULTS=FILENAME
	  STOREOPENED=.TRUE.
C  If this is a new file, write the header
	  IF (.NOT.OFIL) THEN
	    WRITE(24,2200)
2200	    FORMAT(' ','ID',4X,'SURFACE BRIGHTNESS',1X,'RADIUS PARAMETER',
	1   1X,'TOTAL MAGNITUDE',1X,'MEAN ERROR',1X,'MEAN ERROR SB',1X,
	2   'MEAN ERROR RP',3X,'SKY',3X,'M.E. SKY',1X,'COVARIANCE')
	  END IF
 	END IF
	IF (STORERESULTS) THEN
	  IF (.NOT.SOLVESKY) THEN
	    PARAMETRS(3)=SKY
	    SIGMAPARAM(3)=0.0
	  END IF
	  WRITE(24,2201) ID,(PARAMETRS(I),I=1,2),ADUMAGINTEGRAL,SIGMAADU,
	1 (SIGMAPARAM(I),I=1,2),PARAMETRS(3),SIGMAPARAM(3),COVARIANCE(2)
2201	  FORMAT(' ',A6,F15.3,5X,F11.5,8X,F7.3,6X,F7.3,4X,F9.3,5X,F9.4,3X,
	1 F9.3,2X,F6.3,2X,F8.4)
	END IF
	MORE=.FALSE.
	CLOSE (UNIT=22)  ! Closes the smearing file
	CLOSE (UNIT=23)
	CALL USRINP(MORE,1,'L',1,1,'MORE=',
	1'More objects to analyze in this file?   (default = N)')
	IF (MORE) GO TO 2
	CLOSE(UNIT=21) !  Closes the data file
	IF (STORERESULTS) CLOSE(UNIT=24)
	RETURN
	END
