      SUBROUTINE ROTATE
C
C       This subroutine will rotate a FITS file by a given number of
C  degrees.  If more than one map exists in the file, the user can
C  select which maps to rotate.  If a map has more than one plane in 
C  it, all the planes will be rotated.
C			Donald H. Gudehus, February, 2008
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	CHARACTER*1 CHOICE
	CHARACTER*3 FILEFMT
	CHARACTER*3 PRGFMAT_IN(MAPMAX)
	CHARACTER*50 CHFITSMHV
	CHARACTER*80 FILENAME_IN,FILENAME_OUT
C
	INTEGER ICOMMAND,MAPS_IN(MAPMAX),STATUS
	INTEGER NCOLS_TEMP(MAPMAX),NROWS_TEMP(MAPMAX),NPLNS_IN(MAPMAX)
	INTEGER ADDRESS
C
	REAL*4 DATA(MAXSIZE),DATA_ROTATE(MAXSIZE)
	REAL*8 DATADP(MAXSIZE),DATADP_ROTATE(MAXSIZE)
C
	CHARACTER*80 DNAME
	COMMON /DFILE/ DNAME
C
	CALL MESSAGE('ROTATE, Version 1')
C
C  Open the input file and select the maps
C
	LU_IN=11
	FILENAME_IN=' '
	CALL GET_INPUT_FITS_MAPS(LU_IN,FILENAME_IN,MAPS_IN,NMAPS_IN,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  RETURN
	END IF
C
	ICOMMAND=0
	DO WHILE (ICOMMAND.NE.1.AND.ICOMMAND.NE.2.AND.ICOMMAND.NE.3
	1.AND.ICOMMAND.NE.4)
	  ICOMMAND=2
	  CALL MESSAGE('Enter a command:')
	  CALL MESSAGE('     1 = Rotates by 180 degrees')
	  CALL MESSAGE('     2 = Rotates by 90 degrees clockwise (default)')
	  CALL MESSAGE('     3 = Rotates by 90 degrees counterclockwise')
	  CALL USRINP(ICOMMAND,1,'I',4,1,'COMMAND=',
	1 '     4 = Rotates by an arbitrary amount')
	END DO
C
	IF (ICOMMAND.EQ.4) THEN
	  CALL MESSAGE('Enter the rotation angle in degrees:')
	  CALL USRINP(THETA,1,'F',4,0,'ANGLE=',
	1 '(a positive angle rotates the features counterclockwise)')
	  THETA=MOD(THETA,360.0)
	  C=ABS(COSD(THETA))
	  S=ABS(SIND(THETA))
	  CHOICE=' '
	  DO WHILE (CHOICE.NE.'K'.AND.CHOICE.NE.'N')
	    CHOICE='N'
	    CALL MESSAGE('Enter a choice for the ouput map size:')
	    CALL MESSAGE('     K = Keeps the original map size')
	    CALL USRINP(%REF(CHOICE),1,'A',1,1,'CHOICE=',
	1   '     N = Avoids any cropping of the data (default)')
	  END DO
	END IF
C
C  Calculate what size output maps are required for the various commands
C
	DO N=1,NMAPS_IN
	  NPLNS_IN(N)=NAXIS3(MAPS_IN(N),LU_IN)
	  PRGFMAT_IN(N)=CHFITSMHV(LU_IN,MAPS_IN(N),'BITPIX','R4')
	  IF (ICOMMAND.EQ.1) THEN
	    NCOLS_TEMP(N)=NAXIS1(MAPS_IN(N),LU_IN)
	    NROWS_TEMP(N)=NAXIS2(MAPS_IN(N),LU_IN)
	  ELSE IF (ICOMMAND.EQ.2.OR.ICOMMAND.EQ.3) THEN
	    NCOLS_TEMP(N)=NAXIS2(MAPS_IN(N),LU_IN)
	    NROWS_TEMP(N)=NAXIS1(MAPS_IN(N),LU_IN)
	  ELSE IF (ICOMMAND.EQ.4) THEN
	    IF (CHOICE.EQ.'K') THEN
	      NCOLS_TEMP(N)=NAXIS1(MAPS_IN(N),LU_IN)
	      NROWS_TEMP(N)=NAXIS2(MAPS_IN(N),LU_IN)
	    ELSE IF (CHOICE.EQ.'N') THEN
	      NI=NAXIS1(MAPS_IN(N),LU_IN)
	      NJ=NAXIS2(MAPS_IN(N),LU_IN)
C  TEMPX and TEMPY are the projected x and y half-lengths of the rotated image.
C  We make these be integral values by rounding up, if necessary, so that 
C  we do not crop the image.  2X a half-length plus 1 will be odd, allowing
C  the center of rotation to be at the center of a pixel.
	      TEMPX=0.5*(C*NI+S*NJ)
	      IF (TEMPX-INT(TEMPX).NE.0.0) TEMPX=TEMPX+1.0 !  For rounding up
	      TEMPY=0.5*(S*NI+C*NJ)
	      IF (TEMPY-INT(TEMPY).NE.0.0) TEMPY=TEMPY+1.0
	      NCOLS_TEMP(N)=2*INT(TEMPX)+1 !  Makes it odd
	      NROWS_TEMP(N)=2*INT(TEMPY)+1
	    END IF
	  END IF
	END DO
C
C  Create a temporary file.  This allows for several options when we save our
C  results.
C
	LU_TEMP=12
	CALL OPEN_TEMP_FITS(LU_IN,LU_TEMP,FILENAME_IN,MAPS_IN,NMAPS_IN,
	1NCOLS_TEMP,NROWS_TEMP,STATUS)
C
C  Carry out the rotation
C
	IF (ICOMMAND.EQ.1) THEN
	  CALL MESSAGE(' ')
	  CALL MESSAGE('Rotating 180 Degrees')
	  CALL MESSAGE('Input Map No.')
	  DO N=1,NMAPS_IN
	    IMNUM_IN=MAPS_IN(N)
	    WRITE(6,1000) IMNUM_IN
	    WRITE(7,1000) IMNUM_IN
1000	    FORMAT(' ',6X,I4)
	    NI=NAXIS1(MAPS_IN(N),LU_IN)
	    NJ=NAXIS2(MAPS_IN(N),LU_IN)
	    NI2=NCOLS_TEMP(N)
	    NJ2=NROWS_TEMP(N)
	    DO IPNUM_IN=1,NPLNS_IN(N)
	      FILEFMT=CHFITSMHV(LU_IN,IMNUM_IN,'BITPIX','R4')
	      IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1     FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
C  Loop on the output rows
	        DO IR=1,NJ2
	          IROW_READ=NJ-IR+1
	          CALL RDFITSROW(LU_IN,IMNUM_IN,IPNUM_IN,DATA,MAXSIZE,
	1         IROW_READ)
C  Load the output row-array
	          DO I=1,NI2
	            II=NI-I+1
	            DATA_ROTATE(I)=DATA(II)
	          END DO
	          CALL WRFITSROW(LU_TEMP,IMNUM_IN,IPNUM_IN,DATA_ROTATE,
	1         MAXSIZE,IR)
	        END DO
	      ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	        DO IR=1,NJ2
	          IROW_READ=NJ-IR+1
	          CALL RDDPFITSROW(LU_IN,IMNUM_IN,IPNUM,DATADP,MAXSIZE,
	1         IROW_READ)
	          DO I=1,NI2
	            II=NI-I+1
	            DATADP_ROTATE(I)=DATADP(II)
	          END DO
	          CALL WRDPFITSROW(LU_TEMP,IMNUM_TEMP,IPNUM,DATADP_ROTATE,
	1         MAXSIZE,IR)
	        END DO
	      END IF
	    END DO
	  END DO
	ELSE IF (ICOMMAND.EQ.2.OR.ICOMMAND.EQ.3.OR.ICOMMAND.EQ.4) THEN
	  CALL MESSAGE(' ')
	  IF (ICOMMAND.EQ.2) CALL MESSAGE('Rotating 90 Degrees Clockwise')
	  IF (ICOMMAND.EQ.3) CALL MESSAGE('Rotating 90 Degrees '//
	1 'Counterclockwise')
	  IF (ICOMMAND.EQ.4) THEN
	    WRITE(6,2000) THETA
	    WRITE(7,2000) THETA
2000	    FORMAT(' ','Rotating by ',G12.5,' Degrees')
	  END IF
	  CALL MESSAGE('Input Map No.')
	  DO N=1,NMAPS_IN
	    IMNUM_IN=MAPS_IN(N)
	    WRITE(6,1000) IMNUM_IN
	    WRITE(7,1000) IMNUM_IN
	    NI=NAXIS1(MAPS_IN(N),LU_IN)
	    NJ=NAXIS2(MAPS_IN(N),LU_IN)
	    NI2=NCOLS_TEMP(N)
	    NJ2=NROWS_TEMP(N)
	    DO IPNUM_IN=1,NPLNS_IN(N)
	      FILEFMT=CHFITSMHV(LU_IN,IMNUM_IN,'BITPIX','R4')
	      IF (FILEFMT.EQ.'I1'.OR.FILEFMT.EQ.'I2'.OR.FILEFMT.EQ.'I4'.OR.
	1     FILEFMT.EQ.'R4'.OR.FILEFMT.EQ.'C8') THEN
	        CALL DYNAMIC(ADDRESS,NI*NJ,'  R4',.TRUE.)
	        CALL ROTATE1SP(%VAL(ADDRESS),NI,NJ,NI2,NJ2,THETA,CHOICE,
	1       DATA_ROTATE,DATADP_ROTATE,MAXSIZE,ICOMMAND,
	2       LU_IN,LU_TEMP,IMNUM_IN,IPNUM_IN)
	        CALL DYNAMIC(ADDRESS,NI*NJ,'  R4',.FALSE.)
	      ELSE IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	        CALL DYNAMIC(ADDRESS,NI*NJ,'  R8',.TRUE.)
	        CALL ROTATE1DP(%VAL(ADDRESS),NI,NJ,NI2,NJ2,THETA,CHOICE,
	1       DATA_ROTATE,DATADP_ROTATE,MAXSIZE,ICOMMAND,
	2       LU_IN,LU_TEMP,IMNUM_IN,IPNUM_IN)
	        CALL DYNAMIC(ADDRESS,NI*NJ,'  R8',.FALSE.)
	      END IF
	    END DO
	  END DO
	END IF
	LU_OUT=13
	CALL SAVE_SAVE_AS(LU_IN,LU_TEMP,LU_OUT,MAPS_IN,NMAPS_IN,STATUS)
	RETURN
	END
